/* Generated by Cython 0.27 */

#define PY_SSIZE_T_CLEAN
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.6+ or Python 3.3+.
#else
#define CYTHON_ABI "0_27"
#define CYTHON_FUTURE_DIVISION 0
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x02070000
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 0
  #elif !defined(CYTHON_USE_PYTYPE_LOOKUP)
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT (PY_VERSION_HEX >= 0x03050000)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1)
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #include "longintrepr.h"
  #undef SHIFT
  #undef BASE
  #undef MASK
#endif
#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#if PY_VERSION_HEX < 0x030700A0 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject **args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject **args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if !CYTHON_FAST_THREAD_STATE || PY_VERSION_HEX < 0x02070000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x03050200
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_Get()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? PyMethod_New(func, self) : PyInstanceMethod_New(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned char     uint8_t;
           typedef unsigned int      uint32_t;
        #else
           typedef unsigned __int8   uint8_t;
           typedef unsigned __int32  uint32_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#ifndef CYTHON_FALLTHROUGH
  #ifdef __cplusplus
    #if __has_cpp_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH [[fallthrough]]
    #elif __has_cpp_attribute(clang::fallthrough)
      #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough) || (defined(__GNUC__) && defined(__attribute__))
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif


#define __PYX_ERR(f_index, lineno, Ln_error) \
{ \
  __pyx_filename = __pyx_f[f_index]; __pyx_lineno = lineno; __pyx_clineno = __LINE__; goto Ln_error; \
}

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__pyteomics__cparser
#define __PYX_HAVE_API__pyteomics__cparser
#include <string.h>
#include <stdio.h>
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT 0
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u) {
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
#define __Pyx_PyBool_FromLong(b) ((b) ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False))
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c));
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

static PyObject *__pyx_m;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_cython_runtime;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;


static const char *__pyx_f[] = {
  "pyteomics\\cparser.pyx",
  "type.pxd",
};

/*--- Type declarations ---*/
struct __pyx_opt_args_9pyteomics_7cparser_parse;
struct __pyx_opt_args_9pyteomics_7cparser_amino_acid_composition;
struct __pyx_opt_args_9pyteomics_7cparser_tostring;
struct __pyx_opt_args_9pyteomics_7cparser_length;

/* "pyteomics/cparser.pxd":16
 * 
 * 
 * cpdef list parse(str sequence, bint show_unmodified_termini=*, bint split=*,             # <<<<<<<<<<<<<<
 *                  bint allow_unknown_modifications=*, object  labels=*)
 * 
 */
struct __pyx_opt_args_9pyteomics_7cparser_parse {
  int __pyx_n;
  int show_unmodified_termini;
  int split;
  int allow_unknown_modifications;
  PyObject *labels;
};

/* "pyteomics/cparser.pxd":19
 *                  bint allow_unknown_modifications=*, object  labels=*)
 * 
 * cpdef dict amino_acid_composition(object sequence, bint show_unmodified_termini=*, bint term_aa=*,             # <<<<<<<<<<<<<<
 *                                   bint allow_unknown_modifications=*, object labels=*)
 * 
 */
struct __pyx_opt_args_9pyteomics_7cparser_amino_acid_composition {
  int __pyx_n;
  int show_unmodified_termini;
  int term_aa;
  int allow_unknown_modifications;
  PyObject *labels;
};

/* "pyteomics/cparser.pxd":24
 * cpdef tuple _split_label(str label)
 * 
 * cdef str tostring(object parsed_sequence, bint show_unmodified_termini=*)             # <<<<<<<<<<<<<<
 */
struct __pyx_opt_args_9pyteomics_7cparser_tostring {
  int __pyx_n;
  int show_unmodified_termini;
};

/* "pyteomics/cparser.pyx":106
 *     return bool(match_modX(label))
 * 
 * cpdef int length(object sequence,             # <<<<<<<<<<<<<<
 *                  int show_unmodified_termini=0, int split=0,
 *                  int allow_unknown_modifications=0, object labels=std_labels):
 */
struct __pyx_opt_args_9pyteomics_7cparser_length {
  int __pyx_n;
  int show_unmodified_termini;
  int split;
  int allow_unknown_modifications;
  PyObject *labels;
};

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* Profile.proto */
#ifndef CYTHON_PROFILE
#if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_PYSTON
  #define CYTHON_PROFILE 0
#else
  #define CYTHON_PROFILE 1
#endif
#endif
#ifndef CYTHON_TRACE_NOGIL
  #define CYTHON_TRACE_NOGIL 0
#else
  #if CYTHON_TRACE_NOGIL && !defined(CYTHON_TRACE)
    #define CYTHON_TRACE 1
  #endif
#endif
#ifndef CYTHON_TRACE
  #define CYTHON_TRACE 0
#endif
#if CYTHON_TRACE
  #undef CYTHON_PROFILE_REUSE_FRAME
#endif
#ifndef CYTHON_PROFILE_REUSE_FRAME
  #define CYTHON_PROFILE_REUSE_FRAME 0
#endif
#if CYTHON_PROFILE || CYTHON_TRACE
  #include "compile.h"
  #include "frameobject.h"
  #include "traceback.h"
  #if CYTHON_PROFILE_REUSE_FRAME
    #define CYTHON_FRAME_MODIFIER static
    #define CYTHON_FRAME_DEL(frame)
  #else
    #define CYTHON_FRAME_MODIFIER
    #define CYTHON_FRAME_DEL(frame) Py_CLEAR(frame)
  #endif
  #define __Pyx_TraceDeclarations\
  static PyCodeObject *__pyx_frame_code = NULL;\
  CYTHON_FRAME_MODIFIER PyFrameObject *__pyx_frame = NULL;\
  int __Pyx_use_tracing = 0;
  #define __Pyx_TraceFrameInit(codeobj)\
  if (codeobj) __pyx_frame_code = (PyCodeObject*) codeobj;
  #ifdef WITH_THREAD
  #define __Pyx_TraceCall(funcname, srcfile, firstlineno, nogil, goto_error)\
  if (nogil) {\
      if (CYTHON_TRACE_NOGIL) {\
          PyThreadState *tstate;\
          PyGILState_STATE state = PyGILState_Ensure();\
          tstate = __Pyx_PyThreadState_Current;\
          if (unlikely(tstate->use_tracing) && !tstate->tracing &&\
                  (tstate->c_profilefunc || (CYTHON_TRACE && tstate->c_tracefunc))) {\
              __Pyx_use_tracing = __Pyx_TraceSetupAndCall(&__pyx_frame_code, &__pyx_frame, tstate, funcname, srcfile, firstlineno);\
          }\
          PyGILState_Release(state);\
          if (unlikely(__Pyx_use_tracing < 0)) goto_error;\
      }\
  } else {\
      PyThreadState* tstate = PyThreadState_GET();\
      if (unlikely(tstate->use_tracing) && !tstate->tracing &&\
              (tstate->c_profilefunc || (CYTHON_TRACE && tstate->c_tracefunc))) {\
          __Pyx_use_tracing = __Pyx_TraceSetupAndCall(&__pyx_frame_code, &__pyx_frame, tstate, funcname, srcfile, firstlineno);\
          if (unlikely(__Pyx_use_tracing < 0)) goto_error;\
      }\
  }
  #else
  #define __Pyx_TraceCall(funcname, srcfile, firstlineno, nogil, goto_error)\
  {   PyThreadState* tstate = PyThreadState_GET();\
      if (unlikely(tstate->use_tracing) && !tstate->tracing &&\
              (tstate->c_profilefunc || (CYTHON_TRACE && tstate->c_tracefunc))) {\
          __Pyx_use_tracing = __Pyx_TraceSetupAndCall(&__pyx_frame_code, &__pyx_frame, tstate, funcname, srcfile, firstlineno);\
          if (unlikely(__Pyx_use_tracing < 0)) goto_error;\
      }\
  }
  #endif
  #define __Pyx_TraceException()\
  if (likely(!__Pyx_use_tracing)); else {\
      PyThreadState* tstate = __Pyx_PyThreadState_Current;\
      if (tstate->use_tracing &&\
              (tstate->c_profilefunc || (CYTHON_TRACE && tstate->c_tracefunc))) {\
          tstate->tracing++;\
          tstate->use_tracing = 0;\
          PyObject *exc_info = __Pyx_GetExceptionTuple(tstate);\
          if (exc_info) {\
              if (CYTHON_TRACE && tstate->c_tracefunc)\
                  tstate->c_tracefunc(\
                      tstate->c_traceobj, __pyx_frame, PyTrace_EXCEPTION, exc_info);\
              tstate->c_profilefunc(\
                  tstate->c_profileobj, __pyx_frame, PyTrace_EXCEPTION, exc_info);\
              Py_DECREF(exc_info);\
          }\
          tstate->use_tracing = 1;\
          tstate->tracing--;\
      }\
  }
  static void __Pyx_call_return_trace_func(PyThreadState *tstate, PyFrameObject *frame, PyObject *result) {
      PyObject *type, *value, *traceback;
      PyErr_Fetch(&type, &value, &traceback);
      tstate->tracing++;
      tstate->use_tracing = 0;
      if (CYTHON_TRACE && tstate->c_tracefunc)
          tstate->c_tracefunc(tstate->c_traceobj, frame, PyTrace_RETURN, result);
      if (tstate->c_profilefunc)
          tstate->c_profilefunc(tstate->c_profileobj, frame, PyTrace_RETURN, result);
      CYTHON_FRAME_DEL(frame);
      tstate->use_tracing = 1;
      tstate->tracing--;
      PyErr_Restore(type, value, traceback);
  }
  #ifdef WITH_THREAD
  #define __Pyx_TraceReturn(result, nogil)\
  if (likely(!__Pyx_use_tracing)); else {\
      if (nogil) {\
          if (CYTHON_TRACE_NOGIL) {\
              PyThreadState *tstate;\
              PyGILState_STATE state = PyGILState_Ensure();\
              tstate = __Pyx_PyThreadState_Current;\
              if (tstate->use_tracing) {\
                  __Pyx_call_return_trace_func(tstate, __pyx_frame, (PyObject*)result);\
              }\
              PyGILState_Release(state);\
          }\
      } else {\
          PyThreadState* tstate = __Pyx_PyThreadState_Current;\
          if (tstate->use_tracing) {\
              __Pyx_call_return_trace_func(tstate, __pyx_frame, (PyObject*)result);\
          }\
      }\
  }
  #else
  #define __Pyx_TraceReturn(result, nogil)\
  if (likely(!__Pyx_use_tracing)); else {\
      PyThreadState* tstate = __Pyx_PyThreadState_Current;\
      if (tstate->use_tracing) {\
          __Pyx_call_return_trace_func(tstate, __pyx_frame, (PyObject*)result);\
      }\
  }
  #endif
  static PyCodeObject *__Pyx_createFrameCodeObject(const char *funcname, const char *srcfile, int firstlineno);
  static int __Pyx_TraceSetupAndCall(PyCodeObject** code, PyFrameObject** frame, PyThreadState* tstate, const char *funcname, const char *srcfile, int firstlineno);
#else
  #define __Pyx_TraceDeclarations
  #define __Pyx_TraceFrameInit(codeobj)
  #define __Pyx_TraceCall(funcname, srcfile, firstlineno, nogil, goto_error)   if ((1)); else goto_error;
  #define __Pyx_TraceException()
  #define __Pyx_TraceReturn(result, nogil)
#endif
#if CYTHON_TRACE
  static int __Pyx_call_line_trace_func(PyThreadState *tstate, PyFrameObject *frame, int lineno) {
      int ret;
      PyObject *type, *value, *traceback;
      PyErr_Fetch(&type, &value, &traceback);
      __Pyx_PyFrame_SetLineNumber(frame, lineno);
      tstate->tracing++;
      tstate->use_tracing = 0;
      ret = tstate->c_tracefunc(tstate->c_traceobj, frame, PyTrace_LINE, NULL);
      tstate->use_tracing = 1;
      tstate->tracing--;
      if (likely(!ret)) {
          PyErr_Restore(type, value, traceback);
      } else {
          Py_XDECREF(type);
          Py_XDECREF(value);
          Py_XDECREF(traceback);
      }
      return ret;
  }
  #ifdef WITH_THREAD
  #define __Pyx_TraceLine(lineno, nogil, goto_error)\
  if (likely(!__Pyx_use_tracing)); else {\
      if (nogil) {\
          if (CYTHON_TRACE_NOGIL) {\
              int ret = 0;\
              PyThreadState *tstate;\
              PyGILState_STATE state = PyGILState_Ensure();\
              tstate = __Pyx_PyThreadState_Current;\
              if (unlikely(tstate->use_tracing && tstate->c_tracefunc && __pyx_frame->f_trace)) {\
                  ret = __Pyx_call_line_trace_func(tstate, __pyx_frame, lineno);\
              }\
              PyGILState_Release(state);\
              if (unlikely(ret)) goto_error;\
          }\
      } else {\
          PyThreadState* tstate = __Pyx_PyThreadState_Current;\
          if (unlikely(tstate->use_tracing && tstate->c_tracefunc && __pyx_frame->f_trace)) {\
              int ret = __Pyx_call_line_trace_func(tstate, __pyx_frame, lineno);\
              if (unlikely(ret)) goto_error;\
          }\
      }\
  }
  #else
  #define __Pyx_TraceLine(lineno, nogil, goto_error)\
  if (likely(!__Pyx_use_tracing)); else {\
      PyThreadState* tstate = __Pyx_PyThreadState_Current;\
      if (unlikely(tstate->use_tracing && tstate->c_tracefunc && __pyx_frame->f_trace)) {\
          int ret = __Pyx_call_line_trace_func(tstate, __pyx_frame, lineno);\
          if (unlikely(ret)) goto_error;\
      }\
  }
  #endif
#else
  #define __Pyx_TraceLine(lineno, nogil, goto_error)   if ((1)); else goto_error;
#endif

/* bytes_tailmatch.proto */
static int __Pyx_PyBytes_SingleTailmatch(PyObject* self, PyObject* arg,
                                         Py_ssize_t start, Py_ssize_t end, int direction);
static int __Pyx_PyBytes_Tailmatch(PyObject* self, PyObject* substr,
                                   Py_ssize_t start, Py_ssize_t end, int direction);

/* unicode_tailmatch.proto */
static int __Pyx_PyUnicode_Tailmatch(
    PyObject* s, PyObject* substr, Py_ssize_t start, Py_ssize_t end, int direction);

/* str_tailmatch.proto */
static CYTHON_INLINE int __Pyx_PyStr_Tailmatch(PyObject* self, PyObject* arg, Py_ssize_t start,
                                               Py_ssize_t end, int direction);

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyErr_Occurred()  __pyx_tstate->curexc_type
#if PY_VERSION_HEX >= 0x03050000
  #define __Pyx_PyThreadState_assign  __pyx_tstate = _PyThreadState_UncheckedGet();
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_assign  __pyx_tstate = PyThreadState_Get();
#elif PY_VERSION_HEX >= 0x02070000
  #define __Pyx_PyThreadState_assign  __pyx_tstate = _PyThreadState_Current;
#else
  #define __Pyx_PyThreadState_assign  __pyx_tstate = PyThreadState_Get();
#endif
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* WriteUnraisableException.proto */
static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil);

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* PyIntBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_AddObjC(PyObject *op1, PyObject *op2, long intval, int inplace);
#else
#define __Pyx_PyInt_AddObjC(op1, op2, intval, inplace)\
    (inplace ? PyNumber_InPlaceAdd(op1, op2) : PyNumber_Add(op1, op2))
#endif

/* PyCFunctionFastCall.proto */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject *__Pyx_PyCFunction_FastCall(PyObject *func, PyObject **args, Py_ssize_t nargs);
#else
#define __Pyx_PyCFunction_FastCall(func, args, nargs)  (assert(0), NULL)
#endif

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, int nargs, PyObject *kwargs);
#else
#define __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs) _PyFunction_FastCallDict(func, args, nargs, kwargs)
#endif
#endif

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* PyObjectCallNoArg.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

/* RaiseTooManyValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

/* RaiseNeedMoreValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

/* IterFinish.proto */
static CYTHON_INLINE int __Pyx_IterFinish(void);

/* UnpackItemEndCheck.proto */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected);

/* ListCompAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_ListComp_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len)) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        Py_SIZE(list) = len+1;
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_ListComp_Append(L,x) PyList_Append(L,x)
#endif

/* GetModuleGlobalName.proto */
static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name);

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* ListExtend.proto */
static CYTHON_INLINE int __Pyx_PyList_Extend(PyObject* L, PyObject* v) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject* none = _PyList_Extend((PyListObject*)L, v);
    if (unlikely(!none))
        return -1;
    Py_DECREF(none);
    return 0;
#else
    return PyList_SetSlice(L, PY_SSIZE_T_MAX, PY_SSIZE_T_MAX, v);
#endif
}

/* ArgTypeTest.proto */
#define __Pyx_ArgTypeTest(obj, type, none_allowed, name, exact)\
    ((likely((Py_TYPE(obj) == type) | (none_allowed && (obj == Py_None)))) ? 1 :\
        __Pyx__ArgTypeTest(obj, type, name, exact))
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact);

/* ListAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_PyList_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len) & likely(len > (L->allocated >> 1))) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        Py_SIZE(list) = len+1;
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_PyList_Append(L,x) PyList_Append(L,x)
#endif

/* IncludeStringH.proto */
#include <string.h>

/* BytesEquals.proto */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals);

/* UnicodeEquals.proto */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals);

/* StrEquals.proto */
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyString_Equals __Pyx_PyUnicode_Equals
#else
#define __Pyx_PyString_Equals __Pyx_PyBytes_Equals
#endif

/* PySequenceContains.proto */
static CYTHON_INLINE int __Pyx_PySequence_ContainsTF(PyObject* item, PyObject* seq, int eq) {
    int result = PySequence_Contains(seq, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* StringJoin.proto */
#if PY_MAJOR_VERSION < 3
#define __Pyx_PyString_Join __Pyx_PyBytes_Join
#define __Pyx_PyBaseString_Join(s, v) (PyUnicode_CheckExact(s) ? PyUnicode_Join(s, v) : __Pyx_PyBytes_Join(s, v))
#else
#define __Pyx_PyString_Join PyUnicode_Join
#define __Pyx_PyBaseString_Join PyUnicode_Join
#endif
#if CYTHON_COMPILING_IN_CPYTHON
    #if PY_MAJOR_VERSION < 3
    #define __Pyx_PyBytes_Join _PyString_Join
    #else
    #define __Pyx_PyBytes_Join _PyBytes_Join
    #endif
#else
static CYTHON_INLINE PyObject* __Pyx_PyBytes_Join(PyObject* sep, PyObject* values);
#endif

/* PyObjectCallMethod1.proto */
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg);
static PyObject* __Pyx__PyObject_CallMethod1(PyObject* method, PyObject* arg);

/* pop_index.proto */
static PyObject* __Pyx__PyObject_PopNewIndex(PyObject* L, PyObject* py_ix);
static PyObject* __Pyx__PyObject_PopIndex(PyObject* L, PyObject* py_ix);
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static PyObject* __Pyx__PyList_PopIndex(PyObject* L, PyObject* py_ix, Py_ssize_t ix);
#define __Pyx_PyObject_PopIndex(L, py_ix, ix, is_signed, type, to_py_func) (\
    (likely(PyList_CheckExact(L) && __Pyx_fits_Py_ssize_t(ix, type, is_signed))) ?\
        __Pyx__PyList_PopIndex(L, py_ix, ix) : (\
        (unlikely(py_ix == Py_None)) ? __Pyx__PyObject_PopNewIndex(L, to_py_func(ix)) :\
            __Pyx__PyObject_PopIndex(L, py_ix)))
#define __Pyx_PyList_PopIndex(L, py_ix, ix, is_signed, type, to_py_func) (\
    __Pyx_fits_Py_ssize_t(ix, type, is_signed) ?\
        __Pyx__PyList_PopIndex(L, py_ix, ix) : (\
        (unlikely(py_ix == Py_None)) ? __Pyx__PyObject_PopNewIndex(L, to_py_func(ix)) :\
            __Pyx__PyObject_PopIndex(L, py_ix)))
#else
#define __Pyx_PyList_PopIndex(L, py_ix, ix, is_signed, type, to_py_func)\
    __Pyx_PyObject_PopIndex(L, py_ix, ix, is_signed, type, to_py_func)
#define __Pyx_PyObject_PopIndex(L, py_ix, ix, is_signed, type, to_py_func) (\
    (unlikely(py_ix == Py_None)) ? __Pyx__PyObject_PopNewIndex(L, to_py_func(ix)) :\
        __Pyx__PyObject_PopIndex(L, py_ix))
#endif

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* CLineInTraceback.proto */
static int __Pyx_CLineForTraceback(int c_line);

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* FunctionExport.proto */
static int __Pyx_ExportFunction(const char *name, void (*f)(void), const char *sig);

/* PyIdentifierFromString.proto */
#if !defined(__Pyx_PyIdentifier_FromString)
#if PY_MAJOR_VERSION < 3
  #define __Pyx_PyIdentifier_FromString(s) PyString_FromString(s)
#else
  #define __Pyx_PyIdentifier_FromString(s) PyUnicode_FromString(s)
#endif
#endif

/* ModuleImport.proto */
static PyObject *__Pyx_ImportModule(const char *name);

/* TypeImport.proto */
static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name, size_t size, int strict);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);


/* Module declarations from 'cython' */

/* Module declarations from 'libc.string' */

/* Module declarations from 'libc.stdio' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.type' */
static PyTypeObject *__pyx_ptype_7cpython_4type_type = 0;

/* Module declarations from 'cpython' */

/* Module declarations from 'cpython.object' */

/* Module declarations from 'cpython.ref' */

/* Module declarations from 'cpython.dict' */

/* Module declarations from 'cpython.int' */

/* Module declarations from 'cpython.float' */

/* Module declarations from 'cpython.list' */

/* Module declarations from 'cpython.tuple' */

/* Module declarations from 'cpython.sequence' */

/* Module declarations from 'cpython.exc' */

/* Module declarations from 'pyteomics.cparser' */
static PyObject *__pyx_v_9pyteomics_7cparser_std_amino_acids = 0;
static PyObject *__pyx_v_9pyteomics_7cparser_std_labels = 0;
static PyObject *__pyx_v_9pyteomics_7cparser_std_nterm = 0;
static PyObject *__pyx_v_9pyteomics_7cparser_std_cterm = 0;
static PyObject *__pyx_v_9pyteomics_7cparser__modX_sequence = 0;
static PyObject *__pyx_v_9pyteomics_7cparser__modX_group = 0;
static PyObject *__pyx_v_9pyteomics_7cparser__modX_split = 0;
static PyObject *__pyx_v_9pyteomics_7cparser__modX_split_findall = 0;
static PyObject *__pyx_f_9pyteomics_7cparser_parse(PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_9pyteomics_7cparser_parse *__pyx_optional_args); /*proto*/
static PyObject *__pyx_f_9pyteomics_7cparser_amino_acid_composition(PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_9pyteomics_7cparser_amino_acid_composition *__pyx_optional_args); /*proto*/
static CYTHON_INLINE PyObject *__pyx_f_9pyteomics_7cparser__split_label(PyObject *, int __pyx_skip_dispatch); /*proto*/
static PyObject *__pyx_f_9pyteomics_7cparser_tostring(PyObject *, struct __pyx_opt_args_9pyteomics_7cparser_tostring *__pyx_optional_args); /*proto*/
static int __pyx_f_9pyteomics_7cparser_is_term_mod(PyObject *); /*proto*/
static CYTHON_INLINE PyObject *__pyx_f_9pyteomics_7cparser_match_modX(PyObject *); /*proto*/
static int __pyx_f_9pyteomics_7cparser_length(PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_9pyteomics_7cparser_length *__pyx_optional_args); /*proto*/
static CYTHON_INLINE PyObject *__pyx_f_9pyteomics_7cparser_interpolate_labels(PyObject *); /*proto*/
static PyObject *__pyx_f_9pyteomics_7cparser__parse_modx_sequence_split(PyObject *); /*proto*/
#define __Pyx_MODULE_NAME "pyteomics.cparser"
int __pyx_module_is_main_pyteomics__cparser = 0;

/* Implementation of 'pyteomics.cparser' */
static PyObject *__pyx_builtin_sum;
static PyObject *__pyx_builtin_range;
static const char __pyx_k_[] = "-";
static const char __pyx_k_A[] = "A";
static const char __pyx_k_C[] = "C";
static const char __pyx_k_D[] = "D";
static const char __pyx_k_E[] = "E";
static const char __pyx_k_F[] = "F";
static const char __pyx_k_G[] = "G";
static const char __pyx_k_H[] = "H";
static const char __pyx_k_I[] = "I";
static const char __pyx_k_K[] = "K";
static const char __pyx_k_L[] = "L";
static const char __pyx_k_M[] = "M";
static const char __pyx_k_N[] = "N";
static const char __pyx_k_P[] = "P";
static const char __pyx_k_Q[] = "Q";
static const char __pyx_k_R[] = "R";
static const char __pyx_k_S[] = "S";
static const char __pyx_k_T[] = "T";
static const char __pyx_k_V[] = "V";
static const char __pyx_k_W[] = "W";
static const char __pyx_k_Y[] = "Y";
static const char __pyx_k_OH[] = "-OH";
static const char __pyx_k__5[] = "";
static const char __pyx_k_it[] = "it";
static const char __pyx_k_re[] = "re";
static const char __pyx_k_H_2[] = "H-";
static const char __pyx_k_pop[] = "pop";
static const char __pyx_k_sum[] = "sum";
static const char __pyx_k_join[] = "join";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_cterm[] = "cterm";
static const char __pyx_k_deque[] = "deque";
static const char __pyx_k_items[] = "items";
static const char __pyx_k_match[] = "match";
static const char __pyx_k_nterm[] = "nterm";
static const char __pyx_k_range[] = "range";
static const char __pyx_k_split[] = "split";
static const char __pyx_k_format[] = "format";
static const char __pyx_k_groups[] = "groups";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_labels[] = "labels";
static const char __pyx_k_remove[] = "remove";
static const char __pyx_k_a_z_A_Z[] = "^([^-]+-)?((?:[a-z]*[A-Z])+)(-[^-]+)?$";
static const char __pyx_k_compile[] = "compile";
static const char __pyx_k_findall[] = "findall";
static const char __pyx_k_memoize[] = "memoize";
static const char __pyx_k_term_aa[] = "term_aa";
static const char __pyx_k_sequence[] = "sequence";
static const char __pyx_k_a_z_A_Z_2[] = "[a-z]*[A-Z]";
static const char __pyx_k_a_z_A_Z_3[] = "([a-z]*)([A-Z])";
static const char __pyx_k_itertools[] = "itertools";
static const char __pyx_k_collections[] = "collections";
static const char __pyx_k_Unknown_label[] = "Unknown label: {}";
static const char __pyx_k_PyteomicsError[] = "PyteomicsError";
static const char __pyx_k_parse_line_199[] = "parse (line 199)";
static const char __pyx_k_length_line_106[] = "length (line 106)";
static const char __pyx_k_BasicComposition[] = "BasicComposition";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_pyteomics_auxiliary[] = "pyteomics.auxiliary";
static const char __pyx_k_show_unmodified_termini[] = "show_unmodified_termini";
static const char __pyx_k_Not_a_valid_modX_sequence[] = "Not a valid modX sequence: ";
static const char __pyx_k_allow_unknown_modifications[] = "allow_unknown_modifications";
static const char __pyx_k_Unsupported_type_of_sequence[] = "Unsupported type of sequence.";
static const char __pyx_k_Cannot_split_a_non_modX_label_s[] = "Cannot split a non-modX label: %s";
static const char __pyx_k_Parse_a_sequence_string_written[] = "Parse a sequence string written in modX notation into a list of\n    labels or (if `split` argument is :py:const:`True`) into a list of\n    tuples representing amino acid residues and their modifications.\n\n    Parameters\n    ----------\n    sequence : str\n        The sequence of a polypeptide.\n    show_unmodified_termini : bool, optional\n        If :py:const:`True` then the unmodified N- and C-termini are explicitly\n        shown in the returned list. Default value is :py:const:`False`.\n    split : bool, optional\n        If :py:const:`True` then the result will be a list of tuples with 1 to 4\n        elements: terminal modification, modification, residue. Default value is\n        :py:const:`False`.\n    allow_unknown_modifications : bool, optional\n        If :py:const:`True` then do not raise an exception when an unknown\n        modification of a known amino acid residue is found in the sequence.\n        This also includes terminal groups.\n        Default value is :py:const:`False`.\n\n        .. note::\n            Since version 2.5, this parameter has effect only if `labels`\n            are provided.\n    labels : container, optional\n        A container of allowed labels for amino acids,\n        modifications and terminal modifications.\n        If not provided, no checks will be done.\n        Separate labels for modifications (such as 'p' or 'ox')\n        can be supplied, which means they are applicable to all residues.\n\n        .. warning::\n            If `show_unmodified_termini` is set to :py:const:`True`, standard\n            terminal groups need to be present in `labels`.\n\n        .. warning::\n            Avoid using sequences with only one terminal group, as they are\n            ambiguous. If you provide one, `labels` (or :py:const:`std_labels`)\n            will be used to resolve the ambiguity.\n\n    Returns\n    -------\n    out : list\n        List of tuples with labels of modifications and amino acid residues.\n\n    Examp""les\n    --------\n    >>> parse('PEPTIDE', split=True)\n    [('P',), ('E',), ('P',), ('T',), ('I',), ('D',), ('E',)]\n    >>> parse('H-PEPTIDE')\n    ['P', 'E', 'P', 'T', 'I', 'D', 'E']\n    >>> parse('PEPTIDE', show_unmodified_termini=True)\n    ['H-', 'P', 'E', 'P', 'T', 'I', 'D', 'E', '-OH']\n    >>> parse('TEpSToxM', labels=std_labels + ['pS', 'oxM'])\n    ['T', 'E', 'pS', 'T', 'oxM']\n    >>> parse('zPEPzTIDzE', True, True, labels=std_labels+['z'])\n    [('H-', 'z', 'P'), ('E',), ('P',), ('z', 'T'), ('I',), ('D',), ('z', 'E', '-OH')]\n    ";
static const char __pyx_k_amino_acid_composition_line_403[] = "amino_acid_composition (line 403)";
static const char __pyx_k_Calculate_amino_acid_composition[] = "Calculate amino acid composition of a polypeptide.\n\n    Parameters\n    ----------\n    sequence : str or list\n        The sequence of a polypeptide or a list with a parsed sequence.\n    show_unmodified_termini : bool, optional\n        If :py:const:`True` then the unmodified N- and C-terminus are explicitly\n        shown in the returned dict. Default value is :py:const:`False`.\n    term_aa : bool, optional\n        If :py:const:`True` then the terminal amino acid residues are\n        artificially modified with `nterm` or `cterm` modification.\n        Default value is :py:const:`False`.\n    allow_unknown_modifications : bool, optional\n        If :py:const:`True` then do not raise an exception when an unknown\n        modification of a known amino acid residue is found in the sequence.\n        Default value is :py:const:`False`.\n    labels : list, optional\n        A list of allowed labels for amino acids and terminal modifications.\n\n    Returns\n    -------\n    out : dict\n        A dictionary of amino acid composition.\n\n    Examples\n    --------\n    >>> amino_acid_composition('PEPTIDE') ==     {'I': 1, 'P': 2, 'E': 2, 'T': 1, 'D': 1}\n    True\n    >>> amino_acid_composition('PEPTDE', term_aa=True) ==     {'ctermE': 1, 'E': 1, 'D': 1, 'P': 1, 'T': 1, 'ntermP': 1}\n    True\n    >>> amino_acid_composition('PEPpTIDE', labels=std_labels+['pT']) ==     {'I': 1, 'P': 2, 'E': 2, 'D': 1, 'pT': 1}\n    True\n    ";
static const char __pyx_k_Calculate_the_number_of_amino_ac[] = "Calculate the number of amino acid residues in a polypeptide\n    written in modX notation.\n\n    Parameters\n    ----------\n    sequence : str or list or dict\n        A string with a polypeptide sequence, a list with a parsed sequence or\n        a dict of amino acid composition.\n    labels : list, optional\n        A list of allowed labels for amino acids and terminal modifications.\n\n    Examples\n    --------\n    >>> length('PEPTIDE')\n    7\n    >>> length('H-PEPTIDE-OH')\n    7\n    ";
static const char __pyx_k_Unsupported_type_of_a_sequence_M[] = "Unsupported type of a sequence.Must be str or list, not %s";
static PyObject *__pyx_kp_s_;
static PyObject *__pyx_n_s_A;
static PyObject *__pyx_n_s_BasicComposition;
static PyObject *__pyx_n_s_C;
static PyObject *__pyx_kp_u_Calculate_amino_acid_composition;
static PyObject *__pyx_kp_u_Calculate_the_number_of_amino_ac;
static PyObject *__pyx_kp_s_Cannot_split_a_non_modX_label_s;
static PyObject *__pyx_n_s_D;
static PyObject *__pyx_n_s_E;
static PyObject *__pyx_n_s_F;
static PyObject *__pyx_n_s_G;
static PyObject *__pyx_n_s_H;
static PyObject *__pyx_kp_s_H_2;
static PyObject *__pyx_n_s_I;
static PyObject *__pyx_n_s_K;
static PyObject *__pyx_n_s_L;
static PyObject *__pyx_n_s_M;
static PyObject *__pyx_n_s_N;
static PyObject *__pyx_kp_s_Not_a_valid_modX_sequence;
static PyObject *__pyx_kp_s_OH;
static PyObject *__pyx_n_s_P;
static PyObject *__pyx_kp_u_Parse_a_sequence_string_written;
static PyObject *__pyx_n_s_PyteomicsError;
static PyObject *__pyx_n_s_Q;
static PyObject *__pyx_n_s_R;
static PyObject *__pyx_n_s_S;
static PyObject *__pyx_n_s_T;
static PyObject *__pyx_kp_s_Unknown_label;
static PyObject *__pyx_kp_s_Unsupported_type_of_a_sequence_M;
static PyObject *__pyx_kp_s_Unsupported_type_of_sequence;
static PyObject *__pyx_n_s_V;
static PyObject *__pyx_n_s_W;
static PyObject *__pyx_n_s_Y;
static PyObject *__pyx_kp_s__5;
static PyObject *__pyx_kp_s_a_z_A_Z;
static PyObject *__pyx_kp_s_a_z_A_Z_2;
static PyObject *__pyx_kp_s_a_z_A_Z_3;
static PyObject *__pyx_n_s_allow_unknown_modifications;
static PyObject *__pyx_kp_u_amino_acid_composition_line_403;
static PyObject *__pyx_n_s_cline_in_traceback;
static PyObject *__pyx_n_s_collections;
static PyObject *__pyx_n_s_compile;
static PyObject *__pyx_n_s_cterm;
static PyObject *__pyx_n_s_deque;
static PyObject *__pyx_n_s_findall;
static PyObject *__pyx_n_s_format;
static PyObject *__pyx_n_s_groups;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_it;
static PyObject *__pyx_n_s_items;
static PyObject *__pyx_n_s_itertools;
static PyObject *__pyx_n_s_join;
static PyObject *__pyx_n_s_labels;
static PyObject *__pyx_kp_u_length_line_106;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_match;
static PyObject *__pyx_n_s_memoize;
static PyObject *__pyx_n_s_nterm;
static PyObject *__pyx_kp_u_parse_line_199;
static PyObject *__pyx_n_s_pop;
static PyObject *__pyx_n_s_pyteomics_auxiliary;
static PyObject *__pyx_n_s_range;
static PyObject *__pyx_n_s_re;
static PyObject *__pyx_n_s_remove;
static PyObject *__pyx_n_s_sequence;
static PyObject *__pyx_n_s_show_unmodified_termini;
static PyObject *__pyx_n_s_split;
static PyObject *__pyx_n_s_sum;
static PyObject *__pyx_n_s_term_aa;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_pf_9pyteomics_7cparser_length(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_sequence, int __pyx_v_show_unmodified_termini, int __pyx_v_split, int __pyx_v_allow_unknown_modifications, PyObject *__pyx_v_labels); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_2_split_label(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_label); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_4parse(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_sequence, int __pyx_v_show_unmodified_termini, int __pyx_v_split, int __pyx_v_allow_unknown_modifications, PyObject *__pyx_v_labels); /* proto */
static PyObject *__pyx_pf_9pyteomics_7cparser_6amino_acid_composition(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_sequence, int __pyx_v_show_unmodified_termini, int __pyx_v_term_aa, int __pyx_v_allow_unknown_modifications, PyObject *__pyx_v_labels); /* proto */
static PyObject *__pyx_int_0;
static PyObject *__pyx_int_1;
static PyObject *__pyx_k__2;
static PyObject *__pyx_k__4;
static PyObject *__pyx_k__6;
static PyObject *__pyx_tuple__3;
static PyObject *__pyx_tuple__7;
static PyObject *__pyx_tuple__8;
static PyObject *__pyx_tuple__9;

/* "pyteomics/cparser.pyx":60
 * 
 * @cython.nonecheck(False)
 * cdef bint is_term_mod(str label):             # <<<<<<<<<<<<<<
 *     """Check if `label` corresponds to a terminal modification.
 * 
 */

static int __pyx_f_9pyteomics_7cparser_is_term_mod(PyObject *__pyx_v_label) {
  int __pyx_v_result;
  int __pyx_r;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("is_term_mod", 0);
  __Pyx_TraceCall("is_term_mod", __pyx_f[0], 60, 0, __PYX_ERR(0, 60, __pyx_L1_error));

  /* "pyteomics/cparser.pyx":73
 *     cdef:
 *         bint result
 *     result = label.startswith("-")             # <<<<<<<<<<<<<<
 *     if result:
 *         return result
 */
  if (unlikely(__pyx_v_label == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "startswith");
    __PYX_ERR(0, 73, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyStr_Tailmatch(__pyx_v_label, __pyx_kp_s_, 0, PY_SSIZE_T_MAX, -1); if (unlikely(__pyx_t_1 == -1)) __PYX_ERR(0, 73, __pyx_L1_error)
  __pyx_v_result = __pyx_t_1;

  /* "pyteomics/cparser.pyx":74
 *         bint result
 *     result = label.startswith("-")
 *     if result:             # <<<<<<<<<<<<<<
 *         return result
 *     else:
 */
  __pyx_t_1 = (__pyx_v_result != 0);
  if (__pyx_t_1) {

    /* "pyteomics/cparser.pyx":75
 *     result = label.startswith("-")
 *     if result:
 *         return result             # <<<<<<<<<<<<<<
 *     else:
 *         result = label.endswith("-")
 */
    __pyx_r = __pyx_v_result;
    goto __pyx_L0;

    /* "pyteomics/cparser.pyx":74
 *         bint result
 *     result = label.startswith("-")
 *     if result:             # <<<<<<<<<<<<<<
 *         return result
 *     else:
 */
  }

  /* "pyteomics/cparser.pyx":77
 *         return result
 *     else:
 *         result = label.endswith("-")             # <<<<<<<<<<<<<<
 *         return result
 * 
 */
  /*else*/ {
    if (unlikely(__pyx_v_label == Py_None)) {
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "endswith");
      __PYX_ERR(0, 77, __pyx_L1_error)
    }
    __pyx_t_1 = __Pyx_PyStr_Tailmatch(__pyx_v_label, __pyx_kp_s_, 0, PY_SSIZE_T_MAX, 1); if (unlikely(__pyx_t_1 == -1)) __PYX_ERR(0, 77, __pyx_L1_error)
    __pyx_v_result = __pyx_t_1;

    /* "pyteomics/cparser.pyx":78
 *     else:
 *         result = label.endswith("-")
 *         return result             # <<<<<<<<<<<<<<
 * 
 * cdef inline object match_modX(str label):
 */
    __pyx_r = __pyx_v_result;
    goto __pyx_L0;
  }

  /* "pyteomics/cparser.pyx":60
 * 
 * @cython.nonecheck(False)
 * cdef bint is_term_mod(str label):             # <<<<<<<<<<<<<<
 *     """Check if `label` corresponds to a terminal modification.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("pyteomics.cparser.is_term_mod", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_TraceReturn(Py_None, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":80
 *         return result
 * 
 * cdef inline object match_modX(str label):             # <<<<<<<<<<<<<<
 *     """Check if `label` is a valid 'modX' label.
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_9pyteomics_7cparser_match_modX(PyObject *__pyx_v_label) {
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("match_modX", 0);
  __Pyx_TraceCall("match_modX", __pyx_f[0], 80, 0, __PYX_ERR(0, 80, __pyx_L1_error));

  /* "pyteomics/cparser.pyx":91
 *     out : re.match or None
 *     """
 *     return PyObject_CallMethodObjArgs(_modX_split, "match", <PyObject*>label, NULL)             # <<<<<<<<<<<<<<
 * 
 * cdef inline int is_modX(str label):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_v_9pyteomics_7cparser__modX_split;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = PyObject_CallMethodObjArgs(__pyx_t_1, __pyx_n_s_match, ((PyObject *)__pyx_v_label), NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 91, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "pyteomics/cparser.pyx":80
 *         return result
 * 
 * cdef inline object match_modX(str label):             # <<<<<<<<<<<<<<
 *     """Check if `label` is a valid 'modX' label.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("pyteomics.cparser.match_modX", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":93
 *     return PyObject_CallMethodObjArgs(_modX_split, "match", <PyObject*>label, NULL)
 * 
 * cdef inline int is_modX(str label):             # <<<<<<<<<<<<<<
 *     """Check if `label` is a valid 'modX' label.
 * 
 */

static CYTHON_INLINE int __pyx_f_9pyteomics_7cparser_is_modX(PyObject *__pyx_v_label) {
  int __pyx_r;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("is_modX", 0);
  __Pyx_TraceCall("is_modX", __pyx_f[0], 93, 0, __PYX_ERR(0, 93, __pyx_L1_error));

  /* "pyteomics/cparser.pyx":104
 *     out : bool
 *     """
 *     return bool(match_modX(label))             # <<<<<<<<<<<<<<
 * 
 * cpdef int length(object sequence,
 */
  __pyx_t_1 = __pyx_f_9pyteomics_7cparser_match_modX(__pyx_v_label); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 104, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 104, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = (!(!__pyx_t_2));
  goto __pyx_L0;

  /* "pyteomics/cparser.pyx":93
 *     return PyObject_CallMethodObjArgs(_modX_split, "match", <PyObject*>label, NULL)
 * 
 * cdef inline int is_modX(str label):             # <<<<<<<<<<<<<<
 *     """Check if `label` is a valid 'modX' label.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_WriteUnraisable("pyteomics.cparser.is_modX", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_TraceReturn(Py_None, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":106
 *     return bool(match_modX(label))
 * 
 * cpdef int length(object sequence,             # <<<<<<<<<<<<<<
 *                  int show_unmodified_termini=0, int split=0,
 *                  int allow_unknown_modifications=0, object labels=std_labels):
 */

static PyObject *__pyx_pw_9pyteomics_7cparser_1length(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_f_9pyteomics_7cparser_length(PyObject *__pyx_v_sequence, CYTHON_UNUSED int __pyx_skip_dispatch, struct __pyx_opt_args_9pyteomics_7cparser_length *__pyx_optional_args) {
  int __pyx_v_show_unmodified_termini = ((int)0);
  int __pyx_v_split = ((int)0);
  int __pyx_v_allow_unknown_modifications = ((int)0);
  PyObject *__pyx_v_labels = __pyx_k__2;
  PyObject *__pyx_v_parsed_sequence = NULL;
  PyObject *__pyx_v_num_term_groups = NULL;
  PyObject *__pyx_v_aa = NULL;
  PyObject *__pyx_v_amount = NULL;
  int __pyx_r;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  struct __pyx_opt_args_9pyteomics_7cparser_parse __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *(*__pyx_t_11)(PyObject *);
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *(*__pyx_t_14)(PyObject *);
  __Pyx_RefNannySetupContext("length", 0);
  __Pyx_TraceCall("length", __pyx_f[0], 106, 0, __PYX_ERR(0, 106, __pyx_L1_error));
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_show_unmodified_termini = __pyx_optional_args->show_unmodified_termini;
      if (__pyx_optional_args->__pyx_n > 1) {
        __pyx_v_split = __pyx_optional_args->split;
        if (__pyx_optional_args->__pyx_n > 2) {
          __pyx_v_allow_unknown_modifications = __pyx_optional_args->allow_unknown_modifications;
          if (__pyx_optional_args->__pyx_n > 3) {
            __pyx_v_labels = __pyx_optional_args->labels;
          }
        }
      }
    }
  }

  /* "pyteomics/cparser.pyx":127
 *     7
 *     """
 *     if not sequence: return 0             # <<<<<<<<<<<<<<
 * 
 *     if isinstance(sequence, str) or isinstance(sequence, list):
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_sequence); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 127, __pyx_L1_error)
  __pyx_t_2 = ((!__pyx_t_1) != 0);
  if (__pyx_t_2) {
    __pyx_r = 0;
    goto __pyx_L0;
  }

  /* "pyteomics/cparser.pyx":129
 *     if not sequence: return 0
 * 
 *     if isinstance(sequence, str) or isinstance(sequence, list):             # <<<<<<<<<<<<<<
 *         if isinstance(sequence, str):
 *             parsed_sequence = parse(sequence, show_unmodified_termini,
 */
  __pyx_t_1 = PyString_Check(__pyx_v_sequence); 
  __pyx_t_3 = (__pyx_t_1 != 0);
  if (!__pyx_t_3) {
  } else {
    __pyx_t_2 = __pyx_t_3;
    goto __pyx_L5_bool_binop_done;
  }
  __pyx_t_3 = PyList_Check(__pyx_v_sequence); 
  __pyx_t_1 = (__pyx_t_3 != 0);
  __pyx_t_2 = __pyx_t_1;
  __pyx_L5_bool_binop_done:;
  if (__pyx_t_2) {

    /* "pyteomics/cparser.pyx":130
 * 
 *     if isinstance(sequence, str) or isinstance(sequence, list):
 *         if isinstance(sequence, str):             # <<<<<<<<<<<<<<
 *             parsed_sequence = parse(sequence, show_unmodified_termini,
 *                                     split, allow_unknown_modifications, labels)
 */
    __pyx_t_2 = PyString_Check(__pyx_v_sequence); 
    __pyx_t_1 = (__pyx_t_2 != 0);
    if (__pyx_t_1) {

      /* "pyteomics/cparser.pyx":131
 *     if isinstance(sequence, str) or isinstance(sequence, list):
 *         if isinstance(sequence, str):
 *             parsed_sequence = parse(sequence, show_unmodified_termini,             # <<<<<<<<<<<<<<
 *                                     split, allow_unknown_modifications, labels)
 *         else:
 */
      if (!(likely(PyString_CheckExact(__pyx_v_sequence))||((__pyx_v_sequence) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_v_sequence)->tp_name), 0))) __PYX_ERR(0, 131, __pyx_L1_error)

      /* "pyteomics/cparser.pyx":132
 *         if isinstance(sequence, str):
 *             parsed_sequence = parse(sequence, show_unmodified_termini,
 *                                     split, allow_unknown_modifications, labels)             # <<<<<<<<<<<<<<
 *         else:
 *             parsed_sequence = sequence
 */
      __pyx_t_5.__pyx_n = 4;
      __pyx_t_5.show_unmodified_termini = __pyx_v_show_unmodified_termini;
      __pyx_t_5.split = __pyx_v_split;
      __pyx_t_5.allow_unknown_modifications = __pyx_v_allow_unknown_modifications;
      __pyx_t_5.labels = __pyx_v_labels;
      __pyx_t_4 = __pyx_f_9pyteomics_7cparser_parse(((PyObject*)__pyx_v_sequence), 0, &__pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 131, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_v_parsed_sequence = __pyx_t_4;
      __pyx_t_4 = 0;

      /* "pyteomics/cparser.pyx":130
 * 
 *     if isinstance(sequence, str) or isinstance(sequence, list):
 *         if isinstance(sequence, str):             # <<<<<<<<<<<<<<
 *             parsed_sequence = parse(sequence, show_unmodified_termini,
 *                                     split, allow_unknown_modifications, labels)
 */
      goto __pyx_L7;
    }

    /* "pyteomics/cparser.pyx":134
 *                                     split, allow_unknown_modifications, labels)
 *         else:
 *             parsed_sequence = sequence             # <<<<<<<<<<<<<<
 *         num_term_groups = 0
 *         if is_term_mod(parsed_sequence[0]):
 */
    /*else*/ {
      __Pyx_INCREF(__pyx_v_sequence);
      __pyx_v_parsed_sequence = __pyx_v_sequence;
    }
    __pyx_L7:;

    /* "pyteomics/cparser.pyx":135
 *         else:
 *             parsed_sequence = sequence
 *         num_term_groups = 0             # <<<<<<<<<<<<<<
 *         if is_term_mod(parsed_sequence[0]):
 *             num_term_groups += 1
 */
    __Pyx_INCREF(__pyx_int_0);
    __pyx_v_num_term_groups = __pyx_int_0;

    /* "pyteomics/cparser.pyx":136
 *             parsed_sequence = sequence
 *         num_term_groups = 0
 *         if is_term_mod(parsed_sequence[0]):             # <<<<<<<<<<<<<<
 *             num_term_groups += 1
 *         if is_term_mod(parsed_sequence[-1]):
 */
    __pyx_t_4 = __Pyx_GetItemInt(__pyx_v_parsed_sequence, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 136, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (!(likely(PyString_CheckExact(__pyx_t_4))||((__pyx_t_4) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_4)->tp_name), 0))) __PYX_ERR(0, 136, __pyx_L1_error)
    __pyx_t_1 = (__pyx_f_9pyteomics_7cparser_is_term_mod(((PyObject*)__pyx_t_4)) != 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (__pyx_t_1) {

      /* "pyteomics/cparser.pyx":137
 *         num_term_groups = 0
 *         if is_term_mod(parsed_sequence[0]):
 *             num_term_groups += 1             # <<<<<<<<<<<<<<
 *         if is_term_mod(parsed_sequence[-1]):
 *             num_term_groups += 1
 */
      __pyx_t_4 = __Pyx_PyInt_AddObjC(__pyx_v_num_term_groups, __pyx_int_1, 1, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 137, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF_SET(__pyx_v_num_term_groups, __pyx_t_4);
      __pyx_t_4 = 0;

      /* "pyteomics/cparser.pyx":136
 *             parsed_sequence = sequence
 *         num_term_groups = 0
 *         if is_term_mod(parsed_sequence[0]):             # <<<<<<<<<<<<<<
 *             num_term_groups += 1
 *         if is_term_mod(parsed_sequence[-1]):
 */
    }

    /* "pyteomics/cparser.pyx":138
 *         if is_term_mod(parsed_sequence[0]):
 *             num_term_groups += 1
 *         if is_term_mod(parsed_sequence[-1]):             # <<<<<<<<<<<<<<
 *             num_term_groups += 1
 *         return len(parsed_sequence) - num_term_groups
 */
    __pyx_t_4 = __Pyx_GetItemInt(__pyx_v_parsed_sequence, -1L, long, 1, __Pyx_PyInt_From_long, 0, 1, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 138, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (!(likely(PyString_CheckExact(__pyx_t_4))||((__pyx_t_4) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_4)->tp_name), 0))) __PYX_ERR(0, 138, __pyx_L1_error)
    __pyx_t_1 = (__pyx_f_9pyteomics_7cparser_is_term_mod(((PyObject*)__pyx_t_4)) != 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (__pyx_t_1) {

      /* "pyteomics/cparser.pyx":139
 *             num_term_groups += 1
 *         if is_term_mod(parsed_sequence[-1]):
 *             num_term_groups += 1             # <<<<<<<<<<<<<<
 *         return len(parsed_sequence) - num_term_groups
 *     elif isinstance(sequence, dict):
 */
      __pyx_t_4 = __Pyx_PyInt_AddObjC(__pyx_v_num_term_groups, __pyx_int_1, 1, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 139, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF_SET(__pyx_v_num_term_groups, __pyx_t_4);
      __pyx_t_4 = 0;

      /* "pyteomics/cparser.pyx":138
 *         if is_term_mod(parsed_sequence[0]):
 *             num_term_groups += 1
 *         if is_term_mod(parsed_sequence[-1]):             # <<<<<<<<<<<<<<
 *             num_term_groups += 1
 *         return len(parsed_sequence) - num_term_groups
 */
    }

    /* "pyteomics/cparser.pyx":140
 *         if is_term_mod(parsed_sequence[-1]):
 *             num_term_groups += 1
 *         return len(parsed_sequence) - num_term_groups             # <<<<<<<<<<<<<<
 *     elif isinstance(sequence, dict):
 *         return sum([amount for aa, amount in sequence.items()
 */
    __pyx_t_6 = PyObject_Length(__pyx_v_parsed_sequence); if (unlikely(__pyx_t_6 == -1)) __PYX_ERR(0, 140, __pyx_L1_error)
    __pyx_t_4 = PyInt_FromSsize_t(__pyx_t_6); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 140, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_7 = PyNumber_Subtract(__pyx_t_4, __pyx_v_num_term_groups); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 140, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_8 = __Pyx_PyInt_As_int(__pyx_t_7); if (unlikely((__pyx_t_8 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 140, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_r = __pyx_t_8;
    goto __pyx_L0;

    /* "pyteomics/cparser.pyx":129
 *     if not sequence: return 0
 * 
 *     if isinstance(sequence, str) or isinstance(sequence, list):             # <<<<<<<<<<<<<<
 *         if isinstance(sequence, str):
 *             parsed_sequence = parse(sequence, show_unmodified_termini,
 */
  }

  /* "pyteomics/cparser.pyx":141
 *             num_term_groups += 1
 *         return len(parsed_sequence) - num_term_groups
 *     elif isinstance(sequence, dict):             # <<<<<<<<<<<<<<
 *         return sum([amount for aa, amount in sequence.items()
 *                     if not is_term_mod(aa)])
 */
  __pyx_t_1 = PyDict_Check(__pyx_v_sequence); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "pyteomics/cparser.pyx":142
 *         return len(parsed_sequence) - num_term_groups
 *     elif isinstance(sequence, dict):
 *         return sum([amount for aa, amount in sequence.items()             # <<<<<<<<<<<<<<
 *                     if not is_term_mod(aa)])
 * 
 */
    __pyx_t_7 = PyList_New(0); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 142, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_sequence, __pyx_n_s_items); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 142, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_10 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_9))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_9);
      if (likely(__pyx_t_10)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_9, function);
      }
    }
    if (__pyx_t_10) {
      __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_t_10); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 142, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    } else {
      __pyx_t_4 = __Pyx_PyObject_CallNoArg(__pyx_t_9); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 142, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (likely(PyList_CheckExact(__pyx_t_4)) || PyTuple_CheckExact(__pyx_t_4)) {
      __pyx_t_9 = __pyx_t_4; __Pyx_INCREF(__pyx_t_9); __pyx_t_6 = 0;
      __pyx_t_11 = NULL;
    } else {
      __pyx_t_6 = -1; __pyx_t_9 = PyObject_GetIter(__pyx_t_4); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 142, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_11 = Py_TYPE(__pyx_t_9)->tp_iternext; if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 142, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    for (;;) {
      if (likely(!__pyx_t_11)) {
        if (likely(PyList_CheckExact(__pyx_t_9))) {
          if (__pyx_t_6 >= PyList_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_4 = PyList_GET_ITEM(__pyx_t_9, __pyx_t_6); __Pyx_INCREF(__pyx_t_4); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 142, __pyx_L1_error)
          #else
          __pyx_t_4 = PySequence_ITEM(__pyx_t_9, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 142, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          #endif
        } else {
          if (__pyx_t_6 >= PyTuple_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_4 = PyTuple_GET_ITEM(__pyx_t_9, __pyx_t_6); __Pyx_INCREF(__pyx_t_4); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 142, __pyx_L1_error)
          #else
          __pyx_t_4 = PySequence_ITEM(__pyx_t_9, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 142, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          #endif
        }
      } else {
        __pyx_t_4 = __pyx_t_11(__pyx_t_9);
        if (unlikely(!__pyx_t_4)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 142, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_4);
      }
      if ((likely(PyTuple_CheckExact(__pyx_t_4))) || (PyList_CheckExact(__pyx_t_4))) {
        PyObject* sequence = __pyx_t_4;
        #if !CYTHON_COMPILING_IN_PYPY
        Py_ssize_t size = Py_SIZE(sequence);
        #else
        Py_ssize_t size = PySequence_Size(sequence);
        #endif
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          __PYX_ERR(0, 142, __pyx_L1_error)
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_10 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_12 = PyTuple_GET_ITEM(sequence, 1); 
        } else {
          __pyx_t_10 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_12 = PyList_GET_ITEM(sequence, 1); 
        }
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(__pyx_t_12);
        #else
        __pyx_t_10 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 142, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __pyx_t_12 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 142, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_12);
        #endif
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_13 = PyObject_GetIter(__pyx_t_4); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 142, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_t_14 = Py_TYPE(__pyx_t_13)->tp_iternext;
        index = 0; __pyx_t_10 = __pyx_t_14(__pyx_t_13); if (unlikely(!__pyx_t_10)) goto __pyx_L12_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_10);
        index = 1; __pyx_t_12 = __pyx_t_14(__pyx_t_13); if (unlikely(!__pyx_t_12)) goto __pyx_L12_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_12);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_14(__pyx_t_13), 2) < 0) __PYX_ERR(0, 142, __pyx_L1_error)
        __pyx_t_14 = NULL;
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        goto __pyx_L13_unpacking_done;
        __pyx_L12_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        __pyx_t_14 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        __PYX_ERR(0, 142, __pyx_L1_error)
        __pyx_L13_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_v_aa, __pyx_t_10);
      __pyx_t_10 = 0;
      __Pyx_XDECREF_SET(__pyx_v_amount, __pyx_t_12);
      __pyx_t_12 = 0;

      /* "pyteomics/cparser.pyx":143
 *     elif isinstance(sequence, dict):
 *         return sum([amount for aa, amount in sequence.items()
 *                     if not is_term_mod(aa)])             # <<<<<<<<<<<<<<
 * 
 *     raise PyteomicsError('Unsupported type of sequence.')
 */
      if (!(likely(PyString_CheckExact(__pyx_v_aa))||((__pyx_v_aa) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_v_aa)->tp_name), 0))) __PYX_ERR(0, 143, __pyx_L1_error)
      __pyx_t_2 = ((!(__pyx_f_9pyteomics_7cparser_is_term_mod(((PyObject*)__pyx_v_aa)) != 0)) != 0);
      if (__pyx_t_2) {

        /* "pyteomics/cparser.pyx":142
 *         return len(parsed_sequence) - num_term_groups
 *     elif isinstance(sequence, dict):
 *         return sum([amount for aa, amount in sequence.items()             # <<<<<<<<<<<<<<
 *                     if not is_term_mod(aa)])
 * 
 */
        if (unlikely(__Pyx_ListComp_Append(__pyx_t_7, (PyObject*)__pyx_v_amount))) __PYX_ERR(0, 142, __pyx_L1_error)

        /* "pyteomics/cparser.pyx":143
 *     elif isinstance(sequence, dict):
 *         return sum([amount for aa, amount in sequence.items()
 *                     if not is_term_mod(aa)])             # <<<<<<<<<<<<<<
 * 
 *     raise PyteomicsError('Unsupported type of sequence.')
 */
      }

      /* "pyteomics/cparser.pyx":142
 *         return len(parsed_sequence) - num_term_groups
 *     elif isinstance(sequence, dict):
 *         return sum([amount for aa, amount in sequence.items()             # <<<<<<<<<<<<<<
 *                     if not is_term_mod(aa)])
 * 
 */
    }
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(1); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 142, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_7);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_7);
    __pyx_t_7 = 0;
    __pyx_t_7 = __Pyx_PyObject_Call(__pyx_builtin_sum, __pyx_t_9, NULL); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 142, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_8 = __Pyx_PyInt_As_int(__pyx_t_7); if (unlikely((__pyx_t_8 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 142, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_r = __pyx_t_8;
    goto __pyx_L0;

    /* "pyteomics/cparser.pyx":141
 *             num_term_groups += 1
 *         return len(parsed_sequence) - num_term_groups
 *     elif isinstance(sequence, dict):             # <<<<<<<<<<<<<<
 *         return sum([amount for aa, amount in sequence.items()
 *                     if not is_term_mod(aa)])
 */
  }

  /* "pyteomics/cparser.pyx":145
 *                     if not is_term_mod(aa)])
 * 
 *     raise PyteomicsError('Unsupported type of sequence.')             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_PyteomicsError); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 145, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_tuple__3, NULL); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 145, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_Raise(__pyx_t_9, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __PYX_ERR(0, 145, __pyx_L1_error)

  /* "pyteomics/cparser.pyx":106
 *     return bool(match_modX(label))
 * 
 * cpdef int length(object sequence,             # <<<<<<<<<<<<<<
 *                  int show_unmodified_termini=0, int split=0,
 *                  int allow_unknown_modifications=0, object labels=std_labels):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_WriteUnraisable("pyteomics.cparser.length", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_parsed_sequence);
  __Pyx_XDECREF(__pyx_v_num_term_groups);
  __Pyx_XDECREF(__pyx_v_aa);
  __Pyx_XDECREF(__pyx_v_amount);
  __Pyx_TraceReturn(Py_None, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_1length(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_9pyteomics_7cparser_length[] = "length(sequence, int show_unmodified_termini=0, int split=0, int allow_unknown_modifications=0, labels=std_labels) -> int\nCalculate the number of amino acid residues in a polypeptide\n    written in modX notation.\n\n    Parameters\n    ----------\n    sequence : str or list or dict\n        A string with a polypeptide sequence, a list with a parsed sequence or\n        a dict of amino acid composition.\n    labels : list, optional\n        A list of allowed labels for amino acids and terminal modifications.\n\n    Examples\n    --------\n    >>> length('PEPTIDE')\n    7\n    >>> length('H-PEPTIDE-OH')\n    7\n    ";
static PyObject *__pyx_pw_9pyteomics_7cparser_1length(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_sequence = 0;
  int __pyx_v_show_unmodified_termini;
  int __pyx_v_split;
  int __pyx_v_allow_unknown_modifications;
  PyObject *__pyx_v_labels = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("length (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_sequence,&__pyx_n_s_show_unmodified_termini,&__pyx_n_s_split,&__pyx_n_s_allow_unknown_modifications,&__pyx_n_s_labels,0};
    PyObject* values[5] = {0,0,0,0,0};
    values[4] = __pyx_k__2;
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_sequence)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_show_unmodified_termini);
          if (value) { values[1] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_split);
          if (value) { values[2] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_allow_unknown_modifications);
          if (value) { values[3] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_labels);
          if (value) { values[4] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "length") < 0)) __PYX_ERR(0, 106, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_sequence = values[0];
    if (values[1]) {
      __pyx_v_show_unmodified_termini = __Pyx_PyInt_As_int(values[1]); if (unlikely((__pyx_v_show_unmodified_termini == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 107, __pyx_L3_error)
    } else {
      __pyx_v_show_unmodified_termini = ((int)0);
    }
    if (values[2]) {
      __pyx_v_split = __Pyx_PyInt_As_int(values[2]); if (unlikely((__pyx_v_split == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 107, __pyx_L3_error)
    } else {
      __pyx_v_split = ((int)0);
    }
    if (values[3]) {
      __pyx_v_allow_unknown_modifications = __Pyx_PyInt_As_int(values[3]); if (unlikely((__pyx_v_allow_unknown_modifications == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 108, __pyx_L3_error)
    } else {
      __pyx_v_allow_unknown_modifications = ((int)0);
    }
    __pyx_v_labels = values[4];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("length", 0, 1, 5, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 106, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyteomics.cparser.length", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9pyteomics_7cparser_length(__pyx_self, __pyx_v_sequence, __pyx_v_show_unmodified_termini, __pyx_v_split, __pyx_v_allow_unknown_modifications, __pyx_v_labels);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_length(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_sequence, int __pyx_v_show_unmodified_termini, int __pyx_v_split, int __pyx_v_allow_unknown_modifications, PyObject *__pyx_v_labels) {
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  struct __pyx_opt_args_9pyteomics_7cparser_length __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("length", 0);
  __Pyx_TraceCall("length (wrapper)", __pyx_f[0], 106, 0, __PYX_ERR(0, 106, __pyx_L1_error));
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 4;
  __pyx_t_2.show_unmodified_termini = __pyx_v_show_unmodified_termini;
  __pyx_t_2.split = __pyx_v_split;
  __pyx_t_2.allow_unknown_modifications = __pyx_v_allow_unknown_modifications;
  __pyx_t_2.labels = __pyx_v_labels;
  __pyx_t_1 = __pyx_f_9pyteomics_7cparser_length(__pyx_v_sequence, 0, &__pyx_t_2); 
  __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 106, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("pyteomics.cparser.length", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":148
 * 
 * 
 * cdef inline list interpolate_labels(object labels):             # <<<<<<<<<<<<<<
 *     cdef list _labels
 *     if isinstance(labels, list):
 */

static CYTHON_INLINE PyObject *__pyx_f_9pyteomics_7cparser_interpolate_labels(PyObject *__pyx_v_labels) {
  PyObject *__pyx_v__labels = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("interpolate_labels", 0);
  __Pyx_TraceCall("interpolate_labels", __pyx_f[0], 148, 0, __PYX_ERR(0, 148, __pyx_L1_error));

  /* "pyteomics/cparser.pyx":150
 * cdef inline list interpolate_labels(object labels):
 *     cdef list _labels
 *     if isinstance(labels, list):             # <<<<<<<<<<<<<<
 *         _labels = <list>labels
 *     else:
 */
  __pyx_t_1 = PyList_Check(__pyx_v_labels); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "pyteomics/cparser.pyx":151
 *     cdef list _labels
 *     if isinstance(labels, list):
 *         _labels = <list>labels             # <<<<<<<<<<<<<<
 *     else:
 *         _labels = list(labels)
 */
    __pyx_t_3 = __pyx_v_labels;
    __Pyx_INCREF(__pyx_t_3);
    __pyx_v__labels = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "pyteomics/cparser.pyx":150
 * cdef inline list interpolate_labels(object labels):
 *     cdef list _labels
 *     if isinstance(labels, list):             # <<<<<<<<<<<<<<
 *         _labels = <list>labels
 *     else:
 */
    goto __pyx_L3;
  }

  /* "pyteomics/cparser.pyx":153
 *         _labels = <list>labels
 *     else:
 *         _labels = list(labels)             # <<<<<<<<<<<<<<
 *     _labels.extend((std_cterm, std_nterm))
 *     return _labels
 */
  /*else*/ {
    __pyx_t_3 = PySequence_List(__pyx_v_labels); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 153, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v__labels = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;
  }
  __pyx_L3:;

  /* "pyteomics/cparser.pyx":154
 *     else:
 *         _labels = list(labels)
 *     _labels.extend((std_cterm, std_nterm))             # <<<<<<<<<<<<<<
 *     return _labels
 * 
 */
  if (unlikely(__pyx_v__labels == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "extend");
    __PYX_ERR(0, 154, __pyx_L1_error)
  }
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 154, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_v_9pyteomics_7cparser_std_cterm);
  __Pyx_GIVEREF(__pyx_v_9pyteomics_7cparser_std_cterm);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_9pyteomics_7cparser_std_cterm);
  __Pyx_INCREF(__pyx_v_9pyteomics_7cparser_std_nterm);
  __Pyx_GIVEREF(__pyx_v_9pyteomics_7cparser_std_nterm);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_v_9pyteomics_7cparser_std_nterm);
  __pyx_t_4 = __Pyx_PyList_Extend(__pyx_v__labels, __pyx_t_3); if (unlikely(__pyx_t_4 == -1)) __PYX_ERR(0, 154, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "pyteomics/cparser.pyx":155
 *         _labels = list(labels)
 *     _labels.extend((std_cterm, std_nterm))
 *     return _labels             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v__labels);
  __pyx_r = __pyx_v__labels;
  goto __pyx_L0;

  /* "pyteomics/cparser.pyx":148
 * 
 * 
 * cdef inline list interpolate_labels(object labels):             # <<<<<<<<<<<<<<
 *     cdef list _labels
 *     if isinstance(labels, list):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("pyteomics.cparser.interpolate_labels", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v__labels);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":158
 * 
 * 
 * cpdef inline tuple _split_label(str label):             # <<<<<<<<<<<<<<
 *     cdef:
 *         str mod, X
 */

static PyObject *__pyx_pw_9pyteomics_7cparser_3_split_label(PyObject *__pyx_self, PyObject *__pyx_v_label); /*proto*/
static CYTHON_INLINE PyObject *__pyx_f_9pyteomics_7cparser__split_label(PyObject *__pyx_v_label, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_v_mod = 0;
  PyObject *__pyx_v_X = 0;
  PyObject *__pyx_v_temp = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7;
  int __pyx_t_8;
  __Pyx_RefNannySetupContext("_split_label", 0);
  __Pyx_TraceCall("_split_label", __pyx_f[0], 158, 0, __PYX_ERR(0, 158, __pyx_L1_error));

  /* "pyteomics/cparser.pyx":164
 * 
 * 
 *     temp = <tuple>PyObject_CallMethodObjArgs(match_modX(label), "groups", NULL)             # <<<<<<<<<<<<<<
 *     if <PyObject*>temp == NULL:
 *         raise PyteomicsError('Cannot split a non-modX label: %s' % label)
 */
  __pyx_t_1 = __pyx_f_9pyteomics_7cparser_match_modX(__pyx_v_label); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 164, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyObject_CallMethodObjArgs(__pyx_t_1, __pyx_n_s_groups, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 164, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __pyx_t_2;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_temp = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyteomics/cparser.pyx":165
 * 
 *     temp = <tuple>PyObject_CallMethodObjArgs(match_modX(label), "groups", NULL)
 *     if <PyObject*>temp == NULL:             # <<<<<<<<<<<<<<
 *         raise PyteomicsError('Cannot split a non-modX label: %s' % label)
 *     mod = <str>PyTuple_GET_ITEM(<tuple>temp, 0)
 */
  __pyx_t_3 = ((((PyObject *)__pyx_v_temp) == NULL) != 0);
  if (__pyx_t_3) {

    /* "pyteomics/cparser.pyx":166
 *     temp = <tuple>PyObject_CallMethodObjArgs(match_modX(label), "groups", NULL)
 *     if <PyObject*>temp == NULL:
 *         raise PyteomicsError('Cannot split a non-modX label: %s' % label)             # <<<<<<<<<<<<<<
 *     mod = <str>PyTuple_GET_ITEM(<tuple>temp, 0)
 *     X = <str>PyTuple_GET_ITEM(<tuple>temp, 1)
 */
    __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_PyteomicsError); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 166, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Cannot_split_a_non_modX_label_s, __pyx_v_label); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 166, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    if (!__pyx_t_5) {
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 166, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_2)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_4};
        __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 166, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_4};
        __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 166, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else
      #endif
      {
        __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 166, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __pyx_t_5 = NULL;
        __Pyx_GIVEREF(__pyx_t_4);
        PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_t_4);
        __pyx_t_4 = 0;
        __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 166, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 166, __pyx_L1_error)

    /* "pyteomics/cparser.pyx":165
 * 
 *     temp = <tuple>PyObject_CallMethodObjArgs(match_modX(label), "groups", NULL)
 *     if <PyObject*>temp == NULL:             # <<<<<<<<<<<<<<
 *         raise PyteomicsError('Cannot split a non-modX label: %s' % label)
 *     mod = <str>PyTuple_GET_ITEM(<tuple>temp, 0)
 */
  }

  /* "pyteomics/cparser.pyx":167
 *     if <PyObject*>temp == NULL:
 *         raise PyteomicsError('Cannot split a non-modX label: %s' % label)
 *     mod = <str>PyTuple_GET_ITEM(<tuple>temp, 0)             # <<<<<<<<<<<<<<
 *     X = <str>PyTuple_GET_ITEM(<tuple>temp, 1)
 *     if not mod:
 */
  __pyx_t_7 = PyTuple_GET_ITEM(__pyx_v_temp, 0);
  __pyx_t_1 = ((PyObject *)__pyx_t_7);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_mod = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyteomics/cparser.pyx":168
 *         raise PyteomicsError('Cannot split a non-modX label: %s' % label)
 *     mod = <str>PyTuple_GET_ITEM(<tuple>temp, 0)
 *     X = <str>PyTuple_GET_ITEM(<tuple>temp, 1)             # <<<<<<<<<<<<<<
 *     if not mod:
 *         return (X,)
 */
  __pyx_t_7 = PyTuple_GET_ITEM(__pyx_v_temp, 1);
  __pyx_t_1 = ((PyObject *)__pyx_t_7);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_X = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyteomics/cparser.pyx":169
 *     mod = <str>PyTuple_GET_ITEM(<tuple>temp, 0)
 *     X = <str>PyTuple_GET_ITEM(<tuple>temp, 1)
 *     if not mod:             # <<<<<<<<<<<<<<
 *         return (X,)
 *     else:
 */
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_v_mod); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 169, __pyx_L1_error)
  __pyx_t_8 = ((!__pyx_t_3) != 0);
  if (__pyx_t_8) {

    /* "pyteomics/cparser.pyx":170
 *     X = <str>PyTuple_GET_ITEM(<tuple>temp, 1)
 *     if not mod:
 *         return (X,)             # <<<<<<<<<<<<<<
 *     else:
 *         return temp
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 170, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_v_X);
    __Pyx_GIVEREF(__pyx_v_X);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_X);
    __pyx_r = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "pyteomics/cparser.pyx":169
 *     mod = <str>PyTuple_GET_ITEM(<tuple>temp, 0)
 *     X = <str>PyTuple_GET_ITEM(<tuple>temp, 1)
 *     if not mod:             # <<<<<<<<<<<<<<
 *         return (X,)
 *     else:
 */
  }

  /* "pyteomics/cparser.pyx":172
 *         return (X,)
 *     else:
 *         return temp             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_temp);
    __pyx_r = __pyx_v_temp;
    goto __pyx_L0;
  }

  /* "pyteomics/cparser.pyx":158
 * 
 * 
 * cpdef inline tuple _split_label(str label):             # <<<<<<<<<<<<<<
 *     cdef:
 *         str mod, X
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("pyteomics.cparser._split_label", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_mod);
  __Pyx_XDECREF(__pyx_v_X);
  __Pyx_XDECREF(__pyx_v_temp);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_3_split_label(PyObject *__pyx_self, PyObject *__pyx_v_label); /*proto*/
static char __pyx_doc_9pyteomics_7cparser_2_split_label[] = "_split_label(str label) -> tuple";
static PyObject *__pyx_pw_9pyteomics_7cparser_3_split_label(PyObject *__pyx_self, PyObject *__pyx_v_label) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_split_label (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_label), (&PyString_Type), 1, "label", 1))) __PYX_ERR(0, 158, __pyx_L1_error)
  __pyx_r = __pyx_pf_9pyteomics_7cparser_2_split_label(__pyx_self, ((PyObject*)__pyx_v_label));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_2_split_label(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_label) {
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("_split_label", 0);
  __Pyx_TraceCall("_split_label (wrapper)", __pyx_f[0], 158, 0, __PYX_ERR(0, 158, __pyx_L1_error));
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9pyteomics_7cparser__split_label(__pyx_v_label, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 158, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyteomics.cparser._split_label", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":178
 * 
 * 
 * cdef list _parse_modx_sequence_split(str sequence):             # <<<<<<<<<<<<<<
 *     cdef:
 *         list parsed_sequence
 */

static PyObject *__pyx_f_9pyteomics_7cparser__parse_modx_sequence_split(PyObject *__pyx_v_sequence) {
  PyObject *__pyx_v_parsed_sequence = 0;
  PyObject *__pyx_v_parts = 0;
  Py_ssize_t __pyx_v_i;
  Py_ssize_t __pyx_v_n;
  PyObject *__pyx_v_g = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  PyObject *__pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  __Pyx_RefNannySetupContext("_parse_modx_sequence_split", 0);
  __Pyx_TraceCall("_parse_modx_sequence_split", __pyx_f[0], 178, 0, __PYX_ERR(0, 178, __pyx_L1_error));

  /* "pyteomics/cparser.pyx":185
 *         object g
 * 
 *     parsed_sequence = []             # <<<<<<<<<<<<<<
 *     parts = tuple(_modX_split_findall(sequence))
 *     n = len(parts)
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 185, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_parsed_sequence = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyteomics/cparser.pyx":186
 * 
 *     parsed_sequence = []
 *     parts = tuple(_modX_split_findall(sequence))             # <<<<<<<<<<<<<<
 *     n = len(parts)
 * 
 */
  __Pyx_INCREF(__pyx_v_9pyteomics_7cparser__modX_split_findall);
  __pyx_t_2 = __pyx_v_9pyteomics_7cparser__modX_split_findall; __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_sequence); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 186, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_sequence};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 186, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_sequence};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 186, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 186, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_INCREF(__pyx_v_sequence);
      __Pyx_GIVEREF(__pyx_v_sequence);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v_sequence);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 186, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PySequence_Tuple(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 186, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_parts = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "pyteomics/cparser.pyx":187
 *     parsed_sequence = []
 *     parts = tuple(_modX_split_findall(sequence))
 *     n = len(parts)             # <<<<<<<<<<<<<<
 * 
 *     for i in range(n):
 */
  __pyx_t_5 = PyTuple_GET_SIZE(__pyx_v_parts); if (unlikely(__pyx_t_5 == -1)) __PYX_ERR(0, 187, __pyx_L1_error)
  __pyx_v_n = __pyx_t_5;

  /* "pyteomics/cparser.pyx":189
 *     n = len(parts)
 * 
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         g = <object>PyTuple_GET_ITEM(parts, i)
 * 
 */
  __pyx_t_5 = __pyx_v_n;
  for (__pyx_t_6 = 0; __pyx_t_6 < __pyx_t_5; __pyx_t_6+=1) {
    __pyx_v_i = __pyx_t_6;

    /* "pyteomics/cparser.pyx":190
 * 
 *     for i in range(n):
 *         g = <object>PyTuple_GET_ITEM(parts, i)             # <<<<<<<<<<<<<<
 * 
 *         if g[0]:
 */
    __pyx_t_7 = PyTuple_GET_ITEM(__pyx_v_parts, __pyx_v_i);
    __pyx_t_2 = ((PyObject *)__pyx_t_7);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_XDECREF_SET(__pyx_v_g, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "pyteomics/cparser.pyx":192
 *         g = <object>PyTuple_GET_ITEM(parts, i)
 * 
 *         if g[0]:             # <<<<<<<<<<<<<<
 *             parsed_sequence.append(g)
 *         else:
 */
    __pyx_t_2 = __Pyx_GetItemInt(__pyx_v_g, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 192, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 192, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_8) {

      /* "pyteomics/cparser.pyx":193
 * 
 *         if g[0]:
 *             parsed_sequence.append(g)             # <<<<<<<<<<<<<<
 *         else:
 *             parsed_sequence.append((g[1],))
 */
      __pyx_t_9 = __Pyx_PyList_Append(__pyx_v_parsed_sequence, __pyx_v_g); if (unlikely(__pyx_t_9 == -1)) __PYX_ERR(0, 193, __pyx_L1_error)

      /* "pyteomics/cparser.pyx":192
 *         g = <object>PyTuple_GET_ITEM(parts, i)
 * 
 *         if g[0]:             # <<<<<<<<<<<<<<
 *             parsed_sequence.append(g)
 *         else:
 */
      goto __pyx_L5;
    }

    /* "pyteomics/cparser.pyx":195
 *             parsed_sequence.append(g)
 *         else:
 *             parsed_sequence.append((g[1],))             # <<<<<<<<<<<<<<
 *     return parsed_sequence
 * 
 */
    /*else*/ {
      __pyx_t_2 = __Pyx_GetItemInt(__pyx_v_g, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 195, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 195, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GIVEREF(__pyx_t_2);
      PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
      __pyx_t_2 = 0;
      __pyx_t_9 = __Pyx_PyList_Append(__pyx_v_parsed_sequence, __pyx_t_1); if (unlikely(__pyx_t_9 == -1)) __PYX_ERR(0, 195, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    }
    __pyx_L5:;
  }

  /* "pyteomics/cparser.pyx":196
 *         else:
 *             parsed_sequence.append((g[1],))
 *     return parsed_sequence             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_parsed_sequence);
  __pyx_r = __pyx_v_parsed_sequence;
  goto __pyx_L0;

  /* "pyteomics/cparser.pyx":178
 * 
 * 
 * cdef list _parse_modx_sequence_split(str sequence):             # <<<<<<<<<<<<<<
 *     cdef:
 *         list parsed_sequence
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pyteomics.cparser._parse_modx_sequence_split", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_parsed_sequence);
  __Pyx_XDECREF(__pyx_v_parts);
  __Pyx_XDECREF(__pyx_v_g);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":199
 * 
 * 
 * cpdef list parse(str sequence, bint show_unmodified_termini=0, bint split=0,             # <<<<<<<<<<<<<<
 *                  bint allow_unknown_modifications=0, object labels=std_labels):
 *     """Parse a sequence string written in modX notation into a list of
 */

static PyObject *__pyx_pw_9pyteomics_7cparser_5parse(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_9pyteomics_7cparser_parse(PyObject *__pyx_v_sequence, CYTHON_UNUSED int __pyx_skip_dispatch, struct __pyx_opt_args_9pyteomics_7cparser_parse *__pyx_optional_args) {
  int __pyx_v_show_unmodified_termini = ((int)0);
  int __pyx_v_split = ((int)0);
  int __pyx_v_allow_unknown_modifications = ((int)0);
  PyObject *__pyx_v_labels = __pyx_k__4;
  PyObject *__pyx_v_match = 0;
  PyObject *__pyx_v_temp = 0;
  PyObject *__pyx_v_n = 0;
  PyObject *__pyx_v_body = 0;
  PyObject *__pyx_v_c = 0;
  PyObject *__pyx_v_nterm = 0;
  PyObject *__pyx_v_cterm = 0;
  PyObject *__pyx_v_mod = 0;
  PyObject *__pyx_v_X = 0;
  PyObject *__pyx_v_parsed_sequence = 0;
  PyObject *__pyx_v_tgroup = 0;
  PyObject *__pyx_v_sgroup = 0;
  PyObject *__pyx_v__labels = 0;
  Py_ssize_t __pyx_v_i;
  int __pyx_v_is_mod;
  int __pyx_v_in_labels;
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8;
  int __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  Py_ssize_t __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  int __pyx_t_14;
  int __pyx_t_15;
  __Pyx_RefNannySetupContext("parse", 0);
  __Pyx_TraceCall("parse", __pyx_f[0], 199, 0, __PYX_ERR(0, 199, __pyx_L1_error));
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_show_unmodified_termini = __pyx_optional_args->show_unmodified_termini;
      if (__pyx_optional_args->__pyx_n > 1) {
        __pyx_v_split = __pyx_optional_args->split;
        if (__pyx_optional_args->__pyx_n > 2) {
          __pyx_v_allow_unknown_modifications = __pyx_optional_args->allow_unknown_modifications;
          if (__pyx_optional_args->__pyx_n > 3) {
            __pyx_v_labels = __pyx_optional_args->labels;
          }
        }
      }
    }
  }

  /* "pyteomics/cparser.pyx":272
 *         int is_mod, is_residue, in_labels
 * 
 *     _labels = interpolate_labels(labels)             # <<<<<<<<<<<<<<
 * 
 *     match = PyObject_CallMethodObjArgs(_modX_sequence, "match", <PyObject*>sequence, NULL)
 */
  __pyx_t_1 = __pyx_f_9pyteomics_7cparser_interpolate_labels(__pyx_v_labels); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 272, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v__labels = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyteomics/cparser.pyx":274
 *     _labels = interpolate_labels(labels)
 * 
 *     match = PyObject_CallMethodObjArgs(_modX_sequence, "match", <PyObject*>sequence, NULL)             # <<<<<<<<<<<<<<
 *     if match is None:
 *         raise PyteomicsError('Not a valid modX sequence: ' + sequence)
 */
  __pyx_t_1 = __pyx_v_9pyteomics_7cparser__modX_sequence;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = PyObject_CallMethodObjArgs(__pyx_t_1, __pyx_n_s_match, ((PyObject *)__pyx_v_sequence), NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 274, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_match = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "pyteomics/cparser.pyx":275
 * 
 *     match = PyObject_CallMethodObjArgs(_modX_sequence, "match", <PyObject*>sequence, NULL)
 *     if match is None:             # <<<<<<<<<<<<<<
 *         raise PyteomicsError('Not a valid modX sequence: ' + sequence)
 *     temp = <tuple>PyObject_CallMethodObjArgs(match, "groups", NULL)
 */
  __pyx_t_3 = (__pyx_v_match == Py_None);
  __pyx_t_4 = (__pyx_t_3 != 0);
  if (__pyx_t_4) {

    /* "pyteomics/cparser.pyx":276
 *     match = PyObject_CallMethodObjArgs(_modX_sequence, "match", <PyObject*>sequence, NULL)
 *     if match is None:
 *         raise PyteomicsError('Not a valid modX sequence: ' + sequence)             # <<<<<<<<<<<<<<
 *     temp = <tuple>PyObject_CallMethodObjArgs(match, "groups", NULL)
 *     n = <str>PyTuple_GET_ITEM(temp, 0)
 */
    __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_PyteomicsError); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 276, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = PyNumber_Add(__pyx_kp_s_Not_a_valid_modX_sequence, __pyx_v_sequence); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 276, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
      }
    }
    if (!__pyx_t_6) {
      __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 276, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_2);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_1)) {
        PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_t_5};
        __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_1, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 276, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_1)) {
        PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_t_5};
        __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_1, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 276, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      } else
      #endif
      {
        __pyx_t_7 = PyTuple_New(1+1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 276, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_6); __pyx_t_6 = NULL;
        __Pyx_GIVEREF(__pyx_t_5);
        PyTuple_SET_ITEM(__pyx_t_7, 0+1, __pyx_t_5);
        __pyx_t_5 = 0;
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_7, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 276, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 276, __pyx_L1_error)

    /* "pyteomics/cparser.pyx":275
 * 
 *     match = PyObject_CallMethodObjArgs(_modX_sequence, "match", <PyObject*>sequence, NULL)
 *     if match is None:             # <<<<<<<<<<<<<<
 *         raise PyteomicsError('Not a valid modX sequence: ' + sequence)
 *     temp = <tuple>PyObject_CallMethodObjArgs(match, "groups", NULL)
 */
  }

  /* "pyteomics/cparser.pyx":277
 *     if match is None:
 *         raise PyteomicsError('Not a valid modX sequence: ' + sequence)
 *     temp = <tuple>PyObject_CallMethodObjArgs(match, "groups", NULL)             # <<<<<<<<<<<<<<
 *     n = <str>PyTuple_GET_ITEM(temp, 0)
 *     body = <str>PyTuple_GET_ITEM(temp, 1)
 */
  __pyx_t_2 = PyObject_CallMethodObjArgs(__pyx_v_match, __pyx_n_s_groups, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 277, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __pyx_t_2;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_temp = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyteomics/cparser.pyx":278
 *         raise PyteomicsError('Not a valid modX sequence: ' + sequence)
 *     temp = <tuple>PyObject_CallMethodObjArgs(match, "groups", NULL)
 *     n = <str>PyTuple_GET_ITEM(temp, 0)             # <<<<<<<<<<<<<<
 *     body = <str>PyTuple_GET_ITEM(temp, 1)
 *     c = <str>PyTuple_GET_ITEM(temp, 2)
 */
  __pyx_t_8 = PyTuple_GET_ITEM(__pyx_v_temp, 0);
  __pyx_t_1 = ((PyObject *)__pyx_t_8);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_n = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyteomics/cparser.pyx":279
 *     temp = <tuple>PyObject_CallMethodObjArgs(match, "groups", NULL)
 *     n = <str>PyTuple_GET_ITEM(temp, 0)
 *     body = <str>PyTuple_GET_ITEM(temp, 1)             # <<<<<<<<<<<<<<
 *     c = <str>PyTuple_GET_ITEM(temp, 2)
 *     # slabels = set(_labels)
 */
  __pyx_t_8 = PyTuple_GET_ITEM(__pyx_v_temp, 1);
  __pyx_t_1 = ((PyObject *)__pyx_t_8);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_body = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyteomics/cparser.pyx":280
 *     n = <str>PyTuple_GET_ITEM(temp, 0)
 *     body = <str>PyTuple_GET_ITEM(temp, 1)
 *     c = <str>PyTuple_GET_ITEM(temp, 2)             # <<<<<<<<<<<<<<
 *     # slabels = set(_labels)
 *     # labels help save the day when only one terminal group is given
 */
  __pyx_t_8 = PyTuple_GET_ITEM(__pyx_v_temp, 2);
  __pyx_t_1 = ((PyObject *)__pyx_t_8);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_c = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyteomics/cparser.pyx":283
 *     # slabels = set(_labels)
 *     # labels help save the day when only one terminal group is given
 *     if c is None and n is not None:             # <<<<<<<<<<<<<<
 *         # we can try to resolve the ambiguity
 *         if n != std_nterm and n not in _labels:
 */
  __pyx_t_3 = (__pyx_v_c == ((PyObject*)Py_None));
  __pyx_t_9 = (__pyx_t_3 != 0);
  if (__pyx_t_9) {
  } else {
    __pyx_t_4 = __pyx_t_9;
    goto __pyx_L5_bool_binop_done;
  }
  __pyx_t_9 = (__pyx_v_n != ((PyObject*)Py_None));
  __pyx_t_3 = (__pyx_t_9 != 0);
  __pyx_t_4 = __pyx_t_3;
  __pyx_L5_bool_binop_done:;
  if (__pyx_t_4) {

    /* "pyteomics/cparser.pyx":285
 *     if c is None and n is not None:
 *         # we can try to resolve the ambiguity
 *         if n != std_nterm and n not in _labels:             # <<<<<<<<<<<<<<
 *             # n is the body then
 *             c = '-' + body
 */
    __pyx_t_3 = (__Pyx_PyString_Equals(__pyx_v_n, __pyx_v_9pyteomics_7cparser_std_nterm, Py_NE)); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 285, __pyx_L1_error)
    __pyx_t_9 = (__pyx_t_3 != 0);
    if (__pyx_t_9) {
    } else {
      __pyx_t_4 = __pyx_t_9;
      goto __pyx_L8_bool_binop_done;
    }
    __pyx_t_9 = (__Pyx_PySequence_ContainsTF(__pyx_v_n, __pyx_v__labels, Py_NE)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 285, __pyx_L1_error)
    __pyx_t_3 = (__pyx_t_9 != 0);
    __pyx_t_4 = __pyx_t_3;
    __pyx_L8_bool_binop_done:;
    if (__pyx_t_4) {

      /* "pyteomics/cparser.pyx":287
 *         if n != std_nterm and n not in _labels:
 *             # n is the body then
 *             c = '-' + body             # <<<<<<<<<<<<<<
 *             body = n[:-1]
 *             n = None
 */
      __pyx_t_1 = PyNumber_Add(__pyx_kp_s_, __pyx_v_body); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 287, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF_SET(__pyx_v_c, ((PyObject*)__pyx_t_1));
      __pyx_t_1 = 0;

      /* "pyteomics/cparser.pyx":288
 *             # n is the body then
 *             c = '-' + body
 *             body = n[:-1]             # <<<<<<<<<<<<<<
 *             n = None
 * 
 */
      if (unlikely(__pyx_v_n == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
        __PYX_ERR(0, 288, __pyx_L1_error)
      }
      __pyx_t_1 = PySequence_GetSlice(__pyx_v_n, 0, -1L); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 288, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF_SET(__pyx_v_body, ((PyObject*)__pyx_t_1));
      __pyx_t_1 = 0;

      /* "pyteomics/cparser.pyx":289
 *             c = '-' + body
 *             body = n[:-1]
 *             n = None             # <<<<<<<<<<<<<<
 * 
 *     # Actual parsing
 */
      __Pyx_INCREF(Py_None);
      __Pyx_DECREF_SET(__pyx_v_n, ((PyObject*)Py_None));

      /* "pyteomics/cparser.pyx":285
 *     if c is None and n is not None:
 *         # we can try to resolve the ambiguity
 *         if n != std_nterm and n not in _labels:             # <<<<<<<<<<<<<<
 *             # n is the body then
 *             c = '-' + body
 */
    }

    /* "pyteomics/cparser.pyx":283
 *     # slabels = set(_labels)
 *     # labels help save the day when only one terminal group is given
 *     if c is None and n is not None:             # <<<<<<<<<<<<<<
 *         # we can try to resolve the ambiguity
 *         if n != std_nterm and n not in _labels:
 */
  }

  /* "pyteomics/cparser.pyx":292
 * 
 *     # Actual parsing
 *     if split:             # <<<<<<<<<<<<<<
 *         # parsed_sequence = [g if g[0] else (g[1],) for g in re.findall(
 *         #     _modX_split, body)]
 */
  __pyx_t_4 = (__pyx_v_split != 0);
  if (__pyx_t_4) {

    /* "pyteomics/cparser.pyx":295
 *         # parsed_sequence = [g if g[0] else (g[1],) for g in re.findall(
 *         #     _modX_split, body)]
 *         parsed_sequence = _parse_modx_sequence_split(body)             # <<<<<<<<<<<<<<
 *     else:
 *         parsed_sequence = <list>PyObject_CallMethodObjArgs(_modX_group, "findall", <PyObject*>body, NULL)
 */
    __pyx_t_1 = __pyx_f_9pyteomics_7cparser__parse_modx_sequence_split(__pyx_v_body); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 295, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_v_parsed_sequence = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "pyteomics/cparser.pyx":292
 * 
 *     # Actual parsing
 *     if split:             # <<<<<<<<<<<<<<
 *         # parsed_sequence = [g if g[0] else (g[1],) for g in re.findall(
 *         #     _modX_split, body)]
 */
    goto __pyx_L10;
  }

  /* "pyteomics/cparser.pyx":297
 *         parsed_sequence = _parse_modx_sequence_split(body)
 *     else:
 *         parsed_sequence = <list>PyObject_CallMethodObjArgs(_modX_group, "findall", <PyObject*>body, NULL)             # <<<<<<<<<<<<<<
 *     nterm, cterm = (n or std_nterm), (c or std_cterm)
 * 
 */
  /*else*/ {
    __pyx_t_1 = __pyx_v_9pyteomics_7cparser__modX_group;
    __Pyx_INCREF(__pyx_t_1);
    __pyx_t_2 = PyObject_CallMethodObjArgs(__pyx_t_1, __pyx_n_s_findall, ((PyObject *)__pyx_v_body), NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 297, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __pyx_t_2;
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_parsed_sequence = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;
  }
  __pyx_L10:;

  /* "pyteomics/cparser.pyx":298
 *     else:
 *         parsed_sequence = <list>PyObject_CallMethodObjArgs(_modX_group, "findall", <PyObject*>body, NULL)
 *     nterm, cterm = (n or std_nterm), (c or std_cterm)             # <<<<<<<<<<<<<<
 * 
 *     if not allow_unknown_modifications:
 */
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_v_n); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 298, __pyx_L1_error)
  if (!__pyx_t_4) {
  } else {
    __Pyx_INCREF(__pyx_v_n);
    __pyx_t_1 = __pyx_v_n;
    goto __pyx_L11_bool_binop_done;
  }
  __Pyx_INCREF(__pyx_v_9pyteomics_7cparser_std_nterm);
  __pyx_t_1 = __pyx_v_9pyteomics_7cparser_std_nterm;
  __pyx_L11_bool_binop_done:;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_v_c); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 298, __pyx_L1_error)
  if (!__pyx_t_4) {
  } else {
    __Pyx_INCREF(__pyx_v_c);
    __pyx_t_2 = __pyx_v_c;
    goto __pyx_L13_bool_binop_done;
  }
  __Pyx_INCREF(__pyx_v_9pyteomics_7cparser_std_cterm);
  __pyx_t_2 = __pyx_v_9pyteomics_7cparser_std_cterm;
  __pyx_L13_bool_binop_done:;
  __pyx_v_nterm = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_v_cterm = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "pyteomics/cparser.pyx":300
 *     nterm, cterm = (n or std_nterm), (c or std_cterm)
 * 
 *     if not allow_unknown_modifications:             # <<<<<<<<<<<<<<
 *         if nterm is not None and nterm not in _labels:
 *             raise PyteomicsError('Unknown label: {}'.format(nterm))
 */
  __pyx_t_4 = ((!(__pyx_v_allow_unknown_modifications != 0)) != 0);
  if (__pyx_t_4) {

    /* "pyteomics/cparser.pyx":301
 * 
 *     if not allow_unknown_modifications:
 *         if nterm is not None and nterm not in _labels:             # <<<<<<<<<<<<<<
 *             raise PyteomicsError('Unknown label: {}'.format(nterm))
 *         if cterm is not None and cterm not in _labels:
 */
    __pyx_t_3 = (__pyx_v_nterm != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_3 != 0);
    if (__pyx_t_9) {
    } else {
      __pyx_t_4 = __pyx_t_9;
      goto __pyx_L17_bool_binop_done;
    }
    __pyx_t_9 = (__Pyx_PySequence_ContainsTF(__pyx_v_nterm, __pyx_v__labels, Py_NE)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 301, __pyx_L1_error)
    __pyx_t_3 = (__pyx_t_9 != 0);
    __pyx_t_4 = __pyx_t_3;
    __pyx_L17_bool_binop_done:;
    if (__pyx_t_4) {

      /* "pyteomics/cparser.pyx":302
 *     if not allow_unknown_modifications:
 *         if nterm is not None and nterm not in _labels:
 *             raise PyteomicsError('Unknown label: {}'.format(nterm))             # <<<<<<<<<<<<<<
 *         if cterm is not None and cterm not in _labels:
 *             raise PyteomicsError('Unknown label: {}'.format(cterm))
 */
      __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_PyteomicsError); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 302, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_Unknown_label, __pyx_n_s_format); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 302, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
        if (likely(__pyx_t_6)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_6);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_5, function);
        }
      }
      if (!__pyx_t_6) {
        __pyx_t_7 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_v_nterm); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 302, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
      } else {
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_5)) {
          PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_v_nterm};
          __pyx_t_7 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 302, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_GOTREF(__pyx_t_7);
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
          PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_v_nterm};
          __pyx_t_7 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 302, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_GOTREF(__pyx_t_7);
        } else
        #endif
        {
          __pyx_t_10 = PyTuple_New(1+1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 302, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_10);
          __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_6); __pyx_t_6 = NULL;
          __Pyx_INCREF(__pyx_v_nterm);
          __Pyx_GIVEREF(__pyx_v_nterm);
          PyTuple_SET_ITEM(__pyx_t_10, 0+1, __pyx_v_nterm);
          __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_10, NULL); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 302, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_5 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
        __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_1);
        if (likely(__pyx_t_5)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
          __Pyx_INCREF(__pyx_t_5);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_1, function);
        }
      }
      if (!__pyx_t_5) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 302, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else {
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_1)) {
          PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_7};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_1, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 302, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_1)) {
          PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_7};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_1, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 302, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        } else
        #endif
        {
          __pyx_t_10 = PyTuple_New(1+1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 302, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_10);
          __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_5); __pyx_t_5 = NULL;
          __Pyx_GIVEREF(__pyx_t_7);
          PyTuple_SET_ITEM(__pyx_t_10, 0+1, __pyx_t_7);
          __pyx_t_7 = 0;
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_10, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 302, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_Raise(__pyx_t_2, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __PYX_ERR(0, 302, __pyx_L1_error)

      /* "pyteomics/cparser.pyx":301
 * 
 *     if not allow_unknown_modifications:
 *         if nterm is not None and nterm not in _labels:             # <<<<<<<<<<<<<<
 *             raise PyteomicsError('Unknown label: {}'.format(nterm))
 *         if cterm is not None and cterm not in _labels:
 */
    }

    /* "pyteomics/cparser.pyx":303
 *         if nterm is not None and nterm not in _labels:
 *             raise PyteomicsError('Unknown label: {}'.format(nterm))
 *         if cterm is not None and cterm not in _labels:             # <<<<<<<<<<<<<<
 *             raise PyteomicsError('Unknown label: {}'.format(cterm))
 * 
 */
    __pyx_t_3 = (__pyx_v_cterm != ((PyObject*)Py_None));
    __pyx_t_9 = (__pyx_t_3 != 0);
    if (__pyx_t_9) {
    } else {
      __pyx_t_4 = __pyx_t_9;
      goto __pyx_L20_bool_binop_done;
    }
    __pyx_t_9 = (__Pyx_PySequence_ContainsTF(__pyx_v_cterm, __pyx_v__labels, Py_NE)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 303, __pyx_L1_error)
    __pyx_t_3 = (__pyx_t_9 != 0);
    __pyx_t_4 = __pyx_t_3;
    __pyx_L20_bool_binop_done:;
    if (__pyx_t_4) {

      /* "pyteomics/cparser.pyx":304
 *             raise PyteomicsError('Unknown label: {}'.format(nterm))
 *         if cterm is not None and cterm not in _labels:
 *             raise PyteomicsError('Unknown label: {}'.format(cterm))             # <<<<<<<<<<<<<<
 * 
 *     if split:
 */
      __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_PyteomicsError); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 304, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_Unknown_label, __pyx_n_s_format); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 304, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_5 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
        __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_7);
        if (likely(__pyx_t_5)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
          __Pyx_INCREF(__pyx_t_5);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_7, function);
        }
      }
      if (!__pyx_t_5) {
        __pyx_t_10 = __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_v_cterm); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 304, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
      } else {
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_v_cterm};
          __pyx_t_10 = __Pyx_PyFunction_FastCall(__pyx_t_7, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 304, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_GOTREF(__pyx_t_10);
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_v_cterm};
          __pyx_t_10 = __Pyx_PyCFunction_FastCall(__pyx_t_7, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 304, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_GOTREF(__pyx_t_10);
        } else
        #endif
        {
          __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 304, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __pyx_t_5 = NULL;
          __Pyx_INCREF(__pyx_v_cterm);
          __Pyx_GIVEREF(__pyx_v_cterm);
          PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_v_cterm);
          __pyx_t_10 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_6, NULL); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 304, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_10);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_7 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
        __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_1);
        if (likely(__pyx_t_7)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
          __Pyx_INCREF(__pyx_t_7);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_1, function);
        }
      }
      if (!__pyx_t_7) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_10); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 304, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else {
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_1)) {
          PyObject *__pyx_temp[2] = {__pyx_t_7, __pyx_t_10};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_1, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 304, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_1)) {
          PyObject *__pyx_temp[2] = {__pyx_t_7, __pyx_t_10};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_1, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 304, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        } else
        #endif
        {
          __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 304, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_7); __pyx_t_7 = NULL;
          __Pyx_GIVEREF(__pyx_t_10);
          PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_t_10);
          __pyx_t_10 = 0;
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 304, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_Raise(__pyx_t_2, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __PYX_ERR(0, 304, __pyx_L1_error)

      /* "pyteomics/cparser.pyx":303
 *         if nterm is not None and nterm not in _labels:
 *             raise PyteomicsError('Unknown label: {}'.format(nterm))
 *         if cterm is not None and cterm not in _labels:             # <<<<<<<<<<<<<<
 *             raise PyteomicsError('Unknown label: {}'.format(cterm))
 * 
 */
    }

    /* "pyteomics/cparser.pyx":300
 *     nterm, cterm = (n or std_nterm), (c or std_cterm)
 * 
 *     if not allow_unknown_modifications:             # <<<<<<<<<<<<<<
 *         if nterm is not None and nterm not in _labels:
 *             raise PyteomicsError('Unknown label: {}'.format(nterm))
 */
  }

  /* "pyteomics/cparser.pyx":306
 *             raise PyteomicsError('Unknown label: {}'.format(cterm))
 * 
 *     if split:             # <<<<<<<<<<<<<<
 *         i = 0
 *         for i in range(PyList_Size(parsed_sequence)):
 */
  __pyx_t_4 = (__pyx_v_split != 0);
  if (__pyx_t_4) {

    /* "pyteomics/cparser.pyx":307
 * 
 *     if split:
 *         i = 0             # <<<<<<<<<<<<<<
 *         for i in range(PyList_Size(parsed_sequence)):
 *             tgroup = <tuple>PyList_GET_ITEM(parsed_sequence, i)
 */
    __pyx_v_i = 0;

    /* "pyteomics/cparser.pyx":308
 *     if split:
 *         i = 0
 *         for i in range(PyList_Size(parsed_sequence)):             # <<<<<<<<<<<<<<
 *             tgroup = <tuple>PyList_GET_ITEM(parsed_sequence, i)
 *             if len(tgroup) == 2:
 */
    __pyx_t_11 = PyList_Size(__pyx_v_parsed_sequence); if (unlikely(__pyx_t_11 == -1L)) __PYX_ERR(0, 308, __pyx_L1_error)
    for (__pyx_t_12 = 0; __pyx_t_12 < __pyx_t_11; __pyx_t_12+=1) {
      __pyx_v_i = __pyx_t_12;

      /* "pyteomics/cparser.pyx":309
 *         i = 0
 *         for i in range(PyList_Size(parsed_sequence)):
 *             tgroup = <tuple>PyList_GET_ITEM(parsed_sequence, i)             # <<<<<<<<<<<<<<
 *             if len(tgroup) == 2:
 *                 mod = <str>PyTuple_GET_ITEM(tgroup, 0)
 */
      __pyx_t_8 = PyList_GET_ITEM(__pyx_v_parsed_sequence, __pyx_v_i);
      __pyx_t_2 = ((PyObject *)__pyx_t_8);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_XDECREF_SET(__pyx_v_tgroup, ((PyObject*)__pyx_t_2));
      __pyx_t_2 = 0;

      /* "pyteomics/cparser.pyx":310
 *         for i in range(PyList_Size(parsed_sequence)):
 *             tgroup = <tuple>PyList_GET_ITEM(parsed_sequence, i)
 *             if len(tgroup) == 2:             # <<<<<<<<<<<<<<
 *                 mod = <str>PyTuple_GET_ITEM(tgroup, 0)
 *                 X = <str>PyTuple_GET_ITEM(tgroup, 1)
 */
      if (unlikely(__pyx_v_tgroup == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
        __PYX_ERR(0, 310, __pyx_L1_error)
      }
      __pyx_t_13 = PyTuple_GET_SIZE(__pyx_v_tgroup); if (unlikely(__pyx_t_13 == -1)) __PYX_ERR(0, 310, __pyx_L1_error)
      __pyx_t_4 = ((__pyx_t_13 == 2) != 0);
      if (__pyx_t_4) {

        /* "pyteomics/cparser.pyx":311
 *             tgroup = <tuple>PyList_GET_ITEM(parsed_sequence, i)
 *             if len(tgroup) == 2:
 *                 mod = <str>PyTuple_GET_ITEM(tgroup, 0)             # <<<<<<<<<<<<<<
 *                 X = <str>PyTuple_GET_ITEM(tgroup, 1)
 *             else:
 */
        __pyx_t_8 = PyTuple_GET_ITEM(__pyx_v_tgroup, 0);
        __pyx_t_2 = ((PyObject *)__pyx_t_8);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_XDECREF_SET(__pyx_v_mod, ((PyObject*)__pyx_t_2));
        __pyx_t_2 = 0;

        /* "pyteomics/cparser.pyx":312
 *             if len(tgroup) == 2:
 *                 mod = <str>PyTuple_GET_ITEM(tgroup, 0)
 *                 X = <str>PyTuple_GET_ITEM(tgroup, 1)             # <<<<<<<<<<<<<<
 *             else:
 *                 mod = ''
 */
        __pyx_t_8 = PyTuple_GET_ITEM(__pyx_v_tgroup, 1);
        __pyx_t_2 = ((PyObject *)__pyx_t_8);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_XDECREF_SET(__pyx_v_X, ((PyObject*)__pyx_t_2));
        __pyx_t_2 = 0;

        /* "pyteomics/cparser.pyx":310
 *         for i in range(PyList_Size(parsed_sequence)):
 *             tgroup = <tuple>PyList_GET_ITEM(parsed_sequence, i)
 *             if len(tgroup) == 2:             # <<<<<<<<<<<<<<
 *                 mod = <str>PyTuple_GET_ITEM(tgroup, 0)
 *                 X = <str>PyTuple_GET_ITEM(tgroup, 1)
 */
        goto __pyx_L25;
      }

      /* "pyteomics/cparser.pyx":314
 *                 X = <str>PyTuple_GET_ITEM(tgroup, 1)
 *             else:
 *                 mod = ''             # <<<<<<<<<<<<<<
 *                 X = <str>PyTuple_GET_ITEM(tgroup, 0)
 *             if ((not mod) and X not in _labels) or\
 */
      /*else*/ {
        __Pyx_INCREF(__pyx_kp_s__5);
        __Pyx_XDECREF_SET(__pyx_v_mod, __pyx_kp_s__5);

        /* "pyteomics/cparser.pyx":315
 *             else:
 *                 mod = ''
 *                 X = <str>PyTuple_GET_ITEM(tgroup, 0)             # <<<<<<<<<<<<<<
 *             if ((not mod) and X not in _labels) or\
 *                 not ((mod+X in _labels) or\
 */
        __pyx_t_8 = PyTuple_GET_ITEM(__pyx_v_tgroup, 0);
        __pyx_t_2 = ((PyObject *)__pyx_t_8);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_XDECREF_SET(__pyx_v_X, ((PyObject*)__pyx_t_2));
        __pyx_t_2 = 0;
      }
      __pyx_L25:;

      /* "pyteomics/cparser.pyx":316
 *                 mod = ''
 *                 X = <str>PyTuple_GET_ITEM(tgroup, 0)
 *             if ((not mod) and X not in _labels) or\             # <<<<<<<<<<<<<<
 *                 not ((mod+X in _labels) or\
 *                 (X in _labels and (mod in _labels or allow_unknown_modifications))):
 */
      __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_v_mod); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 316, __pyx_L1_error)
      __pyx_t_9 = ((!__pyx_t_3) != 0);
      if (!__pyx_t_9) {
        goto __pyx_L28_next_or;
      } else {
      }
      __pyx_t_9 = (__Pyx_PySequence_ContainsTF(__pyx_v_X, __pyx_v__labels, Py_NE)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 316, __pyx_L1_error)
      __pyx_t_3 = (__pyx_t_9 != 0);
      if (!__pyx_t_3) {
      } else {
        __pyx_t_4 = __pyx_t_3;
        goto __pyx_L27_bool_binop_done;
      }
      __pyx_L28_next_or:;

      /* "pyteomics/cparser.pyx":317
 *                 X = <str>PyTuple_GET_ITEM(tgroup, 0)
 *             if ((not mod) and X not in _labels) or\
 *                 not ((mod+X in _labels) or\             # <<<<<<<<<<<<<<
 *                 (X in _labels and (mod in _labels or allow_unknown_modifications))):
 *                 raise PyteomicsError('Unknown label: {}'.format(tgroup))
 */
      __pyx_t_2 = PyNumber_Add(__pyx_v_mod, __pyx_v_X); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 317, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_9 = (__Pyx_PySequence_ContainsTF(__pyx_t_2, __pyx_v__labels, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 317, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_14 = (__pyx_t_9 != 0);
      if (!__pyx_t_14) {
      } else {
        __pyx_t_3 = __pyx_t_14;
        goto __pyx_L30_bool_binop_done;
      }

      /* "pyteomics/cparser.pyx":318
 *             if ((not mod) and X not in _labels) or\
 *                 not ((mod+X in _labels) or\
 *                 (X in _labels and (mod in _labels or allow_unknown_modifications))):             # <<<<<<<<<<<<<<
 *                 raise PyteomicsError('Unknown label: {}'.format(tgroup))
 * 
 */
      __pyx_t_14 = (__Pyx_PySequence_ContainsTF(__pyx_v_X, __pyx_v__labels, Py_EQ)); if (unlikely(__pyx_t_14 < 0)) __PYX_ERR(0, 318, __pyx_L1_error)
      __pyx_t_9 = (__pyx_t_14 != 0);
      if (__pyx_t_9) {
      } else {
        __pyx_t_3 = __pyx_t_9;
        goto __pyx_L30_bool_binop_done;
      }
      __pyx_t_9 = (__Pyx_PySequence_ContainsTF(__pyx_v_mod, __pyx_v__labels, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 318, __pyx_L1_error)
      __pyx_t_14 = (__pyx_t_9 != 0);
      if (!__pyx_t_14) {
      } else {
        __pyx_t_3 = __pyx_t_14;
        goto __pyx_L30_bool_binop_done;
      }
      __pyx_t_14 = (__pyx_v_allow_unknown_modifications != 0);
      __pyx_t_3 = __pyx_t_14;
      __pyx_L30_bool_binop_done:;

      /* "pyteomics/cparser.pyx":317
 *                 X = <str>PyTuple_GET_ITEM(tgroup, 0)
 *             if ((not mod) and X not in _labels) or\
 *                 not ((mod+X in _labels) or\             # <<<<<<<<<<<<<<
 *                 (X in _labels and (mod in _labels or allow_unknown_modifications))):
 *                 raise PyteomicsError('Unknown label: {}'.format(tgroup))
 */
      __pyx_t_14 = ((!__pyx_t_3) != 0);
      __pyx_t_4 = __pyx_t_14;
      __pyx_L27_bool_binop_done:;

      /* "pyteomics/cparser.pyx":316
 *                 mod = ''
 *                 X = <str>PyTuple_GET_ITEM(tgroup, 0)
 *             if ((not mod) and X not in _labels) or\             # <<<<<<<<<<<<<<
 *                 not ((mod+X in _labels) or\
 *                 (X in _labels and (mod in _labels or allow_unknown_modifications))):
 */
      if (__pyx_t_4) {

        /* "pyteomics/cparser.pyx":319
 *                 not ((mod+X in _labels) or\
 *                 (X in _labels and (mod in _labels or allow_unknown_modifications))):
 *                 raise PyteomicsError('Unknown label: {}'.format(tgroup))             # <<<<<<<<<<<<<<
 * 
 *     else:
 */
        __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_PyteomicsError); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 319, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_Unknown_label, __pyx_n_s_format); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 319, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __pyx_t_7 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_10))) {
          __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_10);
          if (likely(__pyx_t_7)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
            __Pyx_INCREF(__pyx_t_7);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_10, function);
          }
        }
        if (!__pyx_t_7) {
          __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_v_tgroup); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 319, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
        } else {
          #if CYTHON_FAST_PYCALL
          if (PyFunction_Check(__pyx_t_10)) {
            PyObject *__pyx_temp[2] = {__pyx_t_7, __pyx_v_tgroup};
            __pyx_t_6 = __Pyx_PyFunction_FastCall(__pyx_t_10, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 319, __pyx_L1_error)
            __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
            __Pyx_GOTREF(__pyx_t_6);
          } else
          #endif
          #if CYTHON_FAST_PYCCALL
          if (__Pyx_PyFastCFunction_Check(__pyx_t_10)) {
            PyObject *__pyx_temp[2] = {__pyx_t_7, __pyx_v_tgroup};
            __pyx_t_6 = __Pyx_PyCFunction_FastCall(__pyx_t_10, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 319, __pyx_L1_error)
            __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
            __Pyx_GOTREF(__pyx_t_6);
          } else
          #endif
          {
            __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 319, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_5);
            __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_7); __pyx_t_7 = NULL;
            __Pyx_INCREF(__pyx_v_tgroup);
            __Pyx_GIVEREF(__pyx_v_tgroup);
            PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_v_tgroup);
            __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_10, __pyx_t_5, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 319, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_6);
            __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          }
        }
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        __pyx_t_10 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
          __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_1);
          if (likely(__pyx_t_10)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
            __Pyx_INCREF(__pyx_t_10);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_1, function);
          }
        }
        if (!__pyx_t_10) {
          __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 319, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else {
          #if CYTHON_FAST_PYCALL
          if (PyFunction_Check(__pyx_t_1)) {
            PyObject *__pyx_temp[2] = {__pyx_t_10, __pyx_t_6};
            __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_1, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 319, __pyx_L1_error)
            __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
            __Pyx_GOTREF(__pyx_t_2);
            __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          } else
          #endif
          #if CYTHON_FAST_PYCCALL
          if (__Pyx_PyFastCFunction_Check(__pyx_t_1)) {
            PyObject *__pyx_temp[2] = {__pyx_t_10, __pyx_t_6};
            __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_1, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 319, __pyx_L1_error)
            __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
            __Pyx_GOTREF(__pyx_t_2);
            __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          } else
          #endif
          {
            __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 319, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_5);
            __Pyx_GIVEREF(__pyx_t_10); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_10); __pyx_t_10 = NULL;
            __Pyx_GIVEREF(__pyx_t_6);
            PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_6);
            __pyx_t_6 = 0;
            __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_5, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 319, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_2);
            __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          }
        }
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_Raise(__pyx_t_2, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __PYX_ERR(0, 319, __pyx_L1_error)

        /* "pyteomics/cparser.pyx":316
 *                 mod = ''
 *                 X = <str>PyTuple_GET_ITEM(tgroup, 0)
 *             if ((not mod) and X not in _labels) or\             # <<<<<<<<<<<<<<
 *                 not ((mod+X in _labels) or\
 *                 (X in _labels and (mod in _labels or allow_unknown_modifications))):
 */
      }
    }

    /* "pyteomics/cparser.pyx":306
 *             raise PyteomicsError('Unknown label: {}'.format(cterm))
 * 
 *     if split:             # <<<<<<<<<<<<<<
 *         i = 0
 *         for i in range(PyList_Size(parsed_sequence)):
 */
    goto __pyx_L22;
  }

  /* "pyteomics/cparser.pyx":322
 * 
 *     else:
 *         i = 0             # <<<<<<<<<<<<<<
 *         for i in range(PyList_Size(parsed_sequence)):
 *             sgroup = <str>PyList_GET_ITEM(parsed_sequence, i)
 */
  /*else*/ {
    __pyx_v_i = 0;

    /* "pyteomics/cparser.pyx":323
 *     else:
 *         i = 0
 *         for i in range(PyList_Size(parsed_sequence)):             # <<<<<<<<<<<<<<
 *             sgroup = <str>PyList_GET_ITEM(parsed_sequence, i)
 * 
 */
    __pyx_t_11 = PyList_Size(__pyx_v_parsed_sequence); if (unlikely(__pyx_t_11 == -1L)) __PYX_ERR(0, 323, __pyx_L1_error)
    for (__pyx_t_12 = 0; __pyx_t_12 < __pyx_t_11; __pyx_t_12+=1) {
      __pyx_v_i = __pyx_t_12;

      /* "pyteomics/cparser.pyx":324
 *         i = 0
 *         for i in range(PyList_Size(parsed_sequence)):
 *             sgroup = <str>PyList_GET_ITEM(parsed_sequence, i)             # <<<<<<<<<<<<<<
 * 
 *             match = PyObject_CallMethodObjArgs(_modX_split, "match", <PyObject*>sgroup, NULL)
 */
      __pyx_t_8 = PyList_GET_ITEM(__pyx_v_parsed_sequence, __pyx_v_i);
      __pyx_t_2 = ((PyObject *)__pyx_t_8);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_XDECREF_SET(__pyx_v_sgroup, ((PyObject*)__pyx_t_2));
      __pyx_t_2 = 0;

      /* "pyteomics/cparser.pyx":326
 *             sgroup = <str>PyList_GET_ITEM(parsed_sequence, i)
 * 
 *             match = PyObject_CallMethodObjArgs(_modX_split, "match", <PyObject*>sgroup, NULL)             # <<<<<<<<<<<<<<
 *             temp = <tuple>PyObject_CallMethodObjArgs(match, "groups", NULL)
 *             mod = <str>PyTuple_GET_ITEM(temp, 0)
 */
      __pyx_t_2 = __pyx_v_9pyteomics_7cparser__modX_split;
      __Pyx_INCREF(__pyx_t_2);
      __pyx_t_1 = PyObject_CallMethodObjArgs(__pyx_t_2, __pyx_n_s_match, ((PyObject *)__pyx_v_sgroup), NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 326, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF_SET(__pyx_v_match, __pyx_t_1);
      __pyx_t_1 = 0;

      /* "pyteomics/cparser.pyx":327
 * 
 *             match = PyObject_CallMethodObjArgs(_modX_split, "match", <PyObject*>sgroup, NULL)
 *             temp = <tuple>PyObject_CallMethodObjArgs(match, "groups", NULL)             # <<<<<<<<<<<<<<
 *             mod = <str>PyTuple_GET_ITEM(temp, 0)
 *             X = <str>PyTuple_GET_ITEM(temp, 1)
 */
      __pyx_t_1 = PyObject_CallMethodObjArgs(__pyx_v_match, __pyx_n_s_groups, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 327, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_2 = __pyx_t_1;
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF_SET(__pyx_v_temp, ((PyObject*)__pyx_t_2));
      __pyx_t_2 = 0;

      /* "pyteomics/cparser.pyx":328
 *             match = PyObject_CallMethodObjArgs(_modX_split, "match", <PyObject*>sgroup, NULL)
 *             temp = <tuple>PyObject_CallMethodObjArgs(match, "groups", NULL)
 *             mod = <str>PyTuple_GET_ITEM(temp, 0)             # <<<<<<<<<<<<<<
 *             X = <str>PyTuple_GET_ITEM(temp, 1)
 *             is_mod = PyObject_IsTrue(mod)
 */
      __pyx_t_8 = PyTuple_GET_ITEM(__pyx_v_temp, 0);
      __pyx_t_2 = ((PyObject *)__pyx_t_8);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_XDECREF_SET(__pyx_v_mod, ((PyObject*)__pyx_t_2));
      __pyx_t_2 = 0;

      /* "pyteomics/cparser.pyx":329
 *             temp = <tuple>PyObject_CallMethodObjArgs(match, "groups", NULL)
 *             mod = <str>PyTuple_GET_ITEM(temp, 0)
 *             X = <str>PyTuple_GET_ITEM(temp, 1)             # <<<<<<<<<<<<<<
 *             is_mod = PyObject_IsTrue(mod)
 *             in_labels = X not in _labels
 */
      __pyx_t_8 = PyTuple_GET_ITEM(__pyx_v_temp, 1);
      __pyx_t_2 = ((PyObject *)__pyx_t_8);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_XDECREF_SET(__pyx_v_X, ((PyObject*)__pyx_t_2));
      __pyx_t_2 = 0;

      /* "pyteomics/cparser.pyx":330
 *             mod = <str>PyTuple_GET_ITEM(temp, 0)
 *             X = <str>PyTuple_GET_ITEM(temp, 1)
 *             is_mod = PyObject_IsTrue(mod)             # <<<<<<<<<<<<<<
 *             in_labels = X not in _labels
 *             if ((not is_mod) and in_labels) or\
 */
      __pyx_t_4 = PyObject_IsTrue(__pyx_v_mod); if (unlikely(__pyx_t_4 == -1)) __PYX_ERR(0, 330, __pyx_L1_error)
      __pyx_v_is_mod = __pyx_t_4;

      /* "pyteomics/cparser.pyx":331
 *             X = <str>PyTuple_GET_ITEM(temp, 1)
 *             is_mod = PyObject_IsTrue(mod)
 *             in_labels = X not in _labels             # <<<<<<<<<<<<<<
 *             if ((not is_mod) and in_labels) or\
 *                 not ((mod+X in _labels) or\
 */
      __pyx_t_4 = (__Pyx_PySequence_ContainsTF(__pyx_v_X, __pyx_v__labels, Py_NE)); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 331, __pyx_L1_error)
      __pyx_v_in_labels = __pyx_t_4;

      /* "pyteomics/cparser.pyx":332
 *             is_mod = PyObject_IsTrue(mod)
 *             in_labels = X not in _labels
 *             if ((not is_mod) and in_labels) or\             # <<<<<<<<<<<<<<
 *                 not ((mod+X in _labels) or\
 *                 (X in _labels and (mod in _labels or allow_unknown_modifications))):
 */
      __pyx_t_14 = ((!(__pyx_v_is_mod != 0)) != 0);
      if (!__pyx_t_14) {
        goto __pyx_L38_next_or;
      } else {
      }
      __pyx_t_14 = (__pyx_v_in_labels != 0);
      if (!__pyx_t_14) {
      } else {
        __pyx_t_4 = __pyx_t_14;
        goto __pyx_L37_bool_binop_done;
      }
      __pyx_L38_next_or:;

      /* "pyteomics/cparser.pyx":333
 *             in_labels = X not in _labels
 *             if ((not is_mod) and in_labels) or\
 *                 not ((mod+X in _labels) or\             # <<<<<<<<<<<<<<
 *                 (X in _labels and (mod in _labels or allow_unknown_modifications))):
 *                 raise PyteomicsError('Unknown label: {}'.format(sgroup))
 */
      __pyx_t_2 = PyNumber_Add(__pyx_v_mod, __pyx_v_X); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 333, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_3 = (__Pyx_PySequence_ContainsTF(__pyx_t_2, __pyx_v__labels, Py_EQ)); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 333, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_9 = (__pyx_t_3 != 0);
      if (!__pyx_t_9) {
      } else {
        __pyx_t_14 = __pyx_t_9;
        goto __pyx_L40_bool_binop_done;
      }

      /* "pyteomics/cparser.pyx":334
 *             if ((not is_mod) and in_labels) or\
 *                 not ((mod+X in _labels) or\
 *                 (X in _labels and (mod in _labels or allow_unknown_modifications))):             # <<<<<<<<<<<<<<
 *                 raise PyteomicsError('Unknown label: {}'.format(sgroup))
 * 
 */
      __pyx_t_9 = (__Pyx_PySequence_ContainsTF(__pyx_v_X, __pyx_v__labels, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 334, __pyx_L1_error)
      __pyx_t_3 = (__pyx_t_9 != 0);
      if (__pyx_t_3) {
      } else {
        __pyx_t_14 = __pyx_t_3;
        goto __pyx_L40_bool_binop_done;
      }
      __pyx_t_3 = (__Pyx_PySequence_ContainsTF(__pyx_v_mod, __pyx_v__labels, Py_EQ)); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 334, __pyx_L1_error)
      __pyx_t_9 = (__pyx_t_3 != 0);
      if (!__pyx_t_9) {
      } else {
        __pyx_t_14 = __pyx_t_9;
        goto __pyx_L40_bool_binop_done;
      }
      __pyx_t_9 = (__pyx_v_allow_unknown_modifications != 0);
      __pyx_t_14 = __pyx_t_9;
      __pyx_L40_bool_binop_done:;

      /* "pyteomics/cparser.pyx":333
 *             in_labels = X not in _labels
 *             if ((not is_mod) and in_labels) or\
 *                 not ((mod+X in _labels) or\             # <<<<<<<<<<<<<<
 *                 (X in _labels and (mod in _labels or allow_unknown_modifications))):
 *                 raise PyteomicsError('Unknown label: {}'.format(sgroup))
 */
      __pyx_t_9 = ((!__pyx_t_14) != 0);
      __pyx_t_4 = __pyx_t_9;
      __pyx_L37_bool_binop_done:;

      /* "pyteomics/cparser.pyx":332
 *             is_mod = PyObject_IsTrue(mod)
 *             in_labels = X not in _labels
 *             if ((not is_mod) and in_labels) or\             # <<<<<<<<<<<<<<
 *                 not ((mod+X in _labels) or\
 *                 (X in _labels and (mod in _labels or allow_unknown_modifications))):
 */
      if (__pyx_t_4) {

        /* "pyteomics/cparser.pyx":335
 *                 not ((mod+X in _labels) or\
 *                 (X in _labels and (mod in _labels or allow_unknown_modifications))):
 *                 raise PyteomicsError('Unknown label: {}'.format(sgroup))             # <<<<<<<<<<<<<<
 * 
 *     # Append terminal labels
 */
        __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_PyteomicsError); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 335, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_Unknown_label, __pyx_n_s_format); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 335, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_10 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_10)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_10);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
          }
        }
        if (!__pyx_t_10) {
          __pyx_t_5 = __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_v_sgroup); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 335, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
        } else {
          #if CYTHON_FAST_PYCALL
          if (PyFunction_Check(__pyx_t_6)) {
            PyObject *__pyx_temp[2] = {__pyx_t_10, __pyx_v_sgroup};
            __pyx_t_5 = __Pyx_PyFunction_FastCall(__pyx_t_6, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 335, __pyx_L1_error)
            __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
            __Pyx_GOTREF(__pyx_t_5);
          } else
          #endif
          #if CYTHON_FAST_PYCCALL
          if (__Pyx_PyFastCFunction_Check(__pyx_t_6)) {
            PyObject *__pyx_temp[2] = {__pyx_t_10, __pyx_v_sgroup};
            __pyx_t_5 = __Pyx_PyCFunction_FastCall(__pyx_t_6, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 335, __pyx_L1_error)
            __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
            __Pyx_GOTREF(__pyx_t_5);
          } else
          #endif
          {
            __pyx_t_7 = PyTuple_New(1+1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 335, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_7);
            __Pyx_GIVEREF(__pyx_t_10); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_10); __pyx_t_10 = NULL;
            __Pyx_INCREF(__pyx_v_sgroup);
            __Pyx_GIVEREF(__pyx_v_sgroup);
            PyTuple_SET_ITEM(__pyx_t_7, 0+1, __pyx_v_sgroup);
            __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_7, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 335, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_5);
            __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          }
        }
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __pyx_t_6 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
          __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_1);
          if (likely(__pyx_t_6)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
            __Pyx_INCREF(__pyx_t_6);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_1, function);
          }
        }
        if (!__pyx_t_6) {
          __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 335, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else {
          #if CYTHON_FAST_PYCALL
          if (PyFunction_Check(__pyx_t_1)) {
            PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_t_5};
            __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_1, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 335, __pyx_L1_error)
            __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
            __Pyx_GOTREF(__pyx_t_2);
            __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          } else
          #endif
          #if CYTHON_FAST_PYCCALL
          if (__Pyx_PyFastCFunction_Check(__pyx_t_1)) {
            PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_t_5};
            __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_1, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 335, __pyx_L1_error)
            __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
            __Pyx_GOTREF(__pyx_t_2);
            __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          } else
          #endif
          {
            __pyx_t_7 = PyTuple_New(1+1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 335, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_7);
            __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_6); __pyx_t_6 = NULL;
            __Pyx_GIVEREF(__pyx_t_5);
            PyTuple_SET_ITEM(__pyx_t_7, 0+1, __pyx_t_5);
            __pyx_t_5 = 0;
            __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_7, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 335, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_2);
            __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          }
        }
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_Raise(__pyx_t_2, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __PYX_ERR(0, 335, __pyx_L1_error)

        /* "pyteomics/cparser.pyx":332
 *             is_mod = PyObject_IsTrue(mod)
 *             in_labels = X not in _labels
 *             if ((not is_mod) and in_labels) or\             # <<<<<<<<<<<<<<
 *                 not ((mod+X in _labels) or\
 *                 (X in _labels and (mod in _labels or allow_unknown_modifications))):
 */
      }
    }
  }
  __pyx_L22:;

  /* "pyteomics/cparser.pyx":338
 * 
 *     # Append terminal labels
 *     if show_unmodified_termini or nterm != std_nterm:             # <<<<<<<<<<<<<<
 *         if split:
 *             temp = <tuple>PyList_GET_ITEM(parsed_sequence, 0)
 */
  __pyx_t_9 = (__pyx_v_show_unmodified_termini != 0);
  if (!__pyx_t_9) {
  } else {
    __pyx_t_4 = __pyx_t_9;
    goto __pyx_L45_bool_binop_done;
  }
  __pyx_t_9 = (__Pyx_PyString_Equals(__pyx_v_nterm, __pyx_v_9pyteomics_7cparser_std_nterm, Py_NE)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 338, __pyx_L1_error)
  __pyx_t_14 = (__pyx_t_9 != 0);
  __pyx_t_4 = __pyx_t_14;
  __pyx_L45_bool_binop_done:;
  if (__pyx_t_4) {

    /* "pyteomics/cparser.pyx":339
 *     # Append terminal labels
 *     if show_unmodified_termini or nterm != std_nterm:
 *         if split:             # <<<<<<<<<<<<<<
 *             temp = <tuple>PyList_GET_ITEM(parsed_sequence, 0)
 *             temp = (<str>nterm, ) + temp
 */
    __pyx_t_4 = (__pyx_v_split != 0);
    if (__pyx_t_4) {

      /* "pyteomics/cparser.pyx":340
 *     if show_unmodified_termini or nterm != std_nterm:
 *         if split:
 *             temp = <tuple>PyList_GET_ITEM(parsed_sequence, 0)             # <<<<<<<<<<<<<<
 *             temp = (<str>nterm, ) + temp
 *             Py_INCREF(temp)
 */
      __pyx_t_8 = PyList_GET_ITEM(__pyx_v_parsed_sequence, 0);
      __pyx_t_2 = ((PyObject *)__pyx_t_8);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_DECREF_SET(__pyx_v_temp, ((PyObject*)__pyx_t_2));
      __pyx_t_2 = 0;

      /* "pyteomics/cparser.pyx":341
 *         if split:
 *             temp = <tuple>PyList_GET_ITEM(parsed_sequence, 0)
 *             temp = (<str>nterm, ) + temp             # <<<<<<<<<<<<<<
 *             Py_INCREF(temp)
 *             PyList_SET_ITEM(<list>parsed_sequence, 0, tuple(temp))
 */
      __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 341, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_INCREF(((PyObject*)__pyx_v_nterm));
      __Pyx_GIVEREF(__pyx_v_nterm);
      PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_nterm);
      __pyx_t_1 = PyNumber_Add(__pyx_t_2, __pyx_v_temp); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 341, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF_SET(__pyx_v_temp, ((PyObject*)__pyx_t_1));
      __pyx_t_1 = 0;

      /* "pyteomics/cparser.pyx":342
 *             temp = <tuple>PyList_GET_ITEM(parsed_sequence, 0)
 *             temp = (<str>nterm, ) + temp
 *             Py_INCREF(temp)             # <<<<<<<<<<<<<<
 *             PyList_SET_ITEM(<list>parsed_sequence, 0, tuple(temp))
 *         else:
 */
      Py_INCREF(__pyx_v_temp);

      /* "pyteomics/cparser.pyx":343
 *             temp = (<str>nterm, ) + temp
 *             Py_INCREF(temp)
 *             PyList_SET_ITEM(<list>parsed_sequence, 0, tuple(temp))             # <<<<<<<<<<<<<<
 *         else:
 *             PyList_Insert(parsed_sequence, 0, nterm)
 */
      PyList_SET_ITEM(__pyx_v_parsed_sequence, 0, __pyx_v_temp);

      /* "pyteomics/cparser.pyx":339
 *     # Append terminal labels
 *     if show_unmodified_termini or nterm != std_nterm:
 *         if split:             # <<<<<<<<<<<<<<
 *             temp = <tuple>PyList_GET_ITEM(parsed_sequence, 0)
 *             temp = (<str>nterm, ) + temp
 */
      goto __pyx_L47;
    }

    /* "pyteomics/cparser.pyx":345
 *             PyList_SET_ITEM(<list>parsed_sequence, 0, tuple(temp))
 *         else:
 *             PyList_Insert(parsed_sequence, 0, nterm)             # <<<<<<<<<<<<<<
 *     if show_unmodified_termini or cterm != std_cterm:
 *         if split:
 */
    /*else*/ {
      __pyx_t_15 = PyList_Insert(__pyx_v_parsed_sequence, 0, __pyx_v_nterm); if (unlikely(__pyx_t_15 == -1)) __PYX_ERR(0, 345, __pyx_L1_error)
    }
    __pyx_L47:;

    /* "pyteomics/cparser.pyx":338
 * 
 *     # Append terminal labels
 *     if show_unmodified_termini or nterm != std_nterm:             # <<<<<<<<<<<<<<
 *         if split:
 *             temp = <tuple>PyList_GET_ITEM(parsed_sequence, 0)
 */
  }

  /* "pyteomics/cparser.pyx":346
 *         else:
 *             PyList_Insert(parsed_sequence, 0, nterm)
 *     if show_unmodified_termini or cterm != std_cterm:             # <<<<<<<<<<<<<<
 *         if split:
 *             temp = <tuple>PyList_GET_ITEM(parsed_sequence, len(parsed_sequence) - 1)
 */
  __pyx_t_14 = (__pyx_v_show_unmodified_termini != 0);
  if (!__pyx_t_14) {
  } else {
    __pyx_t_4 = __pyx_t_14;
    goto __pyx_L49_bool_binop_done;
  }
  __pyx_t_14 = (__Pyx_PyString_Equals(__pyx_v_cterm, __pyx_v_9pyteomics_7cparser_std_cterm, Py_NE)); if (unlikely(__pyx_t_14 < 0)) __PYX_ERR(0, 346, __pyx_L1_error)
  __pyx_t_9 = (__pyx_t_14 != 0);
  __pyx_t_4 = __pyx_t_9;
  __pyx_L49_bool_binop_done:;
  if (__pyx_t_4) {

    /* "pyteomics/cparser.pyx":347
 *             PyList_Insert(parsed_sequence, 0, nterm)
 *     if show_unmodified_termini or cterm != std_cterm:
 *         if split:             # <<<<<<<<<<<<<<
 *             temp = <tuple>PyList_GET_ITEM(parsed_sequence, len(parsed_sequence) - 1)
 *             temp = temp + (cterm,)
 */
    __pyx_t_4 = (__pyx_v_split != 0);
    if (__pyx_t_4) {

      /* "pyteomics/cparser.pyx":348
 *     if show_unmodified_termini or cterm != std_cterm:
 *         if split:
 *             temp = <tuple>PyList_GET_ITEM(parsed_sequence, len(parsed_sequence) - 1)             # <<<<<<<<<<<<<<
 *             temp = temp + (cterm,)
 *             Py_INCREF(temp)
 */
      if (unlikely(__pyx_v_parsed_sequence == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
        __PYX_ERR(0, 348, __pyx_L1_error)
      }
      __pyx_t_11 = PyList_GET_SIZE(__pyx_v_parsed_sequence); if (unlikely(__pyx_t_11 == -1)) __PYX_ERR(0, 348, __pyx_L1_error)
      __pyx_t_8 = PyList_GET_ITEM(__pyx_v_parsed_sequence, (__pyx_t_11 - 1));
      __pyx_t_1 = ((PyObject *)__pyx_t_8);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_DECREF_SET(__pyx_v_temp, ((PyObject*)__pyx_t_1));
      __pyx_t_1 = 0;

      /* "pyteomics/cparser.pyx":349
 *         if split:
 *             temp = <tuple>PyList_GET_ITEM(parsed_sequence, len(parsed_sequence) - 1)
 *             temp = temp + (cterm,)             # <<<<<<<<<<<<<<
 *             Py_INCREF(temp)
 *             PyList_SET_ITEM(parsed_sequence, len(parsed_sequence) - 1, temp)
 */
      __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 349, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_INCREF(__pyx_v_cterm);
      __Pyx_GIVEREF(__pyx_v_cterm);
      PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_cterm);
      __pyx_t_2 = PyNumber_Add(__pyx_v_temp, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 349, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF_SET(__pyx_v_temp, ((PyObject*)__pyx_t_2));
      __pyx_t_2 = 0;

      /* "pyteomics/cparser.pyx":350
 *             temp = <tuple>PyList_GET_ITEM(parsed_sequence, len(parsed_sequence) - 1)
 *             temp = temp + (cterm,)
 *             Py_INCREF(temp)             # <<<<<<<<<<<<<<
 *             PyList_SET_ITEM(parsed_sequence, len(parsed_sequence) - 1, temp)
 *         else:
 */
      Py_INCREF(__pyx_v_temp);

      /* "pyteomics/cparser.pyx":351
 *             temp = temp + (cterm,)
 *             Py_INCREF(temp)
 *             PyList_SET_ITEM(parsed_sequence, len(parsed_sequence) - 1, temp)             # <<<<<<<<<<<<<<
 *         else:
 *             PyList_Append(parsed_sequence, cterm)
 */
      if (unlikely(__pyx_v_parsed_sequence == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
        __PYX_ERR(0, 351, __pyx_L1_error)
      }
      __pyx_t_11 = PyList_GET_SIZE(__pyx_v_parsed_sequence); if (unlikely(__pyx_t_11 == -1)) __PYX_ERR(0, 351, __pyx_L1_error)
      PyList_SET_ITEM(__pyx_v_parsed_sequence, (__pyx_t_11 - 1), __pyx_v_temp);

      /* "pyteomics/cparser.pyx":347
 *             PyList_Insert(parsed_sequence, 0, nterm)
 *     if show_unmodified_termini or cterm != std_cterm:
 *         if split:             # <<<<<<<<<<<<<<
 *             temp = <tuple>PyList_GET_ITEM(parsed_sequence, len(parsed_sequence) - 1)
 *             temp = temp + (cterm,)
 */
      goto __pyx_L51;
    }

    /* "pyteomics/cparser.pyx":353
 *             PyList_SET_ITEM(parsed_sequence, len(parsed_sequence) - 1, temp)
 *         else:
 *             PyList_Append(parsed_sequence, cterm)             # <<<<<<<<<<<<<<
 * 
 * 
 */
    /*else*/ {
      __pyx_t_15 = PyList_Append(__pyx_v_parsed_sequence, __pyx_v_cterm); if (unlikely(__pyx_t_15 == -1)) __PYX_ERR(0, 353, __pyx_L1_error)
    }
    __pyx_L51:;

    /* "pyteomics/cparser.pyx":346
 *         else:
 *             PyList_Insert(parsed_sequence, 0, nterm)
 *     if show_unmodified_termini or cterm != std_cterm:             # <<<<<<<<<<<<<<
 *         if split:
 *             temp = <tuple>PyList_GET_ITEM(parsed_sequence, len(parsed_sequence) - 1)
 */
  }

  /* "pyteomics/cparser.pyx":356
 * 
 * 
 *     return parsed_sequence             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_parsed_sequence);
  __pyx_r = __pyx_v_parsed_sequence;
  goto __pyx_L0;

  /* "pyteomics/cparser.pyx":199
 * 
 * 
 * cpdef list parse(str sequence, bint show_unmodified_termini=0, bint split=0,             # <<<<<<<<<<<<<<
 *                  bint allow_unknown_modifications=0, object labels=std_labels):
 *     """Parse a sequence string written in modX notation into a list of
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("pyteomics.cparser.parse", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_match);
  __Pyx_XDECREF(__pyx_v_temp);
  __Pyx_XDECREF(__pyx_v_n);
  __Pyx_XDECREF(__pyx_v_body);
  __Pyx_XDECREF(__pyx_v_c);
  __Pyx_XDECREF(__pyx_v_nterm);
  __Pyx_XDECREF(__pyx_v_cterm);
  __Pyx_XDECREF(__pyx_v_mod);
  __Pyx_XDECREF(__pyx_v_X);
  __Pyx_XDECREF(__pyx_v_parsed_sequence);
  __Pyx_XDECREF(__pyx_v_tgroup);
  __Pyx_XDECREF(__pyx_v_sgroup);
  __Pyx_XDECREF(__pyx_v__labels);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_5parse(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_9pyteomics_7cparser_4parse[] = "parse(str sequence, bool show_unmodified_termini=0, bool split=0, bool allow_unknown_modifications=0, labels=std_labels) -> list\nParse a sequence string written in modX notation into a list of\n    labels or (if `split` argument is :py:const:`True`) into a list of\n    tuples representing amino acid residues and their modifications.\n\n    Parameters\n    ----------\n    sequence : str\n        The sequence of a polypeptide.\n    show_unmodified_termini : bool, optional\n        If :py:const:`True` then the unmodified N- and C-termini are explicitly\n        shown in the returned list. Default value is :py:const:`False`.\n    split : bool, optional\n        If :py:const:`True` then the result will be a list of tuples with 1 to 4\n        elements: terminal modification, modification, residue. Default value is\n        :py:const:`False`.\n    allow_unknown_modifications : bool, optional\n        If :py:const:`True` then do not raise an exception when an unknown\n        modification of a known amino acid residue is found in the sequence.\n        This also includes terminal groups.\n        Default value is :py:const:`False`.\n\n        .. note::\n            Since version 2.5, this parameter has effect only if `labels`\n            are provided.\n    labels : container, optional\n        A container of allowed labels for amino acids,\n        modifications and terminal modifications.\n        If not provided, no checks will be done.\n        Separate labels for modifications (such as 'p' or 'ox')\n        can be supplied, which means they are applicable to all residues.\n\n        .. warning::\n            If `show_unmodified_termini` is set to :py:const:`True`, standard\n            terminal groups need to be present in `labels`.\n\n        .. warning::\n            Avoid using sequences with only one terminal group, as they are\n            ambiguous. If you provide one, `labels` (or :py:const:`std_labels`)\n            will be used to resolve the ambiguity.\n""\n    Returns\n    -------\n    out : list\n        List of tuples with labels of modifications and amino acid residues.\n\n    Examples\n    --------\n    >>> parse('PEPTIDE', split=True)\n    [('P',), ('E',), ('P',), ('T',), ('I',), ('D',), ('E',)]\n    >>> parse('H-PEPTIDE')\n    ['P', 'E', 'P', 'T', 'I', 'D', 'E']\n    >>> parse('PEPTIDE', show_unmodified_termini=True)\n    ['H-', 'P', 'E', 'P', 'T', 'I', 'D', 'E', '-OH']\n    >>> parse('TEpSToxM', labels=std_labels + ['pS', 'oxM'])\n    ['T', 'E', 'pS', 'T', 'oxM']\n    >>> parse('zPEPzTIDzE', True, True, labels=std_labels+['z'])\n    [('H-', 'z', 'P'), ('E',), ('P',), ('z', 'T'), ('I',), ('D',), ('z', 'E', '-OH')]\n    ";
static PyObject *__pyx_pw_9pyteomics_7cparser_5parse(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_sequence = 0;
  int __pyx_v_show_unmodified_termini;
  int __pyx_v_split;
  int __pyx_v_allow_unknown_modifications;
  PyObject *__pyx_v_labels = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("parse (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_sequence,&__pyx_n_s_show_unmodified_termini,&__pyx_n_s_split,&__pyx_n_s_allow_unknown_modifications,&__pyx_n_s_labels,0};
    PyObject* values[5] = {0,0,0,0,0};
    values[4] = __pyx_k__4;
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_sequence)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_show_unmodified_termini);
          if (value) { values[1] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_split);
          if (value) { values[2] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_allow_unknown_modifications);
          if (value) { values[3] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_labels);
          if (value) { values[4] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "parse") < 0)) __PYX_ERR(0, 199, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_sequence = ((PyObject*)values[0]);
    if (values[1]) {
      __pyx_v_show_unmodified_termini = __Pyx_PyObject_IsTrue(values[1]); if (unlikely((__pyx_v_show_unmodified_termini == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 199, __pyx_L3_error)
    } else {
      __pyx_v_show_unmodified_termini = ((int)0);
    }
    if (values[2]) {
      __pyx_v_split = __Pyx_PyObject_IsTrue(values[2]); if (unlikely((__pyx_v_split == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 199, __pyx_L3_error)
    } else {
      __pyx_v_split = ((int)0);
    }
    if (values[3]) {
      __pyx_v_allow_unknown_modifications = __Pyx_PyObject_IsTrue(values[3]); if (unlikely((__pyx_v_allow_unknown_modifications == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 200, __pyx_L3_error)
    } else {
      __pyx_v_allow_unknown_modifications = ((int)0);
    }
    __pyx_v_labels = values[4];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("parse", 0, 1, 5, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 199, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyteomics.cparser.parse", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_sequence), (&PyString_Type), 1, "sequence", 1))) __PYX_ERR(0, 199, __pyx_L1_error)
  __pyx_r = __pyx_pf_9pyteomics_7cparser_4parse(__pyx_self, __pyx_v_sequence, __pyx_v_show_unmodified_termini, __pyx_v_split, __pyx_v_allow_unknown_modifications, __pyx_v_labels);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_4parse(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_sequence, int __pyx_v_show_unmodified_termini, int __pyx_v_split, int __pyx_v_allow_unknown_modifications, PyObject *__pyx_v_labels) {
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_9pyteomics_7cparser_parse __pyx_t_2;
  __Pyx_RefNannySetupContext("parse", 0);
  __Pyx_TraceCall("parse (wrapper)", __pyx_f[0], 199, 0, __PYX_ERR(0, 199, __pyx_L1_error));
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 4;
  __pyx_t_2.show_unmodified_termini = __pyx_v_show_unmodified_termini;
  __pyx_t_2.split = __pyx_v_split;
  __pyx_t_2.allow_unknown_modifications = __pyx_v_allow_unknown_modifications;
  __pyx_t_2.labels = __pyx_v_labels;
  __pyx_t_1 = __pyx_f_9pyteomics_7cparser_parse(__pyx_v_sequence, 0, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 199, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyteomics.cparser.parse", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":359
 * 
 * 
 * cdef str tostring(object parsed_sequence, bint show_unmodified_termini=True):             # <<<<<<<<<<<<<<
 *     """Create a string from a parsed sequence.
 * 
 */

static PyObject *__pyx_f_9pyteomics_7cparser_tostring(PyObject *__pyx_v_parsed_sequence, struct __pyx_opt_args_9pyteomics_7cparser_tostring *__pyx_optional_args) {
  int __pyx_v_show_unmodified_termini = ((int)1);
  PyObject *__pyx_v_labels = 0;
  PyObject *__pyx_v_group_l = 0;
  PyObject *__pyx_v_group = 0;
  PyObject *__pyx_v_remove_fn = 0;
  int __pyx_v_is_term;
  Py_ssize_t __pyx_v_i;
  Py_ssize_t __pyx_v_n;
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  __Pyx_RefNannySetupContext("tostring", 0);
  __Pyx_TraceCall("tostring", __pyx_f[0], 359, 0, __PYX_ERR(0, 359, __pyx_L1_error));
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_show_unmodified_termini = __pyx_optional_args->show_unmodified_termini;
    }
  }

  /* "pyteomics/cparser.pyx":383
 *         bint is_term
 *         Py_ssize_t i, n
 *     n = len(parsed_sequence)             # <<<<<<<<<<<<<<
 *     labels = []
 *     for i in range(n):
 */
  __pyx_t_1 = PyObject_Length(__pyx_v_parsed_sequence); if (unlikely(__pyx_t_1 == -1)) __PYX_ERR(0, 383, __pyx_L1_error)
  __pyx_v_n = __pyx_t_1;

  /* "pyteomics/cparser.pyx":384
 *         Py_ssize_t i, n
 *     n = len(parsed_sequence)
 *     labels = []             # <<<<<<<<<<<<<<
 *     for i in range(n):
 *         group = parsed_sequence[i]
 */
  __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 384, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_labels = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "pyteomics/cparser.pyx":385
 *     n = len(parsed_sequence)
 *     labels = []
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         group = parsed_sequence[i]
 *         if isinstance(group, str):
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_1; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "pyteomics/cparser.pyx":386
 *     labels = []
 *     for i in range(n):
 *         group = parsed_sequence[i]             # <<<<<<<<<<<<<<
 *         if isinstance(group, str):
 *             is_term = group == std_cterm or group == std_nterm
 */
    __pyx_t_2 = __Pyx_GetItemInt(__pyx_v_parsed_sequence, __pyx_v_i, Py_ssize_t, 1, PyInt_FromSsize_t, 0, 1, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 386, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_XDECREF_SET(__pyx_v_group, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "pyteomics/cparser.pyx":387
 *     for i in range(n):
 *         group = parsed_sequence[i]
 *         if isinstance(group, str):             # <<<<<<<<<<<<<<
 *             is_term = group == std_cterm or group == std_nterm
 *             if is_term or show_unmodified_termini:
 */
    __pyx_t_4 = PyString_Check(__pyx_v_group); 
    __pyx_t_5 = (__pyx_t_4 != 0);
    if (__pyx_t_5) {

      /* "pyteomics/cparser.pyx":388
 *         group = parsed_sequence[i]
 *         if isinstance(group, str):
 *             is_term = group == std_cterm or group == std_nterm             # <<<<<<<<<<<<<<
 *             if is_term or show_unmodified_termini:
 *                 labels.append(group)
 */
      __pyx_t_4 = (__Pyx_PyString_Equals(__pyx_v_group, __pyx_v_9pyteomics_7cparser_std_cterm, Py_EQ)); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 388, __pyx_L1_error)
      if (!__pyx_t_4) {
      } else {
        __pyx_t_5 = __pyx_t_4;
        goto __pyx_L6_bool_binop_done;
      }
      __pyx_t_4 = (__Pyx_PyString_Equals(__pyx_v_group, __pyx_v_9pyteomics_7cparser_std_nterm, Py_EQ)); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 388, __pyx_L1_error)
      __pyx_t_5 = __pyx_t_4;
      __pyx_L6_bool_binop_done:;
      __pyx_v_is_term = __pyx_t_5;

      /* "pyteomics/cparser.pyx":389
 *         if isinstance(group, str):
 *             is_term = group == std_cterm or group == std_nterm
 *             if is_term or show_unmodified_termini:             # <<<<<<<<<<<<<<
 *                 labels.append(group)
 *         else: # treat `group` as a tuple
 */
      __pyx_t_4 = (__pyx_v_is_term != 0);
      if (!__pyx_t_4) {
      } else {
        __pyx_t_5 = __pyx_t_4;
        goto __pyx_L9_bool_binop_done;
      }
      __pyx_t_4 = (__pyx_v_show_unmodified_termini != 0);
      __pyx_t_5 = __pyx_t_4;
      __pyx_L9_bool_binop_done:;
      if (__pyx_t_5) {

        /* "pyteomics/cparser.pyx":390
 *             is_term = group == std_cterm or group == std_nterm
 *             if is_term or show_unmodified_termini:
 *                 labels.append(group)             # <<<<<<<<<<<<<<
 *         else: # treat `group` as a tuple
 *             group_l = list(group)
 */
        __pyx_t_6 = __Pyx_PyList_Append(__pyx_v_labels, __pyx_v_group); if (unlikely(__pyx_t_6 == -1)) __PYX_ERR(0, 390, __pyx_L1_error)

        /* "pyteomics/cparser.pyx":389
 *         if isinstance(group, str):
 *             is_term = group == std_cterm or group == std_nterm
 *             if is_term or show_unmodified_termini:             # <<<<<<<<<<<<<<
 *                 labels.append(group)
 *         else: # treat `group` as a tuple
 */
      }

      /* "pyteomics/cparser.pyx":387
 *     for i in range(n):
 *         group = parsed_sequence[i]
 *         if isinstance(group, str):             # <<<<<<<<<<<<<<
 *             is_term = group == std_cterm or group == std_nterm
 *             if is_term or show_unmodified_termini:
 */
      goto __pyx_L5;
    }

    /* "pyteomics/cparser.pyx":392
 *                 labels.append(group)
 *         else: # treat `group` as a tuple
 *             group_l = list(group)             # <<<<<<<<<<<<<<
 *             remove_fn = group_l.remove
 *             if not show_unmodified_termini:
 */
    /*else*/ {
      __pyx_t_2 = PySequence_List(__pyx_v_group); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 392, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_XDECREF_SET(__pyx_v_group_l, ((PyObject*)__pyx_t_2));
      __pyx_t_2 = 0;

      /* "pyteomics/cparser.pyx":393
 *         else: # treat `group` as a tuple
 *             group_l = list(group)
 *             remove_fn = group_l.remove             # <<<<<<<<<<<<<<
 *             if not show_unmodified_termini:
 *                 if std_cterm in group_l:
 */
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_group_l, __pyx_n_s_remove); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 393, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_XDECREF_SET(__pyx_v_remove_fn, __pyx_t_2);
      __pyx_t_2 = 0;

      /* "pyteomics/cparser.pyx":394
 *             group_l = list(group)
 *             remove_fn = group_l.remove
 *             if not show_unmodified_termini:             # <<<<<<<<<<<<<<
 *                 if std_cterm in group_l:
 *                     remove_fn(std_cterm)
 */
      __pyx_t_5 = ((!(__pyx_v_show_unmodified_termini != 0)) != 0);
      if (__pyx_t_5) {

        /* "pyteomics/cparser.pyx":395
 *             remove_fn = group_l.remove
 *             if not show_unmodified_termini:
 *                 if std_cterm in group_l:             # <<<<<<<<<<<<<<
 *                     remove_fn(std_cterm)
 *                 if std_nterm in group_l:
 */
        __pyx_t_5 = (__Pyx_PySequence_ContainsTF(__pyx_v_9pyteomics_7cparser_std_cterm, __pyx_v_group_l, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 395, __pyx_L1_error)
        __pyx_t_4 = (__pyx_t_5 != 0);
        if (__pyx_t_4) {

          /* "pyteomics/cparser.pyx":396
 *             if not show_unmodified_termini:
 *                 if std_cterm in group_l:
 *                     remove_fn(std_cterm)             # <<<<<<<<<<<<<<
 *                 if std_nterm in group_l:
 *                     remove_fn(std_nterm)
 */
          __Pyx_INCREF(__pyx_v_remove_fn);
          __pyx_t_7 = __pyx_v_remove_fn; __pyx_t_8 = NULL;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
            __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
            if (likely(__pyx_t_8)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
              __Pyx_INCREF(__pyx_t_8);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_7, function);
            }
          }
          if (!__pyx_t_8) {
            __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_v_9pyteomics_7cparser_std_cterm); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 396, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_2);
          } else {
            #if CYTHON_FAST_PYCALL
            if (PyFunction_Check(__pyx_t_7)) {
              PyObject *__pyx_temp[2] = {__pyx_t_8, __pyx_v_9pyteomics_7cparser_std_cterm};
              __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_7, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 396, __pyx_L1_error)
              __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
              __Pyx_GOTREF(__pyx_t_2);
            } else
            #endif
            #if CYTHON_FAST_PYCCALL
            if (__Pyx_PyFastCFunction_Check(__pyx_t_7)) {
              PyObject *__pyx_temp[2] = {__pyx_t_8, __pyx_v_9pyteomics_7cparser_std_cterm};
              __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_7, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 396, __pyx_L1_error)
              __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
              __Pyx_GOTREF(__pyx_t_2);
            } else
            #endif
            {
              __pyx_t_9 = PyTuple_New(1+1); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 396, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_9);
              __Pyx_GIVEREF(__pyx_t_8); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_8); __pyx_t_8 = NULL;
              __Pyx_INCREF(__pyx_v_9pyteomics_7cparser_std_cterm);
              __Pyx_GIVEREF(__pyx_v_9pyteomics_7cparser_std_cterm);
              PyTuple_SET_ITEM(__pyx_t_9, 0+1, __pyx_v_9pyteomics_7cparser_std_cterm);
              __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_9, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 396, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_2);
              __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
            }
          }
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

          /* "pyteomics/cparser.pyx":395
 *             remove_fn = group_l.remove
 *             if not show_unmodified_termini:
 *                 if std_cterm in group_l:             # <<<<<<<<<<<<<<
 *                     remove_fn(std_cterm)
 *                 if std_nterm in group_l:
 */
        }

        /* "pyteomics/cparser.pyx":397
 *                 if std_cterm in group_l:
 *                     remove_fn(std_cterm)
 *                 if std_nterm in group_l:             # <<<<<<<<<<<<<<
 *                     remove_fn(std_nterm)
 *             labels.append(''.join(group_l))
 */
        __pyx_t_4 = (__Pyx_PySequence_ContainsTF(__pyx_v_9pyteomics_7cparser_std_nterm, __pyx_v_group_l, Py_EQ)); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 397, __pyx_L1_error)
        __pyx_t_5 = (__pyx_t_4 != 0);
        if (__pyx_t_5) {

          /* "pyteomics/cparser.pyx":398
 *                     remove_fn(std_cterm)
 *                 if std_nterm in group_l:
 *                     remove_fn(std_nterm)             # <<<<<<<<<<<<<<
 *             labels.append(''.join(group_l))
 *     return ''.join(labels)
 */
          __Pyx_INCREF(__pyx_v_remove_fn);
          __pyx_t_7 = __pyx_v_remove_fn; __pyx_t_9 = NULL;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
            __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_7);
            if (likely(__pyx_t_9)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
              __Pyx_INCREF(__pyx_t_9);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_7, function);
            }
          }
          if (!__pyx_t_9) {
            __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_v_9pyteomics_7cparser_std_nterm); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 398, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_2);
          } else {
            #if CYTHON_FAST_PYCALL
            if (PyFunction_Check(__pyx_t_7)) {
              PyObject *__pyx_temp[2] = {__pyx_t_9, __pyx_v_9pyteomics_7cparser_std_nterm};
              __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_7, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 398, __pyx_L1_error)
              __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
              __Pyx_GOTREF(__pyx_t_2);
            } else
            #endif
            #if CYTHON_FAST_PYCCALL
            if (__Pyx_PyFastCFunction_Check(__pyx_t_7)) {
              PyObject *__pyx_temp[2] = {__pyx_t_9, __pyx_v_9pyteomics_7cparser_std_nterm};
              __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_7, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 398, __pyx_L1_error)
              __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
              __Pyx_GOTREF(__pyx_t_2);
            } else
            #endif
            {
              __pyx_t_8 = PyTuple_New(1+1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 398, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_8);
              __Pyx_GIVEREF(__pyx_t_9); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_9); __pyx_t_9 = NULL;
              __Pyx_INCREF(__pyx_v_9pyteomics_7cparser_std_nterm);
              __Pyx_GIVEREF(__pyx_v_9pyteomics_7cparser_std_nterm);
              PyTuple_SET_ITEM(__pyx_t_8, 0+1, __pyx_v_9pyteomics_7cparser_std_nterm);
              __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_8, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 398, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_2);
              __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
            }
          }
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

          /* "pyteomics/cparser.pyx":397
 *                 if std_cterm in group_l:
 *                     remove_fn(std_cterm)
 *                 if std_nterm in group_l:             # <<<<<<<<<<<<<<
 *                     remove_fn(std_nterm)
 *             labels.append(''.join(group_l))
 */
        }

        /* "pyteomics/cparser.pyx":394
 *             group_l = list(group)
 *             remove_fn = group_l.remove
 *             if not show_unmodified_termini:             # <<<<<<<<<<<<<<
 *                 if std_cterm in group_l:
 *                     remove_fn(std_cterm)
 */
      }

      /* "pyteomics/cparser.pyx":399
 *                 if std_nterm in group_l:
 *                     remove_fn(std_nterm)
 *             labels.append(''.join(group_l))             # <<<<<<<<<<<<<<
 *     return ''.join(labels)
 * 
 */
      __pyx_t_2 = __Pyx_PyString_Join(__pyx_kp_s__5, __pyx_v_group_l); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 399, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_6 = __Pyx_PyList_Append(__pyx_v_labels, __pyx_t_2); if (unlikely(__pyx_t_6 == -1)) __PYX_ERR(0, 399, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    }
    __pyx_L5:;
  }

  /* "pyteomics/cparser.pyx":400
 *                     remove_fn(std_nterm)
 *             labels.append(''.join(group_l))
 *     return ''.join(labels)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyString_Join(__pyx_kp_s__5, __pyx_v_labels); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 400, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (!(likely(PyString_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_2)->tp_name), 0))) __PYX_ERR(0, 400, __pyx_L1_error)
  __pyx_r = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "pyteomics/cparser.pyx":359
 * 
 * 
 * cdef str tostring(object parsed_sequence, bint show_unmodified_termini=True):             # <<<<<<<<<<<<<<
 *     """Create a string from a parsed sequence.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("pyteomics.cparser.tostring", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_labels);
  __Pyx_XDECREF(__pyx_v_group_l);
  __Pyx_XDECREF(__pyx_v_group);
  __Pyx_XDECREF(__pyx_v_remove_fn);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyteomics/cparser.pyx":403
 * 
 * 
 * cpdef dict amino_acid_composition(object sequence, bint show_unmodified_termini=0, bint term_aa=0,             # <<<<<<<<<<<<<<
 *                                   bint allow_unknown_modifications=0, object labels=std_labels):
 *     """Calculate amino acid composition of a polypeptide.
 */

static PyObject *__pyx_pw_9pyteomics_7cparser_7amino_acid_composition(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_9pyteomics_7cparser_amino_acid_composition(PyObject *__pyx_v_sequence, CYTHON_UNUSED int __pyx_skip_dispatch, struct __pyx_opt_args_9pyteomics_7cparser_amino_acid_composition *__pyx_optional_args) {
  int __pyx_v_show_unmodified_termini = ((int)0);
  int __pyx_v_term_aa = ((int)0);
  int __pyx_v_allow_unknown_modifications = ((int)0);
  PyObject *__pyx_v_labels = __pyx_k__6;
  PyObject *__pyx_v_parsed_sequence = 0;
  int __pyx_v_nterm_aa_position;
  int __pyx_v_cterm_aa_position;
  int __pyx_v_aa_count;
  PyObject *__pyx_v_aa_dict = 0;
  PyObject *__pyx_v_aa = 0;
  PyObject *__pyx_v__labels = 0;
  PyObject *__pyx_v_temp = 0;
  PyObject *__pyx_v_pvalue;
  Py_ssize_t __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  struct __pyx_opt_args_9pyteomics_7cparser_parse __pyx_t_4;
  PyObject *__pyx_t_5;
  int __pyx_t_6;
  struct __pyx_opt_args_9pyteomics_7cparser_tostring __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  Py_ssize_t __pyx_t_12;
  int __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  long __pyx_t_15;
  __Pyx_RefNannySetupContext("amino_acid_composition", 0);
  __Pyx_TraceCall("amino_acid_composition", __pyx_f[0], 403, 0, __PYX_ERR(0, 403, __pyx_L1_error));
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_show_unmodified_termini = __pyx_optional_args->show_unmodified_termini;
      if (__pyx_optional_args->__pyx_n > 1) {
        __pyx_v_term_aa = __pyx_optional_args->term_aa;
        if (__pyx_optional_args->__pyx_n > 2) {
          __pyx_v_allow_unknown_modifications = __pyx_optional_args->allow_unknown_modifications;
          if (__pyx_optional_args->__pyx_n > 3) {
            __pyx_v_labels = __pyx_optional_args->labels;
          }
        }
      }
    }
  }

  /* "pyteomics/cparser.pyx":452
 *         Py_ssize_t i
 * 
 *     _labels = interpolate_labels(labels)             # <<<<<<<<<<<<<<
 * 
 *     if isinstance(sequence, str):
 */
  __pyx_t_1 = __pyx_f_9pyteomics_7cparser_interpolate_labels(__pyx_v_labels); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 452, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v__labels = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyteomics/cparser.pyx":454
 *     _labels = interpolate_labels(labels)
 * 
 *     if isinstance(sequence, str):             # <<<<<<<<<<<<<<
 *         parsed_sequence = parse(<str>sequence, show_unmodified_termini, split=False,
 *                                 allow_unknown_modifications=allow_unknown_modifications,
 */
  __pyx_t_2 = PyString_Check(__pyx_v_sequence); 
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {

    /* "pyteomics/cparser.pyx":455
 * 
 *     if isinstance(sequence, str):
 *         parsed_sequence = parse(<str>sequence, show_unmodified_termini, split=False,             # <<<<<<<<<<<<<<
 *                                 allow_unknown_modifications=allow_unknown_modifications,
 *                                 labels=_labels)
 */
    __pyx_t_4.__pyx_n = 4;
    __pyx_t_4.show_unmodified_termini = __pyx_v_show_unmodified_termini;
    __pyx_t_4.split = 0;
    __pyx_t_4.allow_unknown_modifications = __pyx_v_allow_unknown_modifications;
    __pyx_t_4.labels = __pyx_v__labels;
    __pyx_t_1 = __pyx_f_9pyteomics_7cparser_parse(((PyObject*)__pyx_v_sequence), 0, &__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 455, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_v_parsed_sequence = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "pyteomics/cparser.pyx":454
 *     _labels = interpolate_labels(labels)
 * 
 *     if isinstance(sequence, str):             # <<<<<<<<<<<<<<
 *         parsed_sequence = parse(<str>sequence, show_unmodified_termini, split=False,
 *                                 allow_unknown_modifications=allow_unknown_modifications,
 */
    goto __pyx_L3;
  }

  /* "pyteomics/cparser.pyx":458
 *                                 allow_unknown_modifications=allow_unknown_modifications,
 *                                 labels=_labels)
 *     elif isinstance(sequence, list):             # <<<<<<<<<<<<<<
 *         if sequence and isinstance(<object>PyList_GetItem(sequence, 0), tuple):
 *             parsed_sequence = parse(tostring(sequence, True),
 */
  __pyx_t_3 = PyList_Check(__pyx_v_sequence); 
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {

    /* "pyteomics/cparser.pyx":459
 *                                 labels=_labels)
 *     elif isinstance(sequence, list):
 *         if sequence and isinstance(<object>PyList_GetItem(sequence, 0), tuple):             # <<<<<<<<<<<<<<
 *             parsed_sequence = parse(tostring(sequence, True),
 *                 show_unmodified_termini,
 */
    __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_v_sequence); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 459, __pyx_L1_error)
    if (__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L5_bool_binop_done;
    }
    __pyx_t_5 = PyList_GetItem(__pyx_v_sequence, 0); if (unlikely(__pyx_t_5 == NULL)) __PYX_ERR(0, 459, __pyx_L1_error)
    __pyx_t_3 = PyTuple_Check(((PyObject *)__pyx_t_5)); 
    __pyx_t_6 = (__pyx_t_3 != 0);
    __pyx_t_2 = __pyx_t_6;
    __pyx_L5_bool_binop_done:;
    if (__pyx_t_2) {

      /* "pyteomics/cparser.pyx":460
 *     elif isinstance(sequence, list):
 *         if sequence and isinstance(<object>PyList_GetItem(sequence, 0), tuple):
 *             parsed_sequence = parse(tostring(sequence, True),             # <<<<<<<<<<<<<<
 *                 show_unmodified_termini,
 *                 split=False,
 */
      __pyx_t_7.__pyx_n = 1;
      __pyx_t_7.show_unmodified_termini = 1;
      __pyx_t_1 = __pyx_f_9pyteomics_7cparser_tostring(__pyx_v_sequence, &__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 460, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);

      /* "pyteomics/cparser.pyx":464
 *                 split=False,
 *                 allow_unknown_modifications=allow_unknown_modifications,
 *                 labels=_labels)             # <<<<<<<<<<<<<<
 *         else:
 *             parsed_sequence = sequence
 */
      __pyx_t_4.__pyx_n = 4;
      __pyx_t_4.show_unmodified_termini = __pyx_v_show_unmodified_termini;
      __pyx_t_4.split = 0;
      __pyx_t_4.allow_unknown_modifications = __pyx_v_allow_unknown_modifications;
      __pyx_t_4.labels = __pyx_v__labels;
      __pyx_t_8 = __pyx_f_9pyteomics_7cparser_parse(((PyObject*)__pyx_t_1), 0, &__pyx_t_4); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 460, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_v_parsed_sequence = ((PyObject*)__pyx_t_8);
      __pyx_t_8 = 0;

      /* "pyteomics/cparser.pyx":459
 *                                 labels=_labels)
 *     elif isinstance(sequence, list):
 *         if sequence and isinstance(<object>PyList_GetItem(sequence, 0), tuple):             # <<<<<<<<<<<<<<
 *             parsed_sequence = parse(tostring(sequence, True),
 *                 show_unmodified_termini,
 */
      goto __pyx_L4;
    }

    /* "pyteomics/cparser.pyx":466
 *                 labels=_labels)
 *         else:
 *             parsed_sequence = sequence             # <<<<<<<<<<<<<<
 *     else:
 *         raise PyteomicsError('Unsupported type of a sequence.'
 */
    /*else*/ {
      if (!(likely(PyList_CheckExact(__pyx_v_sequence))||((__pyx_v_sequence) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_v_sequence)->tp_name), 0))) __PYX_ERR(0, 466, __pyx_L1_error)
      __pyx_t_8 = __pyx_v_sequence;
      __Pyx_INCREF(__pyx_t_8);
      __pyx_v_parsed_sequence = ((PyObject*)__pyx_t_8);
      __pyx_t_8 = 0;
    }
    __pyx_L4:;

    /* "pyteomics/cparser.pyx":458
 *                                 allow_unknown_modifications=allow_unknown_modifications,
 *                                 labels=_labels)
 *     elif isinstance(sequence, list):             # <<<<<<<<<<<<<<
 *         if sequence and isinstance(<object>PyList_GetItem(sequence, 0), tuple):
 *             parsed_sequence = parse(tostring(sequence, True),
 */
    goto __pyx_L3;
  }

  /* "pyteomics/cparser.pyx":468
 *             parsed_sequence = sequence
 *     else:
 *         raise PyteomicsError('Unsupported type of a sequence.'             # <<<<<<<<<<<<<<
 *                 'Must be str or list, not %s' % type(sequence))
 * 
 */
  /*else*/ {
    __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_PyteomicsError); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 468, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);

    /* "pyteomics/cparser.pyx":469
 *     else:
 *         raise PyteomicsError('Unsupported type of a sequence.'
 *                 'Must be str or list, not %s' % type(sequence))             # <<<<<<<<<<<<<<
 * 
 *     aa_dict = {}
 */
    __pyx_t_9 = __Pyx_PyString_Format(__pyx_kp_s_Unsupported_type_of_a_sequence_M, ((PyObject *)Py_TYPE(__pyx_v_sequence))); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 469, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_10 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_10)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
      }
    }
    if (!__pyx_t_10) {
      __pyx_t_8 = __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_9); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 468, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_GOTREF(__pyx_t_8);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_1)) {
        PyObject *__pyx_temp[2] = {__pyx_t_10, __pyx_t_9};
        __pyx_t_8 = __Pyx_PyFunction_FastCall(__pyx_t_1, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 468, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_1)) {
        PyObject *__pyx_temp[2] = {__pyx_t_10, __pyx_t_9};
        __pyx_t_8 = __Pyx_PyCFunction_FastCall(__pyx_t_1, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 468, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      } else
      #endif
      {
        __pyx_t_11 = PyTuple_New(1+1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 468, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_GIVEREF(__pyx_t_10); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_10); __pyx_t_10 = NULL;
        __Pyx_GIVEREF(__pyx_t_9);
        PyTuple_SET_ITEM(__pyx_t_11, 0+1, __pyx_t_9);
        __pyx_t_9 = 0;
        __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_11, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 468, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __PYX_ERR(0, 468, __pyx_L1_error)
  }
  __pyx_L3:;

  /* "pyteomics/cparser.pyx":471
 *                 'Must be str or list, not %s' % type(sequence))
 * 
 *     aa_dict = {}             # <<<<<<<<<<<<<<
 *     # Process terminal amino acids.
 *     if term_aa:
 */
  __pyx_t_8 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 471, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_v_aa_dict = ((PyObject*)__pyx_t_8);
  __pyx_t_8 = 0;

  /* "pyteomics/cparser.pyx":473
 *     aa_dict = {}
 *     # Process terminal amino acids.
 *     if term_aa:             # <<<<<<<<<<<<<<
 *         if is_term_mod(<object>PyList_GetItem(parsed_sequence, 0)):
 *             nterm_aa_position = 1
 */
  __pyx_t_2 = (__pyx_v_term_aa != 0);
  if (__pyx_t_2) {

    /* "pyteomics/cparser.pyx":474
 *     # Process terminal amino acids.
 *     if term_aa:
 *         if is_term_mod(<object>PyList_GetItem(parsed_sequence, 0)):             # <<<<<<<<<<<<<<
 *             nterm_aa_position = 1
 *         else:
 */
    __pyx_t_5 = PyList_GetItem(__pyx_v_parsed_sequence, 0); if (unlikely(__pyx_t_5 == NULL)) __PYX_ERR(0, 474, __pyx_L1_error)
    if (!(likely(PyString_CheckExact(((PyObject *)__pyx_t_5)))||((((PyObject *)__pyx_t_5)) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(((PyObject *)__pyx_t_5))->tp_name), 0))) __PYX_ERR(0, 474, __pyx_L1_error)
    __pyx_t_2 = (__pyx_f_9pyteomics_7cparser_is_term_mod(((PyObject*)((PyObject *)__pyx_t_5))) != 0);
    if (__pyx_t_2) {

      /* "pyteomics/cparser.pyx":475
 *     if term_aa:
 *         if is_term_mod(<object>PyList_GetItem(parsed_sequence, 0)):
 *             nterm_aa_position = 1             # <<<<<<<<<<<<<<
 *         else:
 *             nterm_aa_position = 0
 */
      __pyx_v_nterm_aa_position = 1;

      /* "pyteomics/cparser.pyx":474
 *     # Process terminal amino acids.
 *     if term_aa:
 *         if is_term_mod(<object>PyList_GetItem(parsed_sequence, 0)):             # <<<<<<<<<<<<<<
 *             nterm_aa_position = 1
 *         else:
 */
      goto __pyx_L8;
    }

    /* "pyteomics/cparser.pyx":477
 *             nterm_aa_position = 1
 *         else:
 *             nterm_aa_position = 0             # <<<<<<<<<<<<<<
 *         if is_term_mod(parsed_sequence[-1]):
 *             cterm_aa_position = PyList_Size(parsed_sequence) - 2
 */
    /*else*/ {
      __pyx_v_nterm_aa_position = 0;
    }
    __pyx_L8:;

    /* "pyteomics/cparser.pyx":478
 *         else:
 *             nterm_aa_position = 0
 *         if is_term_mod(parsed_sequence[-1]):             # <<<<<<<<<<<<<<
 *             cterm_aa_position = PyList_Size(parsed_sequence) - 2
 *         else:
 */
    if (unlikely(__pyx_v_parsed_sequence == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 478, __pyx_L1_error)
    }
    __pyx_t_8 = __Pyx_GetItemInt_List(__pyx_v_parsed_sequence, -1L, long, 1, __Pyx_PyInt_From_long, 1, 1, 1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 478, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    if (!(likely(PyString_CheckExact(__pyx_t_8))||((__pyx_t_8) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_8)->tp_name), 0))) __PYX_ERR(0, 478, __pyx_L1_error)
    __pyx_t_2 = (__pyx_f_9pyteomics_7cparser_is_term_mod(((PyObject*)__pyx_t_8)) != 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (__pyx_t_2) {

      /* "pyteomics/cparser.pyx":479
 *             nterm_aa_position = 0
 *         if is_term_mod(parsed_sequence[-1]):
 *             cterm_aa_position = PyList_Size(parsed_sequence) - 2             # <<<<<<<<<<<<<<
 *         else:
 *             cterm_aa_position = PyList_Size(parsed_sequence) - 1
 */
      __pyx_t_12 = PyList_Size(__pyx_v_parsed_sequence); if (unlikely(__pyx_t_12 == -1L)) __PYX_ERR(0, 479, __pyx_L1_error)
      __pyx_v_cterm_aa_position = (__pyx_t_12 - 2);

      /* "pyteomics/cparser.pyx":478
 *         else:
 *             nterm_aa_position = 0
 *         if is_term_mod(parsed_sequence[-1]):             # <<<<<<<<<<<<<<
 *             cterm_aa_position = PyList_Size(parsed_sequence) - 2
 *         else:
 */
      goto __pyx_L9;
    }

    /* "pyteomics/cparser.pyx":481
 *             cterm_aa_position = PyList_Size(parsed_sequence) - 2
 *         else:
 *             cterm_aa_position = PyList_Size(parsed_sequence) - 1             # <<<<<<<<<<<<<<
 *         if PyList_Size(parsed_sequence) > 1:
 *             # temp = PyInt_FromLong(cterm_aa_position)
 */
    /*else*/ {
      __pyx_t_12 = PyList_Size(__pyx_v_parsed_sequence); if (unlikely(__pyx_t_12 == -1L)) __PYX_ERR(0, 481, __pyx_L1_error)
      __pyx_v_cterm_aa_position = (__pyx_t_12 - 1);
    }
    __pyx_L9:;

    /* "pyteomics/cparser.pyx":482
 *         else:
 *             cterm_aa_position = PyList_Size(parsed_sequence) - 1
 *         if PyList_Size(parsed_sequence) > 1:             # <<<<<<<<<<<<<<
 *             # temp = PyInt_FromLong(cterm_aa_position)
 *             # temp = PyObject_CallMethodObjArgs(parsed_sequence, "pop", <PyObject*>temp, NULL)
 */
    __pyx_t_12 = PyList_Size(__pyx_v_parsed_sequence); if (unlikely(__pyx_t_12 == -1L)) __PYX_ERR(0, 482, __pyx_L1_error)
    __pyx_t_2 = ((__pyx_t_12 > 1) != 0);
    if (__pyx_t_2) {

      /* "pyteomics/cparser.pyx":485
 *             # temp = PyInt_FromLong(cterm_aa_position)
 *             # temp = PyObject_CallMethodObjArgs(parsed_sequence, "pop", <PyObject*>temp, NULL)
 *             temp = parsed_sequence.pop(cterm_aa_position)             # <<<<<<<<<<<<<<
 *             PyDict_SetItem(aa_dict, 'cterm' + temp, 1)
 *         # temp = PyInt_FromLong(nterm_aa_position)
 */
      if (unlikely(__pyx_v_parsed_sequence == Py_None)) {
        PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "pop");
        __PYX_ERR(0, 485, __pyx_L1_error)
      }
      __pyx_t_8 = __Pyx_PyList_PopIndex(__pyx_v_parsed_sequence, Py_None, __pyx_v_cterm_aa_position, 1, int, __Pyx_PyInt_From_int); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 485, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_v_temp = __pyx_t_8;
      __pyx_t_8 = 0;

      /* "pyteomics/cparser.pyx":486
 *             # temp = PyObject_CallMethodObjArgs(parsed_sequence, "pop", <PyObject*>temp, NULL)
 *             temp = parsed_sequence.pop(cterm_aa_position)
 *             PyDict_SetItem(aa_dict, 'cterm' + temp, 1)             # <<<<<<<<<<<<<<
 *         # temp = PyInt_FromLong(nterm_aa_position)
 *         # temp = PyObject_CallMethodObjArgs(parsed_sequence, "pop", <PyObject*>temp, NULL)
 */
      __pyx_t_8 = PyNumber_Add(__pyx_n_s_cterm, __pyx_v_temp); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 486, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_13 = PyDict_SetItem(__pyx_v_aa_dict, __pyx_t_8, __pyx_int_1); if (unlikely(__pyx_t_13 == -1)) __PYX_ERR(0, 486, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

      /* "pyteomics/cparser.pyx":482
 *         else:
 *             cterm_aa_position = PyList_Size(parsed_sequence) - 1
 *         if PyList_Size(parsed_sequence) > 1:             # <<<<<<<<<<<<<<
 *             # temp = PyInt_FromLong(cterm_aa_position)
 *             # temp = PyObject_CallMethodObjArgs(parsed_sequence, "pop", <PyObject*>temp, NULL)
 */
    }

    /* "pyteomics/cparser.pyx":489
 *         # temp = PyInt_FromLong(nterm_aa_position)
 *         # temp = PyObject_CallMethodObjArgs(parsed_sequence, "pop", <PyObject*>temp, NULL)
 *         temp = parsed_sequence.pop(nterm_aa_position)             # <<<<<<<<<<<<<<
 *         PyDict_SetItem(aa_dict, 'nterm' + temp, 1)
 *     # Process core amino acids.
 */
    if (unlikely(__pyx_v_parsed_sequence == Py_None)) {
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "pop");
      __PYX_ERR(0, 489, __pyx_L1_error)
    }
    __pyx_t_8 = __Pyx_PyList_PopIndex(__pyx_v_parsed_sequence, Py_None, __pyx_v_nterm_aa_position, 1, int, __Pyx_PyInt_From_int); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 489, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_XDECREF_SET(__pyx_v_temp, __pyx_t_8);
    __pyx_t_8 = 0;

    /* "pyteomics/cparser.pyx":490
 *         # temp = PyObject_CallMethodObjArgs(parsed_sequence, "pop", <PyObject*>temp, NULL)
 *         temp = parsed_sequence.pop(nterm_aa_position)
 *         PyDict_SetItem(aa_dict, 'nterm' + temp, 1)             # <<<<<<<<<<<<<<
 *     # Process core amino acids.
 *     i = 0
 */
    __pyx_t_8 = PyNumber_Add(__pyx_n_s_nterm, __pyx_v_temp); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 490, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_13 = PyDict_SetItem(__pyx_v_aa_dict, __pyx_t_8, __pyx_int_1); if (unlikely(__pyx_t_13 == -1)) __PYX_ERR(0, 490, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

    /* "pyteomics/cparser.pyx":473
 *     aa_dict = {}
 *     # Process terminal amino acids.
 *     if term_aa:             # <<<<<<<<<<<<<<
 *         if is_term_mod(<object>PyList_GetItem(parsed_sequence, 0)):
 *             nterm_aa_position = 1
 */
  }

  /* "pyteomics/cparser.pyx":492
 *         PyDict_SetItem(aa_dict, 'nterm' + temp, 1)
 *     # Process core amino acids.
 *     i = 0             # <<<<<<<<<<<<<<
 *     for i in range(PyList_Size(parsed_sequence)):
 *         aa = <tuple>PyList_GET_ITEM(parsed_sequence, i)
 */
  __pyx_v_i = 0;

  /* "pyteomics/cparser.pyx":493
 *     # Process core amino acids.
 *     i = 0
 *     for i in range(PyList_Size(parsed_sequence)):             # <<<<<<<<<<<<<<
 *         aa = <tuple>PyList_GET_ITEM(parsed_sequence, i)
 *         pvalue = PyDict_GetItem(aa_dict, aa)
 */
  __pyx_t_12 = PyList_Size(__pyx_v_parsed_sequence); if (unlikely(__pyx_t_12 == -1L)) __PYX_ERR(0, 493, __pyx_L1_error)
  for (__pyx_t_14 = 0; __pyx_t_14 < __pyx_t_12; __pyx_t_14+=1) {
    __pyx_v_i = __pyx_t_14;

    /* "pyteomics/cparser.pyx":494
 *     i = 0
 *     for i in range(PyList_Size(parsed_sequence)):
 *         aa = <tuple>PyList_GET_ITEM(parsed_sequence, i)             # <<<<<<<<<<<<<<
 *         pvalue = PyDict_GetItem(aa_dict, aa)
 *         if pvalue == NULL:
 */
    __pyx_t_5 = PyList_GET_ITEM(__pyx_v_parsed_sequence, __pyx_v_i);
    __pyx_t_8 = ((PyObject *)__pyx_t_5);
    __Pyx_INCREF(__pyx_t_8);
    __Pyx_XDECREF_SET(__pyx_v_aa, ((PyObject*)__pyx_t_8));
    __pyx_t_8 = 0;

    /* "pyteomics/cparser.pyx":495
 *     for i in range(PyList_Size(parsed_sequence)):
 *         aa = <tuple>PyList_GET_ITEM(parsed_sequence, i)
 *         pvalue = PyDict_GetItem(aa_dict, aa)             # <<<<<<<<<<<<<<
 *         if pvalue == NULL:
 *             aa_count = 0
 */
    __pyx_v_pvalue = PyDict_GetItem(__pyx_v_aa_dict, __pyx_v_aa);

    /* "pyteomics/cparser.pyx":496
 *         aa = <tuple>PyList_GET_ITEM(parsed_sequence, i)
 *         pvalue = PyDict_GetItem(aa_dict, aa)
 *         if pvalue == NULL:             # <<<<<<<<<<<<<<
 *             aa_count = 0
 *         else:
 */
    __pyx_t_2 = ((__pyx_v_pvalue == NULL) != 0);
    if (__pyx_t_2) {

      /* "pyteomics/cparser.pyx":497
 *         pvalue = PyDict_GetItem(aa_dict, aa)
 *         if pvalue == NULL:
 *             aa_count = 0             # <<<<<<<<<<<<<<
 *         else:
 *             aa_count = PyInt_AsLong(<object>pvalue)
 */
      __pyx_v_aa_count = 0;

      /* "pyteomics/cparser.pyx":496
 *         aa = <tuple>PyList_GET_ITEM(parsed_sequence, i)
 *         pvalue = PyDict_GetItem(aa_dict, aa)
 *         if pvalue == NULL:             # <<<<<<<<<<<<<<
 *             aa_count = 0
 *         else:
 */
      goto __pyx_L13;
    }

    /* "pyteomics/cparser.pyx":499
 *             aa_count = 0
 *         else:
 *             aa_count = PyInt_AsLong(<object>pvalue)             # <<<<<<<<<<<<<<
 *         aa_count += 1
 *         PyDict_SetItem(aa_dict, aa, aa_count)
 */
    /*else*/ {
      __pyx_t_15 = PyInt_AsLong(((PyObject *)__pyx_v_pvalue)); if (unlikely(__pyx_t_15 == -1L && PyErr_Occurred())) __PYX_ERR(0, 499, __pyx_L1_error)
      __pyx_v_aa_count = __pyx_t_15;
    }
    __pyx_L13:;

    /* "pyteomics/cparser.pyx":500
 *         else:
 *             aa_count = PyInt_AsLong(<object>pvalue)
 *         aa_count += 1             # <<<<<<<<<<<<<<
 *         PyDict_SetItem(aa_dict, aa, aa_count)
 * 
 */
    __pyx_v_aa_count = (__pyx_v_aa_count + 1);

    /* "pyteomics/cparser.pyx":501
 *             aa_count = PyInt_AsLong(<object>pvalue)
 *         aa_count += 1
 *         PyDict_SetItem(aa_dict, aa, aa_count)             # <<<<<<<<<<<<<<
 * 
 *     return aa_dict
 */
    __pyx_t_8 = __Pyx_PyInt_From_int(__pyx_v_aa_count); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 501, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_13 = PyDict_SetItem(__pyx_v_aa_dict, __pyx_v_aa, __pyx_t_8); if (unlikely(__pyx_t_13 == -1)) __PYX_ERR(0, 501, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  }

  /* "pyteomics/cparser.pyx":503
 *         PyDict_SetItem(aa_dict, aa, aa_count)
 * 
 *     return aa_dict             # <<<<<<<<<<<<<<
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_aa_dict);
  __pyx_r = __pyx_v_aa_dict;
  goto __pyx_L0;

  /* "pyteomics/cparser.pyx":403
 * 
 * 
 * cpdef dict amino_acid_composition(object sequence, bint show_unmodified_termini=0, bint term_aa=0,             # <<<<<<<<<<<<<<
 *                                   bint allow_unknown_modifications=0, object labels=std_labels):
 *     """Calculate amino acid composition of a polypeptide.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("pyteomics.cparser.amino_acid_composition", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_parsed_sequence);
  __Pyx_XDECREF(__pyx_v_aa_dict);
  __Pyx_XDECREF(__pyx_v_aa);
  __Pyx_XDECREF(__pyx_v__labels);
  __Pyx_XDECREF(__pyx_v_temp);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9pyteomics_7cparser_7amino_acid_composition(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_9pyteomics_7cparser_6amino_acid_composition[] = "amino_acid_composition(sequence, bool show_unmodified_termini=0, bool term_aa=0, bool allow_unknown_modifications=0, labels=std_labels) -> dict\nCalculate amino acid composition of a polypeptide.\n\n    Parameters\n    ----------\n    sequence : str or list\n        The sequence of a polypeptide or a list with a parsed sequence.\n    show_unmodified_termini : bool, optional\n        If :py:const:`True` then the unmodified N- and C-terminus are explicitly\n        shown in the returned dict. Default value is :py:const:`False`.\n    term_aa : bool, optional\n        If :py:const:`True` then the terminal amino acid residues are\n        artificially modified with `nterm` or `cterm` modification.\n        Default value is :py:const:`False`.\n    allow_unknown_modifications : bool, optional\n        If :py:const:`True` then do not raise an exception when an unknown\n        modification of a known amino acid residue is found in the sequence.\n        Default value is :py:const:`False`.\n    labels : list, optional\n        A list of allowed labels for amino acids and terminal modifications.\n\n    Returns\n    -------\n    out : dict\n        A dictionary of amino acid composition.\n\n    Examples\n    --------\n    >>> amino_acid_composition('PEPTIDE') ==     {'I': 1, 'P': 2, 'E': 2, 'T': 1, 'D': 1}\n    True\n    >>> amino_acid_composition('PEPTDE', term_aa=True) ==     {'ctermE': 1, 'E': 1, 'D': 1, 'P': 1, 'T': 1, 'ntermP': 1}\n    True\n    >>> amino_acid_composition('PEPpTIDE', labels=std_labels+['pT']) ==     {'I': 1, 'P': 2, 'E': 2, 'D': 1, 'pT': 1}\n    True\n    ";
static PyObject *__pyx_pw_9pyteomics_7cparser_7amino_acid_composition(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_sequence = 0;
  int __pyx_v_show_unmodified_termini;
  int __pyx_v_term_aa;
  int __pyx_v_allow_unknown_modifications;
  PyObject *__pyx_v_labels = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("amino_acid_composition (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_sequence,&__pyx_n_s_show_unmodified_termini,&__pyx_n_s_term_aa,&__pyx_n_s_allow_unknown_modifications,&__pyx_n_s_labels,0};
    PyObject* values[5] = {0,0,0,0,0};
    values[4] = __pyx_k__6;
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_sequence)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_show_unmodified_termini);
          if (value) { values[1] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_term_aa);
          if (value) { values[2] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_allow_unknown_modifications);
          if (value) { values[3] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_labels);
          if (value) { values[4] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "amino_acid_composition") < 0)) __PYX_ERR(0, 403, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_sequence = values[0];
    if (values[1]) {
      __pyx_v_show_unmodified_termini = __Pyx_PyObject_IsTrue(values[1]); if (unlikely((__pyx_v_show_unmodified_termini == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 403, __pyx_L3_error)
    } else {
      __pyx_v_show_unmodified_termini = ((int)0);
    }
    if (values[2]) {
      __pyx_v_term_aa = __Pyx_PyObject_IsTrue(values[2]); if (unlikely((__pyx_v_term_aa == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 403, __pyx_L3_error)
    } else {
      __pyx_v_term_aa = ((int)0);
    }
    if (values[3]) {
      __pyx_v_allow_unknown_modifications = __Pyx_PyObject_IsTrue(values[3]); if (unlikely((__pyx_v_allow_unknown_modifications == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 404, __pyx_L3_error)
    } else {
      __pyx_v_allow_unknown_modifications = ((int)0);
    }
    __pyx_v_labels = values[4];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("amino_acid_composition", 0, 1, 5, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 403, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyteomics.cparser.amino_acid_composition", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9pyteomics_7cparser_6amino_acid_composition(__pyx_self, __pyx_v_sequence, __pyx_v_show_unmodified_termini, __pyx_v_term_aa, __pyx_v_allow_unknown_modifications, __pyx_v_labels);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9pyteomics_7cparser_6amino_acid_composition(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_sequence, int __pyx_v_show_unmodified_termini, int __pyx_v_term_aa, int __pyx_v_allow_unknown_modifications, PyObject *__pyx_v_labels) {
  PyObject *__pyx_r = NULL;
  __Pyx_TraceDeclarations
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_9pyteomics_7cparser_amino_acid_composition __pyx_t_2;
  __Pyx_RefNannySetupContext("amino_acid_composition", 0);
  __Pyx_TraceCall("amino_acid_composition (wrapper)", __pyx_f[0], 403, 0, __PYX_ERR(0, 403, __pyx_L1_error));
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 4;
  __pyx_t_2.show_unmodified_termini = __pyx_v_show_unmodified_termini;
  __pyx_t_2.term_aa = __pyx_v_term_aa;
  __pyx_t_2.allow_unknown_modifications = __pyx_v_allow_unknown_modifications;
  __pyx_t_2.labels = __pyx_v_labels;
  __pyx_t_1 = __pyx_f_9pyteomics_7cparser_amino_acid_composition(__pyx_v_sequence, 0, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 403, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyteomics.cparser.amino_acid_composition", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_TraceReturn(__pyx_r, 0);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyMethodDef __pyx_methods[] = {
  {"length", (PyCFunction)__pyx_pw_9pyteomics_7cparser_1length, METH_VARARGS|METH_KEYWORDS, __pyx_doc_9pyteomics_7cparser_length},
  {"_split_label", (PyCFunction)__pyx_pw_9pyteomics_7cparser_3_split_label, METH_O, __pyx_doc_9pyteomics_7cparser_2_split_label},
  {"parse", (PyCFunction)__pyx_pw_9pyteomics_7cparser_5parse, METH_VARARGS|METH_KEYWORDS, __pyx_doc_9pyteomics_7cparser_4parse},
  {"amino_acid_composition", (PyCFunction)__pyx_pw_9pyteomics_7cparser_7amino_acid_composition, METH_VARARGS|METH_KEYWORDS, __pyx_doc_9pyteomics_7cparser_6amino_acid_composition},
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_cparser(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_cparser},
  {0, NULL}
};
#endif

static struct PyModuleDef __pyx_moduledef = {
    PyModuleDef_HEAD_INIT,
    "cparser",
    0, /* m_doc */
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    0, /* m_size */
  #else
    -1, /* m_size */
  #endif
    __pyx_methods /* m_methods */,
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    __pyx_moduledef_slots, /* m_slots */
  #else
    NULL, /* m_reload */
  #endif
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_kp_s_, __pyx_k_, sizeof(__pyx_k_), 0, 0, 1, 0},
  {&__pyx_n_s_A, __pyx_k_A, sizeof(__pyx_k_A), 0, 0, 1, 1},
  {&__pyx_n_s_BasicComposition, __pyx_k_BasicComposition, sizeof(__pyx_k_BasicComposition), 0, 0, 1, 1},
  {&__pyx_n_s_C, __pyx_k_C, sizeof(__pyx_k_C), 0, 0, 1, 1},
  {&__pyx_kp_u_Calculate_amino_acid_composition, __pyx_k_Calculate_amino_acid_composition, sizeof(__pyx_k_Calculate_amino_acid_composition), 0, 1, 0, 0},
  {&__pyx_kp_u_Calculate_the_number_of_amino_ac, __pyx_k_Calculate_the_number_of_amino_ac, sizeof(__pyx_k_Calculate_the_number_of_amino_ac), 0, 1, 0, 0},
  {&__pyx_kp_s_Cannot_split_a_non_modX_label_s, __pyx_k_Cannot_split_a_non_modX_label_s, sizeof(__pyx_k_Cannot_split_a_non_modX_label_s), 0, 0, 1, 0},
  {&__pyx_n_s_D, __pyx_k_D, sizeof(__pyx_k_D), 0, 0, 1, 1},
  {&__pyx_n_s_E, __pyx_k_E, sizeof(__pyx_k_E), 0, 0, 1, 1},
  {&__pyx_n_s_F, __pyx_k_F, sizeof(__pyx_k_F), 0, 0, 1, 1},
  {&__pyx_n_s_G, __pyx_k_G, sizeof(__pyx_k_G), 0, 0, 1, 1},
  {&__pyx_n_s_H, __pyx_k_H, sizeof(__pyx_k_H), 0, 0, 1, 1},
  {&__pyx_kp_s_H_2, __pyx_k_H_2, sizeof(__pyx_k_H_2), 0, 0, 1, 0},
  {&__pyx_n_s_I, __pyx_k_I, sizeof(__pyx_k_I), 0, 0, 1, 1},
  {&__pyx_n_s_K, __pyx_k_K, sizeof(__pyx_k_K), 0, 0, 1, 1},
  {&__pyx_n_s_L, __pyx_k_L, sizeof(__pyx_k_L), 0, 0, 1, 1},
  {&__pyx_n_s_M, __pyx_k_M, sizeof(__pyx_k_M), 0, 0, 1, 1},
  {&__pyx_n_s_N, __pyx_k_N, sizeof(__pyx_k_N), 0, 0, 1, 1},
  {&__pyx_kp_s_Not_a_valid_modX_sequence, __pyx_k_Not_a_valid_modX_sequence, sizeof(__pyx_k_Not_a_valid_modX_sequence), 0, 0, 1, 0},
  {&__pyx_kp_s_OH, __pyx_k_OH, sizeof(__pyx_k_OH), 0, 0, 1, 0},
  {&__pyx_n_s_P, __pyx_k_P, sizeof(__pyx_k_P), 0, 0, 1, 1},
  {&__pyx_kp_u_Parse_a_sequence_string_written, __pyx_k_Parse_a_sequence_string_written, sizeof(__pyx_k_Parse_a_sequence_string_written), 0, 1, 0, 0},
  {&__pyx_n_s_PyteomicsError, __pyx_k_PyteomicsError, sizeof(__pyx_k_PyteomicsError), 0, 0, 1, 1},
  {&__pyx_n_s_Q, __pyx_k_Q, sizeof(__pyx_k_Q), 0, 0, 1, 1},
  {&__pyx_n_s_R, __pyx_k_R, sizeof(__pyx_k_R), 0, 0, 1, 1},
  {&__pyx_n_s_S, __pyx_k_S, sizeof(__pyx_k_S), 0, 0, 1, 1},
  {&__pyx_n_s_T, __pyx_k_T, sizeof(__pyx_k_T), 0, 0, 1, 1},
  {&__pyx_kp_s_Unknown_label, __pyx_k_Unknown_label, sizeof(__pyx_k_Unknown_label), 0, 0, 1, 0},
  {&__pyx_kp_s_Unsupported_type_of_a_sequence_M, __pyx_k_Unsupported_type_of_a_sequence_M, sizeof(__pyx_k_Unsupported_type_of_a_sequence_M), 0, 0, 1, 0},
  {&__pyx_kp_s_Unsupported_type_of_sequence, __pyx_k_Unsupported_type_of_sequence, sizeof(__pyx_k_Unsupported_type_of_sequence), 0, 0, 1, 0},
  {&__pyx_n_s_V, __pyx_k_V, sizeof(__pyx_k_V), 0, 0, 1, 1},
  {&__pyx_n_s_W, __pyx_k_W, sizeof(__pyx_k_W), 0, 0, 1, 1},
  {&__pyx_n_s_Y, __pyx_k_Y, sizeof(__pyx_k_Y), 0, 0, 1, 1},
  {&__pyx_kp_s__5, __pyx_k__5, sizeof(__pyx_k__5), 0, 0, 1, 0},
  {&__pyx_kp_s_a_z_A_Z, __pyx_k_a_z_A_Z, sizeof(__pyx_k_a_z_A_Z), 0, 0, 1, 0},
  {&__pyx_kp_s_a_z_A_Z_2, __pyx_k_a_z_A_Z_2, sizeof(__pyx_k_a_z_A_Z_2), 0, 0, 1, 0},
  {&__pyx_kp_s_a_z_A_Z_3, __pyx_k_a_z_A_Z_3, sizeof(__pyx_k_a_z_A_Z_3), 0, 0, 1, 0},
  {&__pyx_n_s_allow_unknown_modifications, __pyx_k_allow_unknown_modifications, sizeof(__pyx_k_allow_unknown_modifications), 0, 0, 1, 1},
  {&__pyx_kp_u_amino_acid_composition_line_403, __pyx_k_amino_acid_composition_line_403, sizeof(__pyx_k_amino_acid_composition_line_403), 0, 1, 0, 0},
  {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {&__pyx_n_s_collections, __pyx_k_collections, sizeof(__pyx_k_collections), 0, 0, 1, 1},
  {&__pyx_n_s_compile, __pyx_k_compile, sizeof(__pyx_k_compile), 0, 0, 1, 1},
  {&__pyx_n_s_cterm, __pyx_k_cterm, sizeof(__pyx_k_cterm), 0, 0, 1, 1},
  {&__pyx_n_s_deque, __pyx_k_deque, sizeof(__pyx_k_deque), 0, 0, 1, 1},
  {&__pyx_n_s_findall, __pyx_k_findall, sizeof(__pyx_k_findall), 0, 0, 1, 1},
  {&__pyx_n_s_format, __pyx_k_format, sizeof(__pyx_k_format), 0, 0, 1, 1},
  {&__pyx_n_s_groups, __pyx_k_groups, sizeof(__pyx_k_groups), 0, 0, 1, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_it, __pyx_k_it, sizeof(__pyx_k_it), 0, 0, 1, 1},
  {&__pyx_n_s_items, __pyx_k_items, sizeof(__pyx_k_items), 0, 0, 1, 1},
  {&__pyx_n_s_itertools, __pyx_k_itertools, sizeof(__pyx_k_itertools), 0, 0, 1, 1},
  {&__pyx_n_s_join, __pyx_k_join, sizeof(__pyx_k_join), 0, 0, 1, 1},
  {&__pyx_n_s_labels, __pyx_k_labels, sizeof(__pyx_k_labels), 0, 0, 1, 1},
  {&__pyx_kp_u_length_line_106, __pyx_k_length_line_106, sizeof(__pyx_k_length_line_106), 0, 1, 0, 0},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_match, __pyx_k_match, sizeof(__pyx_k_match), 0, 0, 1, 1},
  {&__pyx_n_s_memoize, __pyx_k_memoize, sizeof(__pyx_k_memoize), 0, 0, 1, 1},
  {&__pyx_n_s_nterm, __pyx_k_nterm, sizeof(__pyx_k_nterm), 0, 0, 1, 1},
  {&__pyx_kp_u_parse_line_199, __pyx_k_parse_line_199, sizeof(__pyx_k_parse_line_199), 0, 1, 0, 0},
  {&__pyx_n_s_pop, __pyx_k_pop, sizeof(__pyx_k_pop), 0, 0, 1, 1},
  {&__pyx_n_s_pyteomics_auxiliary, __pyx_k_pyteomics_auxiliary, sizeof(__pyx_k_pyteomics_auxiliary), 0, 0, 1, 1},
  {&__pyx_n_s_range, __pyx_k_range, sizeof(__pyx_k_range), 0, 0, 1, 1},
  {&__pyx_n_s_re, __pyx_k_re, sizeof(__pyx_k_re), 0, 0, 1, 1},
  {&__pyx_n_s_remove, __pyx_k_remove, sizeof(__pyx_k_remove), 0, 0, 1, 1},
  {&__pyx_n_s_sequence, __pyx_k_sequence, sizeof(__pyx_k_sequence), 0, 0, 1, 1},
  {&__pyx_n_s_show_unmodified_termini, __pyx_k_show_unmodified_termini, sizeof(__pyx_k_show_unmodified_termini), 0, 0, 1, 1},
  {&__pyx_n_s_split, __pyx_k_split, sizeof(__pyx_k_split), 0, 0, 1, 1},
  {&__pyx_n_s_sum, __pyx_k_sum, sizeof(__pyx_k_sum), 0, 0, 1, 1},
  {&__pyx_n_s_term_aa, __pyx_k_term_aa, sizeof(__pyx_k_term_aa), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_sum = __Pyx_GetBuiltinName(__pyx_n_s_sum); if (!__pyx_builtin_sum) __PYX_ERR(0, 142, __pyx_L1_error)
  __pyx_builtin_range = __Pyx_GetBuiltinName(__pyx_n_s_range); if (!__pyx_builtin_range) __PYX_ERR(0, 189, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "pyteomics/cparser.pyx":145
 *                     if not is_term_mod(aa)])
 * 
 *     raise PyteomicsError('Unsupported type of sequence.')             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_tuple__3 = PyTuple_Pack(1, __pyx_kp_s_Unsupported_type_of_sequence); if (unlikely(!__pyx_tuple__3)) __PYX_ERR(0, 145, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__3);
  __Pyx_GIVEREF(__pyx_tuple__3);

  /* "pyteomics/cparser.pyx":54
 * """modX labels for the standard amino acids and unmodified termini."""
 * 
 * _modX_sequence = re.compile(r'^([^-]+-)?((?:[a-z]*[A-Z])+)(-[^-]+)?$')             # <<<<<<<<<<<<<<
 * _modX_group = re.compile(r'[a-z]*[A-Z]')
 * _modX_split = re.compile(r'([a-z]*)([A-Z])')
 */
  __pyx_tuple__7 = PyTuple_Pack(1, __pyx_kp_s_a_z_A_Z); if (unlikely(!__pyx_tuple__7)) __PYX_ERR(0, 54, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__7);
  __Pyx_GIVEREF(__pyx_tuple__7);

  /* "pyteomics/cparser.pyx":55
 * 
 * _modX_sequence = re.compile(r'^([^-]+-)?((?:[a-z]*[A-Z])+)(-[^-]+)?$')
 * _modX_group = re.compile(r'[a-z]*[A-Z]')             # <<<<<<<<<<<<<<
 * _modX_split = re.compile(r'([a-z]*)([A-Z])')
 * 
 */
  __pyx_tuple__8 = PyTuple_Pack(1, __pyx_kp_s_a_z_A_Z_2); if (unlikely(!__pyx_tuple__8)) __PYX_ERR(0, 55, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__8);
  __Pyx_GIVEREF(__pyx_tuple__8);

  /* "pyteomics/cparser.pyx":56
 * _modX_sequence = re.compile(r'^([^-]+-)?((?:[a-z]*[A-Z])+)(-[^-]+)?$')
 * _modX_group = re.compile(r'[a-z]*[A-Z]')
 * _modX_split = re.compile(r'([a-z]*)([A-Z])')             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_tuple__9 = PyTuple_Pack(1, __pyx_kp_s_a_z_A_Z_3); if (unlikely(!__pyx_tuple__9)) __PYX_ERR(0, 56, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__9);
  __Pyx_GIVEREF(__pyx_tuple__9);
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  __pyx_int_0 = PyInt_FromLong(0); if (unlikely(!__pyx_int_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_1 = PyInt_FromLong(1); if (unlikely(!__pyx_int_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

#if PY_MAJOR_VERSION < 3
PyMODINIT_FUNC initcparser(void); /*proto*/
PyMODINIT_FUNC initcparser(void)
#else
PyMODINIT_FUNC PyInit_cparser(void); /*proto*/
PyMODINIT_FUNC PyInit_cparser(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name) {
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        result = PyDict_SetItemString(moddict, to_name, value);
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static PyObject* __pyx_pymod_create(PyObject *spec, CYTHON_UNUSED PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__") < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static int __pyx_pymod_exec_cparser(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  __Pyx_TraceDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannyDeclarations
  #if CYTHON_REFNANNY
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
  if (!__Pyx_RefNanny) {
      PyErr_Clear();
      __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
      if (!__Pyx_RefNanny)
          Py_FatalError("failed to import 'refnanny' module");
  }
  #endif
  __Pyx_RefNannySetupContext("PyMODINIT_FUNC PyInit_cparser(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  #ifdef WITH_THREAD /* Python build with threading support? */
  PyEval_InitThreads();
  #endif
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("cparser", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_cython_runtime = PyImport_AddModule((char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  #if CYTHON_COMPILING_IN_PYPY
  Py_INCREF(__pyx_b);
  #endif
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_pyteomics__cparser) {
    if (PyObject_SetAttrString(__pyx_m, "__name__", __pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "pyteomics.cparser")) {
      if (unlikely(PyDict_SetItemString(modules, "pyteomics.cparser", __pyx_m) < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global init code ---*/
  __pyx_v_9pyteomics_7cparser_std_amino_acids = ((PyObject*)Py_None); Py_INCREF(Py_None);
  __pyx_v_9pyteomics_7cparser_std_labels = ((PyObject*)Py_None); Py_INCREF(Py_None);
  __pyx_v_9pyteomics_7cparser_std_nterm = ((PyObject*)Py_None); Py_INCREF(Py_None);
  __pyx_v_9pyteomics_7cparser_std_cterm = ((PyObject*)Py_None); Py_INCREF(Py_None);
  __pyx_v_9pyteomics_7cparser__modX_sequence = Py_None; Py_INCREF(Py_None);
  __pyx_v_9pyteomics_7cparser__modX_group = Py_None; Py_INCREF(Py_None);
  __pyx_v_9pyteomics_7cparser__modX_split = Py_None; Py_INCREF(Py_None);
  __pyx_v_9pyteomics_7cparser__modX_split_findall = Py_None; Py_INCREF(Py_None);
  /*--- Variable export code ---*/
  /*--- Function export code ---*/
  if (__Pyx_ExportFunction("parse", (void (*)(void))__pyx_f_9pyteomics_7cparser_parse, "PyObject *(PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_9pyteomics_7cparser_parse *__pyx_optional_args)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("amino_acid_composition", (void (*)(void))__pyx_f_9pyteomics_7cparser_amino_acid_composition, "PyObject *(PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_9pyteomics_7cparser_amino_acid_composition *__pyx_optional_args)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("_split_label", (void (*)(void))__pyx_f_9pyteomics_7cparser__split_label, "PyObject *(PyObject *, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("tostring", (void (*)(void))__pyx_f_9pyteomics_7cparser_tostring, "PyObject *(PyObject *, struct __pyx_opt_args_9pyteomics_7cparser_tostring *__pyx_optional_args)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Type init code ---*/
  /*--- Type import code ---*/
  __pyx_ptype_7cpython_4type_type = __Pyx_ImportType(__Pyx_BUILTIN_MODULE_NAME, "type", 
  #if CYTHON_COMPILING_IN_PYPY
  sizeof(PyTypeObject),
  #else
  sizeof(PyHeapTypeObject),
  #endif
  0); if (unlikely(!__pyx_ptype_7cpython_4type_type)) __PYX_ERR(1, 9, __pyx_L1_error)
  /*--- Variable import code ---*/
  /*--- Function import code ---*/
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  __Pyx_TraceCall("PyMODINIT_FUNC PyInit_cparser(void)", __pyx_f[0], 1, 0, __PYX_ERR(0, 1, __pyx_L1_error));

  /* "pyteomics/cparser.pyx":30
 * from cpython.object cimport PyObject_CallMethodObjArgs, PyObject_Not, PyObject_IsTrue
 * 
 * import re             # <<<<<<<<<<<<<<
 * from collections import deque
 * import itertools as it
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_re, 0, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 30, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_re, __pyx_t_1) < 0) __PYX_ERR(0, 30, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyteomics/cparser.pyx":31
 * 
 * import re
 * from collections import deque             # <<<<<<<<<<<<<<
 * import itertools as it
 * from pyteomics.auxiliary import PyteomicsError, memoize, BasicComposition
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_deque);
  __Pyx_GIVEREF(__pyx_n_s_deque);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_deque);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_collections, __pyx_t_1, -1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_deque); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_deque, __pyx_t_1) < 0) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyteomics/cparser.pyx":32
 * import re
 * from collections import deque
 * import itertools as it             # <<<<<<<<<<<<<<
 * from pyteomics.auxiliary import PyteomicsError, memoize, BasicComposition
 * 
 */
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_itertools, 0, -1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 32, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_it, __pyx_t_2) < 0) __PYX_ERR(0, 32, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pyteomics/cparser.pyx":33
 * from collections import deque
 * import itertools as it
 * from pyteomics.auxiliary import PyteomicsError, memoize, BasicComposition             # <<<<<<<<<<<<<<
 * 
 * cdef:
 */
  __pyx_t_2 = PyList_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 33, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_s_PyteomicsError);
  __Pyx_GIVEREF(__pyx_n_s_PyteomicsError);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PyteomicsError);
  __Pyx_INCREF(__pyx_n_s_memoize);
  __Pyx_GIVEREF(__pyx_n_s_memoize);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_n_s_memoize);
  __Pyx_INCREF(__pyx_n_s_BasicComposition);
  __Pyx_GIVEREF(__pyx_n_s_BasicComposition);
  PyList_SET_ITEM(__pyx_t_2, 2, __pyx_n_s_BasicComposition);
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_pyteomics_auxiliary, __pyx_t_2, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 33, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_PyteomicsError); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 33, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_PyteomicsError, __pyx_t_2) < 0) __PYX_ERR(0, 33, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_memoize); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 33, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_memoize, __pyx_t_2) < 0) __PYX_ERR(0, 33, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_BasicComposition); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 33, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_BasicComposition, __pyx_t_2) < 0) __PYX_ERR(0, 33, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyteomics/cparser.pyx":41
 * 
 * 
 * std_amino_acids = ['Q','W','E','R','T','Y','I','P','A','S',             # <<<<<<<<<<<<<<
 *                    'D','F','G','H','K','L','C','V','N','M']
 * """modX labels for the 20 standard amino acids."""
 */
  __pyx_t_1 = PyList_New(20); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 41, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_Q);
  __Pyx_GIVEREF(__pyx_n_s_Q);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_Q);
  __Pyx_INCREF(__pyx_n_s_W);
  __Pyx_GIVEREF(__pyx_n_s_W);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_s_W);
  __Pyx_INCREF(__pyx_n_s_E);
  __Pyx_GIVEREF(__pyx_n_s_E);
  PyList_SET_ITEM(__pyx_t_1, 2, __pyx_n_s_E);
  __Pyx_INCREF(__pyx_n_s_R);
  __Pyx_GIVEREF(__pyx_n_s_R);
  PyList_SET_ITEM(__pyx_t_1, 3, __pyx_n_s_R);
  __Pyx_INCREF(__pyx_n_s_T);
  __Pyx_GIVEREF(__pyx_n_s_T);
  PyList_SET_ITEM(__pyx_t_1, 4, __pyx_n_s_T);
  __Pyx_INCREF(__pyx_n_s_Y);
  __Pyx_GIVEREF(__pyx_n_s_Y);
  PyList_SET_ITEM(__pyx_t_1, 5, __pyx_n_s_Y);
  __Pyx_INCREF(__pyx_n_s_I);
  __Pyx_GIVEREF(__pyx_n_s_I);
  PyList_SET_ITEM(__pyx_t_1, 6, __pyx_n_s_I);
  __Pyx_INCREF(__pyx_n_s_P);
  __Pyx_GIVEREF(__pyx_n_s_P);
  PyList_SET_ITEM(__pyx_t_1, 7, __pyx_n_s_P);
  __Pyx_INCREF(__pyx_n_s_A);
  __Pyx_GIVEREF(__pyx_n_s_A);
  PyList_SET_ITEM(__pyx_t_1, 8, __pyx_n_s_A);
  __Pyx_INCREF(__pyx_n_s_S);
  __Pyx_GIVEREF(__pyx_n_s_S);
  PyList_SET_ITEM(__pyx_t_1, 9, __pyx_n_s_S);
  __Pyx_INCREF(__pyx_n_s_D);
  __Pyx_GIVEREF(__pyx_n_s_D);
  PyList_SET_ITEM(__pyx_t_1, 10, __pyx_n_s_D);
  __Pyx_INCREF(__pyx_n_s_F);
  __Pyx_GIVEREF(__pyx_n_s_F);
  PyList_SET_ITEM(__pyx_t_1, 11, __pyx_n_s_F);
  __Pyx_INCREF(__pyx_n_s_G);
  __Pyx_GIVEREF(__pyx_n_s_G);
  PyList_SET_ITEM(__pyx_t_1, 12, __pyx_n_s_G);
  __Pyx_INCREF(__pyx_n_s_H);
  __Pyx_GIVEREF(__pyx_n_s_H);
  PyList_SET_ITEM(__pyx_t_1, 13, __pyx_n_s_H);
  __Pyx_INCREF(__pyx_n_s_K);
  __Pyx_GIVEREF(__pyx_n_s_K);
  PyList_SET_ITEM(__pyx_t_1, 14, __pyx_n_s_K);
  __Pyx_INCREF(__pyx_n_s_L);
  __Pyx_GIVEREF(__pyx_n_s_L);
  PyList_SET_ITEM(__pyx_t_1, 15, __pyx_n_s_L);
  __Pyx_INCREF(__pyx_n_s_C);
  __Pyx_GIVEREF(__pyx_n_s_C);
  PyList_SET_ITEM(__pyx_t_1, 16, __pyx_n_s_C);
  __Pyx_INCREF(__pyx_n_s_V);
  __Pyx_GIVEREF(__pyx_n_s_V);
  PyList_SET_ITEM(__pyx_t_1, 17, __pyx_n_s_V);
  __Pyx_INCREF(__pyx_n_s_N);
  __Pyx_GIVEREF(__pyx_n_s_N);
  PyList_SET_ITEM(__pyx_t_1, 18, __pyx_n_s_N);
  __Pyx_INCREF(__pyx_n_s_M);
  __Pyx_GIVEREF(__pyx_n_s_M);
  PyList_SET_ITEM(__pyx_t_1, 19, __pyx_n_s_M);
  __Pyx_XGOTREF(__pyx_v_9pyteomics_7cparser_std_amino_acids);
  __Pyx_DECREF_SET(__pyx_v_9pyteomics_7cparser_std_amino_acids, ((PyObject*)__pyx_t_1));
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyteomics/cparser.pyx":45
 * """modX labels for the 20 standard amino acids."""
 * 
 * std_nterm = 'H-'             # <<<<<<<<<<<<<<
 * """modX label for the unmodified N-terminus."""
 * 
 */
  __Pyx_INCREF(__pyx_kp_s_H_2);
  __Pyx_XGOTREF(__pyx_v_9pyteomics_7cparser_std_nterm);
  __Pyx_DECREF_SET(__pyx_v_9pyteomics_7cparser_std_nterm, __pyx_kp_s_H_2);
  __Pyx_GIVEREF(__pyx_kp_s_H_2);

  /* "pyteomics/cparser.pyx":48
 * """modX label for the unmodified N-terminus."""
 * 
 * std_cterm = '-OH'             # <<<<<<<<<<<<<<
 * """modX label for the unmodified C-terminus."""
 * 
 */
  __Pyx_INCREF(__pyx_kp_s_OH);
  __Pyx_XGOTREF(__pyx_v_9pyteomics_7cparser_std_cterm);
  __Pyx_DECREF_SET(__pyx_v_9pyteomics_7cparser_std_cterm, __pyx_kp_s_OH);
  __Pyx_GIVEREF(__pyx_kp_s_OH);

  /* "pyteomics/cparser.pyx":51
 * """modX label for the unmodified C-terminus."""
 * 
 * std_labels = std_amino_acids + [std_nterm, std_cterm]             # <<<<<<<<<<<<<<
 * """modX labels for the standard amino acids and unmodified termini."""
 * 
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 51, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_9pyteomics_7cparser_std_nterm);
  __Pyx_GIVEREF(__pyx_v_9pyteomics_7cparser_std_nterm);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_v_9pyteomics_7cparser_std_nterm);
  __Pyx_INCREF(__pyx_v_9pyteomics_7cparser_std_cterm);
  __Pyx_GIVEREF(__pyx_v_9pyteomics_7cparser_std_cterm);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_v_9pyteomics_7cparser_std_cterm);
  __pyx_t_2 = PyNumber_Add(__pyx_v_9pyteomics_7cparser_std_amino_acids, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 51, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_XGOTREF(__pyx_v_9pyteomics_7cparser_std_labels);
  __Pyx_DECREF_SET(__pyx_v_9pyteomics_7cparser_std_labels, ((PyObject*)__pyx_t_2));
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;

  /* "pyteomics/cparser.pyx":54
 * """modX labels for the standard amino acids and unmodified termini."""
 * 
 * _modX_sequence = re.compile(r'^([^-]+-)?((?:[a-z]*[A-Z])+)(-[^-]+)?$')             # <<<<<<<<<<<<<<
 * _modX_group = re.compile(r'[a-z]*[A-Z]')
 * _modX_split = re.compile(r'([a-z]*)([A-Z])')
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_re); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 54, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_compile); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 54, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__7, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 54, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_XGOTREF(__pyx_v_9pyteomics_7cparser__modX_sequence);
  __Pyx_DECREF_SET(__pyx_v_9pyteomics_7cparser__modX_sequence, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;

  /* "pyteomics/cparser.pyx":55
 * 
 * _modX_sequence = re.compile(r'^([^-]+-)?((?:[a-z]*[A-Z])+)(-[^-]+)?$')
 * _modX_group = re.compile(r'[a-z]*[A-Z]')             # <<<<<<<<<<<<<<
 * _modX_split = re.compile(r'([a-z]*)([A-Z])')
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_re); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 55, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_compile); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 55, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__8, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 55, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_XGOTREF(__pyx_v_9pyteomics_7cparser__modX_group);
  __Pyx_DECREF_SET(__pyx_v_9pyteomics_7cparser__modX_group, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;

  /* "pyteomics/cparser.pyx":56
 * _modX_sequence = re.compile(r'^([^-]+-)?((?:[a-z]*[A-Z])+)(-[^-]+)?$')
 * _modX_group = re.compile(r'[a-z]*[A-Z]')
 * _modX_split = re.compile(r'([a-z]*)([A-Z])')             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_re); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 56, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_compile); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 56, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__9, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 56, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_XGOTREF(__pyx_v_9pyteomics_7cparser__modX_split);
  __Pyx_DECREF_SET(__pyx_v_9pyteomics_7cparser__modX_split, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;

  /* "pyteomics/cparser.pyx":108
 * cpdef int length(object sequence,
 *                  int show_unmodified_termini=0, int split=0,
 *                  int allow_unknown_modifications=0, object labels=std_labels):             # <<<<<<<<<<<<<<
 *     """Calculate the number of amino acid residues in a polypeptide
 *     written in modX notation.
 */
  __Pyx_INCREF(__pyx_v_9pyteomics_7cparser_std_labels);
  __pyx_k__2 = __pyx_v_9pyteomics_7cparser_std_labels;
  __Pyx_GIVEREF(__pyx_v_9pyteomics_7cparser_std_labels);

  /* "pyteomics/cparser.pyx":106
 *     return bool(match_modX(label))
 * 
 * cpdef int length(object sequence,             # <<<<<<<<<<<<<<
 *                  int show_unmodified_termini=0, int split=0,
 *                  int allow_unknown_modifications=0, object labels=std_labels):
 */
  __Pyx_INCREF(__pyx_v_9pyteomics_7cparser_std_labels);
  __pyx_k__2 = __pyx_v_9pyteomics_7cparser_std_labels;
  __Pyx_GIVEREF(__pyx_v_9pyteomics_7cparser_std_labels);

  /* "pyteomics/cparser.pyx":175
 * 
 * 
 * cdef object _modX_split_findall = _modX_split.findall             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_9pyteomics_7cparser__modX_split, __pyx_n_s_findall); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 175, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_XGOTREF(__pyx_v_9pyteomics_7cparser__modX_split_findall);
  __Pyx_DECREF_SET(__pyx_v_9pyteomics_7cparser__modX_split_findall, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;

  /* "pyteomics/cparser.pyx":200
 * 
 * cpdef list parse(str sequence, bint show_unmodified_termini=0, bint split=0,
 *                  bint allow_unknown_modifications=0, object labels=std_labels):             # <<<<<<<<<<<<<<
 *     """Parse a sequence string written in modX notation into a list of
 *     labels or (if `split` argument is :py:const:`True`) into a list of
 */
  __Pyx_INCREF(__pyx_v_9pyteomics_7cparser_std_labels);
  __pyx_k__4 = __pyx_v_9pyteomics_7cparser_std_labels;
  __Pyx_GIVEREF(__pyx_v_9pyteomics_7cparser_std_labels);

  /* "pyteomics/cparser.pyx":199
 * 
 * 
 * cpdef list parse(str sequence, bint show_unmodified_termini=0, bint split=0,             # <<<<<<<<<<<<<<
 *                  bint allow_unknown_modifications=0, object labels=std_labels):
 *     """Parse a sequence string written in modX notation into a list of
 */
  __Pyx_INCREF(__pyx_v_9pyteomics_7cparser_std_labels);
  __pyx_k__4 = __pyx_v_9pyteomics_7cparser_std_labels;
  __Pyx_GIVEREF(__pyx_v_9pyteomics_7cparser_std_labels);

  /* "pyteomics/cparser.pyx":404
 * 
 * cpdef dict amino_acid_composition(object sequence, bint show_unmodified_termini=0, bint term_aa=0,
 *                                   bint allow_unknown_modifications=0, object labels=std_labels):             # <<<<<<<<<<<<<<
 *     """Calculate amino acid composition of a polypeptide.
 * 
 */
  __Pyx_INCREF(__pyx_v_9pyteomics_7cparser_std_labels);
  __pyx_k__6 = __pyx_v_9pyteomics_7cparser_std_labels;
  __Pyx_GIVEREF(__pyx_v_9pyteomics_7cparser_std_labels);

  /* "pyteomics/cparser.pyx":403
 * 
 * 
 * cpdef dict amino_acid_composition(object sequence, bint show_unmodified_termini=0, bint term_aa=0,             # <<<<<<<<<<<<<<
 *                                   bint allow_unknown_modifications=0, object labels=std_labels):
 *     """Calculate amino acid composition of a polypeptide.
 */
  __Pyx_INCREF(__pyx_v_9pyteomics_7cparser_std_labels);
  __pyx_k__6 = __pyx_v_9pyteomics_7cparser_std_labels;
  __Pyx_GIVEREF(__pyx_v_9pyteomics_7cparser_std_labels);

  /* "pyteomics/cparser.pyx":1
 * # cython: embedsignature=True             # <<<<<<<<<<<<<<
 * # cython: profile=True
 * #   Copyright 2016 Joshua Klein, Lev Levitsky
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_kp_u_length_line_106, __pyx_kp_u_Calculate_the_number_of_amino_ac) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_kp_u_parse_line_199, __pyx_kp_u_Parse_a_sequence_string_written) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_kp_u_amino_acid_composition_line_403, __pyx_kp_u_Calculate_amino_acid_composition) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_2) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_TraceReturn(Py_None, 0);

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init pyteomics.cparser", 0, __pyx_lineno, __pyx_filename);
    }
    Py_DECREF(__pyx_m); __pyx_m = 0;
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init pyteomics.cparser");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule((char *)modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, (char *)"RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* Profile */
#if CYTHON_PROFILE
static int __Pyx_TraceSetupAndCall(PyCodeObject** code,
                                   PyFrameObject** frame,
                                   PyThreadState* tstate,
                                   const char *funcname,
                                   const char *srcfile,
                                   int firstlineno) {
    PyObject *type, *value, *traceback;
    int retval;
    if (*frame == NULL || !CYTHON_PROFILE_REUSE_FRAME) {
        if (*code == NULL) {
            *code = __Pyx_createFrameCodeObject(funcname, srcfile, firstlineno);
            if (*code == NULL) return 0;
        }
        *frame = PyFrame_New(
            tstate,                          /*PyThreadState *tstate*/
            *code,                           /*PyCodeObject *code*/
            __pyx_d,                  /*PyObject *globals*/
            0                                /*PyObject *locals*/
        );
        if (*frame == NULL) return 0;
        if (CYTHON_TRACE && (*frame)->f_trace == NULL) {
            Py_INCREF(Py_None);
            (*frame)->f_trace = Py_None;
        }
#if PY_VERSION_HEX < 0x030400B1
    } else {
        (*frame)->f_tstate = tstate;
#endif
    }
      __Pyx_PyFrame_SetLineNumber(*frame, firstlineno);
    retval = 1;
    tstate->tracing++;
    tstate->use_tracing = 0;
    PyErr_Fetch(&type, &value, &traceback);
    #if CYTHON_TRACE
    if (tstate->c_tracefunc)
        retval = tstate->c_tracefunc(tstate->c_traceobj, *frame, PyTrace_CALL, NULL) == 0;
    if (retval && tstate->c_profilefunc)
    #endif
        retval = tstate->c_profilefunc(tstate->c_profileobj, *frame, PyTrace_CALL, NULL) == 0;
    tstate->use_tracing = (tstate->c_profilefunc ||
                           (CYTHON_TRACE && tstate->c_tracefunc));
    tstate->tracing--;
    if (retval) {
        PyErr_Restore(type, value, traceback);
        return tstate->use_tracing && retval;
    } else {
        Py_XDECREF(type);
        Py_XDECREF(value);
        Py_XDECREF(traceback);
        return -1;
    }
}
static PyCodeObject *__Pyx_createFrameCodeObject(const char *funcname, const char *srcfile, int firstlineno) {
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    PyCodeObject *py_code = 0;
    #if PY_MAJOR_VERSION < 3
    py_funcname = PyString_FromString(funcname);
    py_srcfile = PyString_FromString(srcfile);
    #else
    py_funcname = PyUnicode_FromString(funcname);
    py_srcfile = PyUnicode_FromString(srcfile);
    #endif
    if (!py_funcname | !py_srcfile) goto bad;
    py_code = PyCode_New(
        0,
        #if PY_MAJOR_VERSION >= 3
        0,
        #endif
        0,
        0,
        CO_OPTIMIZED | CO_NEWLOCALS,
        __pyx_empty_bytes,     /*PyObject *code,*/
        __pyx_empty_tuple,     /*PyObject *consts,*/
        __pyx_empty_tuple,     /*PyObject *names,*/
        __pyx_empty_tuple,     /*PyObject *varnames,*/
        __pyx_empty_tuple,     /*PyObject *freevars,*/
        __pyx_empty_tuple,     /*PyObject *cellvars,*/
        py_srcfile,       /*PyObject *filename,*/
        py_funcname,      /*PyObject *name,*/
        firstlineno,
        __pyx_empty_bytes      /*PyObject *lnotab*/
    );
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return py_code;
}
#endif

/* bytes_tailmatch */
static int __Pyx_PyBytes_SingleTailmatch(PyObject* self, PyObject* arg,
                                         Py_ssize_t start, Py_ssize_t end, int direction) {
    const char* self_ptr = PyBytes_AS_STRING(self);
    Py_ssize_t self_len = PyBytes_GET_SIZE(self);
    const char* sub_ptr;
    Py_ssize_t sub_len;
    int retval;
    Py_buffer view;
    view.obj = NULL;
    if ( PyBytes_Check(arg) ) {
        sub_ptr = PyBytes_AS_STRING(arg);
        sub_len = PyBytes_GET_SIZE(arg);
    }
#if PY_MAJOR_VERSION < 3
    else if ( PyUnicode_Check(arg) ) {
        return (int) PyUnicode_Tailmatch(self, arg, start, end, direction);
    }
#endif
    else {
        if (unlikely(PyObject_GetBuffer(self, &view, PyBUF_SIMPLE) == -1))
            return -1;
        sub_ptr = (const char*) view.buf;
        sub_len = view.len;
    }
    if (end > self_len)
        end = self_len;
    else if (end < 0)
        end += self_len;
    if (end < 0)
        end = 0;
    if (start < 0)
        start += self_len;
    if (start < 0)
        start = 0;
    if (direction > 0) {
        if (end-sub_len > start)
            start = end - sub_len;
    }
    if (start + sub_len <= end)
        retval = !memcmp(self_ptr+start, sub_ptr, (size_t)sub_len);
    else
        retval = 0;
    if (view.obj)
        PyBuffer_Release(&view);
    return retval;
}
static int __Pyx_PyBytes_TailmatchTuple(PyObject* self, PyObject* substrings,
                                        Py_ssize_t start, Py_ssize_t end, int direction) {
    Py_ssize_t i, count = PyTuple_GET_SIZE(substrings);
    for (i = 0; i < count; i++) {
        int result;
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        result = __Pyx_PyBytes_SingleTailmatch(self, PyTuple_GET_ITEM(substrings, i),
                                               start, end, direction);
#else
        PyObject* sub = PySequence_ITEM(substrings, i);
        if (unlikely(!sub)) return -1;
        result = __Pyx_PyBytes_SingleTailmatch(self, sub, start, end, direction);
        Py_DECREF(sub);
#endif
        if (result) {
            return result;
        }
    }
    return 0;
}
static int __Pyx_PyBytes_Tailmatch(PyObject* self, PyObject* substr,
                                   Py_ssize_t start, Py_ssize_t end, int direction) {
    if (unlikely(PyTuple_Check(substr))) {
        return __Pyx_PyBytes_TailmatchTuple(self, substr, start, end, direction);
    }
    return __Pyx_PyBytes_SingleTailmatch(self, substr, start, end, direction);
}

/* unicode_tailmatch */
static int __Pyx_PyUnicode_TailmatchTuple(PyObject* s, PyObject* substrings,
                                          Py_ssize_t start, Py_ssize_t end, int direction) {
    Py_ssize_t i, count = PyTuple_GET_SIZE(substrings);
    for (i = 0; i < count; i++) {
        Py_ssize_t result;
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        result = PyUnicode_Tailmatch(s, PyTuple_GET_ITEM(substrings, i),
                                     start, end, direction);
#else
        PyObject* sub = PySequence_ITEM(substrings, i);
        if (unlikely(!sub)) return -1;
        result = PyUnicode_Tailmatch(s, sub, start, end, direction);
        Py_DECREF(sub);
#endif
        if (result) {
            return (int) result;
        }
    }
    return 0;
}
static int __Pyx_PyUnicode_Tailmatch(PyObject* s, PyObject* substr,
                                     Py_ssize_t start, Py_ssize_t end, int direction) {
    if (unlikely(PyTuple_Check(substr))) {
        return __Pyx_PyUnicode_TailmatchTuple(s, substr, start, end, direction);
    }
    return (int) PyUnicode_Tailmatch(s, substr, start, end, direction);
}

/* str_tailmatch */
static CYTHON_INLINE int __Pyx_PyStr_Tailmatch(PyObject* self, PyObject* arg, Py_ssize_t start,
                                               Py_ssize_t end, int direction)
{
    if (PY_MAJOR_VERSION < 3)
        return __Pyx_PyBytes_Tailmatch(self, arg, start, end, direction);
    else
        return __Pyx_PyUnicode_Tailmatch(self, arg, start, end, direction);
}

/* PyErrFetchRestore */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* WriteUnraisableException */
static void __Pyx_WriteUnraisable(const char *name, CYTHON_UNUSED int clineno,
                                  CYTHON_UNUSED int lineno, CYTHON_UNUSED const char *filename,
                                  int full_traceback, CYTHON_UNUSED int nogil) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
    __Pyx_PyThreadState_declare
#ifdef WITH_THREAD
    PyGILState_STATE state;
    if (nogil)
        state = PyGILState_Ensure();
#ifdef _MSC_VER
    else state = (PyGILState_STATE)-1;
#endif
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    if (full_traceback) {
        Py_XINCREF(old_exc);
        Py_XINCREF(old_val);
        Py_XINCREF(old_tb);
        __Pyx_ErrRestore(old_exc, old_val, old_tb);
        PyErr_PrintEx(1);
    }
    #if PY_MAJOR_VERSION < 3
    ctx = PyString_FromString(name);
    #else
    ctx = PyUnicode_FromString(name);
    #endif
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
#ifdef WITH_THREAD
    if (nogil)
        PyGILState_Release(state);
#endif
}

/* GetItemInt */
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely((0 <= wrapped_i) & (wrapped_i < PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely((0 <= wrapped_i) & (wrapped_i < PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely((n >= 0) & (n < PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely((n >= 0) & (n < PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* PyIntBinop */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_AddObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, CYTHON_UNUSED int inplace) {
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long x;
        long a = PyInt_AS_LONG(op1);
            x = (long)((unsigned long)a + b);
            if (likely((x^a) >= 0 || (x^b) >= 0))
                return PyInt_FromLong(x);
            return PyLong_Type.tp_as_number->nb_add(op1, op2);
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        const long b = intval;
        long a, x;
#ifdef HAVE_LONG_LONG
        const PY_LONG_LONG llb = intval;
        PY_LONG_LONG lla, llx;
#endif
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op1);
        if (likely(__Pyx_sst_abs(size) <= 1)) {
            a = likely(size) ? digits[0] : 0;
            if (size == -1) a = -a;
        } else {
            switch (size) {
                case -2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                case 2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                case -3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                case 3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                case -4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                case 4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                default: return PyLong_Type.tp_as_number->nb_add(op1, op2);
            }
        }
                x = a + b;
            return PyLong_FromLong(x);
#ifdef HAVE_LONG_LONG
        long_long:
                llx = lla + llb;
            return PyLong_FromLongLong(llx);
#endif
        
        
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
            double result;
            PyFPE_START_PROTECT("add", return NULL)
            result = ((double)a) + (double)b;
            PyFPE_END_PROTECT(result)
            return PyFloat_FromDouble(result);
    }
    return (inplace ? PyNumber_InPlaceAdd : PyNumber_Add)(op1, op2);
}
#endif

/* PyCFunctionFastCall */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject * __Pyx_PyCFunction_FastCall(PyObject *func_obj, PyObject **args, Py_ssize_t nargs) {
    PyCFunctionObject *func = (PyCFunctionObject*)func_obj;
    PyCFunction meth = PyCFunction_GET_FUNCTION(func);
    PyObject *self = PyCFunction_GET_SELF(func);
    int flags = PyCFunction_GET_FLAGS(func);
    assert(PyCFunction_Check(func));
    assert(METH_FASTCALL == (flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS)));
    assert(nargs >= 0);
    assert(nargs == 0 || args != NULL);
    /* _PyCFunction_FastCallDict() must not be called with an exception set,
       because it may clear it (directly or indirectly) and so the
       caller loses its exception */
    assert(!PyErr_Occurred());
    if ((PY_VERSION_HEX < 0x030700A0) || unlikely(flags & METH_KEYWORDS)) {
        return (*((__Pyx_PyCFunctionFastWithKeywords)meth)) (self, args, nargs, NULL);
    } else {
        return (*((__Pyx_PyCFunctionFast)meth)) (self, args, nargs);
    }
}
#endif

/* PyFunctionFastCall */
#if CYTHON_FAST_PYCALL
#include "frameobject.h"
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = f->f_localsplus;
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, int nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (Py_EnterRecursiveCall((char*)" while calling a Python object")) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif
#endif

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = func->ob_type->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallMethO */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallOneArg */
#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, &arg, 1);
    }
#endif
    if (likely(PyCFunction_Check(func))) {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
#if CYTHON_FAST_PYCCALL
        } else if (PyCFunction_GET_FLAGS(func) & METH_FASTCALL) {
            return __Pyx_PyCFunction_FastCall(func, &arg, 1);
#endif
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

/* PyObjectCallNoArg */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, NULL, 0);
    }
#endif
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || __Pyx_TypeCheck(func, __pyx_CyFunctionType))) {
#else
    if (likely(PyCFunction_Check(func))) {
#endif
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

/* RaiseTooManyValuesToUnpack */
  static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

/* RaiseNeedMoreValuesToUnpack */
  static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

/* IterFinish */
  static CYTHON_INLINE int __Pyx_IterFinish(void) {
#if CYTHON_FAST_THREAD_STATE
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject* exc_type = tstate->curexc_type;
    if (unlikely(exc_type)) {
        if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) {
            PyObject *exc_value, *exc_tb;
            exc_value = tstate->curexc_value;
            exc_tb = tstate->curexc_traceback;
            tstate->curexc_type = 0;
            tstate->curexc_value = 0;
            tstate->curexc_traceback = 0;
            Py_DECREF(exc_type);
            Py_XDECREF(exc_value);
            Py_XDECREF(exc_tb);
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#else
    if (unlikely(PyErr_Occurred())) {
        if (likely(PyErr_ExceptionMatches(PyExc_StopIteration))) {
            PyErr_Clear();
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#endif
}

/* UnpackItemEndCheck */
  static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected) {
    if (unlikely(retval)) {
        Py_DECREF(retval);
        __Pyx_RaiseTooManyValuesError(expected);
        return -1;
    } else {
        return __Pyx_IterFinish();
    }
    return 0;
}

/* GetModuleGlobalName */
  static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name) {
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
    result = PyDict_GetItem(__pyx_d, name);
    if (likely(result)) {
        Py_INCREF(result);
    } else {
#else
    result = PyObject_GetItem(__pyx_d, name);
    if (!result) {
        PyErr_Clear();
#endif
        result = __Pyx_GetBuiltinName(name);
    }
    return result;
}

/* RaiseException */
    #if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    __Pyx_PyThreadState_declare
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* RaiseDoubleKeywords */
    static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
    static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_CheckExact(key)) || likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (PyUnicode_GET_SIZE(**name) != PyUnicode_GET_SIZE(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (PyUnicode_GET_SIZE(**argname) != PyUnicode_GET_SIZE(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

/* RaiseArgTupleInvalid */
    static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* ArgTypeTest */
    static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact)
{
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    else if (exact) {
        #if PY_MAJOR_VERSION == 2
        if ((type == &PyBaseString_Type) && likely(__Pyx_PyBaseString_CheckExact(obj))) return 1;
        #endif
    }
    else {
        if (likely(__Pyx_TypeCheck(obj, type))) return 1;
    }
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected %.200s, got %.200s)",
        name, type->tp_name, Py_TYPE(obj)->tp_name);
    return 0;
}

/* BytesEquals */
    static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    if (s1 == s2) {
        return (equals == Py_EQ);
    } else if (PyBytes_CheckExact(s1) & PyBytes_CheckExact(s2)) {
        const char *ps1, *ps2;
        Py_ssize_t length = PyBytes_GET_SIZE(s1);
        if (length != PyBytes_GET_SIZE(s2))
            return (equals == Py_NE);
        ps1 = PyBytes_AS_STRING(s1);
        ps2 = PyBytes_AS_STRING(s2);
        if (ps1[0] != ps2[0]) {
            return (equals == Py_NE);
        } else if (length == 1) {
            return (equals == Py_EQ);
        } else {
            int result;
#if CYTHON_USE_UNICODE_INTERNALS
            Py_hash_t hash1, hash2;
            hash1 = ((PyBytesObject*)s1)->ob_shash;
            hash2 = ((PyBytesObject*)s2)->ob_shash;
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                return (equals == Py_NE);
            }
#endif
            result = memcmp(ps1, ps2, (size_t)length);
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & PyBytes_CheckExact(s2)) {
        return (equals == Py_NE);
    } else if ((s2 == Py_None) & PyBytes_CheckExact(s1)) {
        return (equals == Py_NE);
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
#endif
}

/* UnicodeEquals */
    static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
#if PY_MAJOR_VERSION < 3
    PyObject* owned_ref = NULL;
#endif
    int s1_is_unicode, s2_is_unicode;
    if (s1 == s2) {
        goto return_eq;
    }
    s1_is_unicode = PyUnicode_CheckExact(s1);
    s2_is_unicode = PyUnicode_CheckExact(s2);
#if PY_MAJOR_VERSION < 3
    if ((s1_is_unicode & (!s2_is_unicode)) && PyString_CheckExact(s2)) {
        owned_ref = PyUnicode_FromObject(s2);
        if (unlikely(!owned_ref))
            return -1;
        s2 = owned_ref;
        s2_is_unicode = 1;
    } else if ((s2_is_unicode & (!s1_is_unicode)) && PyString_CheckExact(s1)) {
        owned_ref = PyUnicode_FromObject(s1);
        if (unlikely(!owned_ref))
            return -1;
        s1 = owned_ref;
        s1_is_unicode = 1;
    } else if (((!s2_is_unicode) & (!s1_is_unicode))) {
        return __Pyx_PyBytes_Equals(s1, s2, equals);
    }
#endif
    if (s1_is_unicode & s2_is_unicode) {
        Py_ssize_t length;
        int kind;
        void *data1, *data2;
        if (unlikely(__Pyx_PyUnicode_READY(s1) < 0) || unlikely(__Pyx_PyUnicode_READY(s2) < 0))
            return -1;
        length = __Pyx_PyUnicode_GET_LENGTH(s1);
        if (length != __Pyx_PyUnicode_GET_LENGTH(s2)) {
            goto return_ne;
        }
#if CYTHON_USE_UNICODE_INTERNALS
        {
            Py_hash_t hash1, hash2;
        #if CYTHON_PEP393_ENABLED
            hash1 = ((PyASCIIObject*)s1)->hash;
            hash2 = ((PyASCIIObject*)s2)->hash;
        #else
            hash1 = ((PyUnicodeObject*)s1)->hash;
            hash2 = ((PyUnicodeObject*)s2)->hash;
        #endif
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                goto return_ne;
            }
        }
#endif
        kind = __Pyx_PyUnicode_KIND(s1);
        if (kind != __Pyx_PyUnicode_KIND(s2)) {
            goto return_ne;
        }
        data1 = __Pyx_PyUnicode_DATA(s1);
        data2 = __Pyx_PyUnicode_DATA(s2);
        if (__Pyx_PyUnicode_READ(kind, data1, 0) != __Pyx_PyUnicode_READ(kind, data2, 0)) {
            goto return_ne;
        } else if (length == 1) {
            goto return_eq;
        } else {
            int result = memcmp(data1, data2, (size_t)(length * kind));
            #if PY_MAJOR_VERSION < 3
            Py_XDECREF(owned_ref);
            #endif
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & s2_is_unicode) {
        goto return_ne;
    } else if ((s2 == Py_None) & s1_is_unicode) {
        goto return_ne;
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
return_eq:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_EQ);
return_ne:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_NE);
#endif
}

/* StringJoin */
    #if !CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyBytes_Join(PyObject* sep, PyObject* values) {
    return PyObject_CallMethodObjArgs(sep, __pyx_n_s_join, values, NULL);
}
#endif

/* PyObjectCallMethod1 */
    static PyObject* __Pyx__PyObject_CallMethod1(PyObject* method, PyObject* arg) {
    PyObject *result = NULL;
#if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(method))) {
        PyObject *self = PyMethod_GET_SELF(method);
        if (likely(self)) {
            PyObject *args;
            PyObject *function = PyMethod_GET_FUNCTION(method);
            #if CYTHON_FAST_PYCALL
            if (PyFunction_Check(function)) {
                PyObject *args[2] = {self, arg};
                result = __Pyx_PyFunction_FastCall(function, args, 2);
                goto done;
            }
            #endif
            #if CYTHON_FAST_PYCCALL
            if (__Pyx_PyFastCFunction_Check(function)) {
                PyObject *args[2] = {self, arg};
                result = __Pyx_PyCFunction_FastCall(function, args, 2);
                goto done;
            }
            #endif
            args = PyTuple_New(2);
            if (unlikely(!args)) goto done;
            Py_INCREF(self);
            PyTuple_SET_ITEM(args, 0, self);
            Py_INCREF(arg);
            PyTuple_SET_ITEM(args, 1, arg);
            Py_INCREF(function);
            result = __Pyx_PyObject_Call(function, args, NULL);
            Py_DECREF(args);
            Py_DECREF(function);
            return result;
        }
    }
#endif
    result = __Pyx_PyObject_CallOneArg(method, arg);
    goto done;
done:
    return result;
}
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg) {
    PyObject *method, *result = NULL;
    method = __Pyx_PyObject_GetAttrStr(obj, method_name);
    if (unlikely(!method)) goto done;
    result = __Pyx__PyObject_CallMethod1(method, arg);
done:
    Py_XDECREF(method);
    return result;
}

/* pop_index */
    static PyObject* __Pyx__PyObject_PopNewIndex(PyObject* L, PyObject* py_ix) {
    PyObject *r;
    if (unlikely(!py_ix)) return NULL;
    r = __Pyx__PyObject_PopIndex(L, py_ix);
    Py_DECREF(py_ix);
    return r;
}
static PyObject* __Pyx__PyObject_PopIndex(PyObject* L, PyObject* py_ix) {
    return __Pyx_PyObject_CallMethod1(L, __pyx_n_s_pop, py_ix);
}
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static PyObject* __Pyx__PyList_PopIndex(PyObject* L, PyObject* py_ix, Py_ssize_t ix) {
    Py_ssize_t size = PyList_GET_SIZE(L);
    if (likely(size > (((PyListObject*)L)->allocated >> 1))) {
        Py_ssize_t cix = ix;
        if (cix < 0) {
            cix += size;
        }
        if (likely(0 <= cix && cix < size)) {
            PyObject* v = PyList_GET_ITEM(L, cix);
            Py_SIZE(L) -= 1;
            size -= 1;
            memmove(&PyList_GET_ITEM(L, cix), &PyList_GET_ITEM(L, cix+1), (size_t)(size-cix)*sizeof(PyObject*));
            return v;
        }
    }
    if (py_ix == Py_None) {
        return __Pyx__PyObject_PopNewIndex(L, PyInt_FromSsize_t(ix));
    } else {
        return __Pyx__PyObject_PopIndex(L, py_ix);
    }
}
#endif

/* Import */
    static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if (strchr(__Pyx_MODULE_NAME, '.')) {
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* ImportFrom */
    static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Format(PyExc_ImportError,
        #if PY_MAJOR_VERSION < 3
            "cannot import name %.230s", PyString_AS_STRING(name));
        #else
            "cannot import name %S", name);
        #endif
    }
    return value;
}

/* CLineInTraceback */
    static int __Pyx_CLineForTraceback(int c_line) {
#ifdef CYTHON_CLINE_IN_TRACEBACK
    return ((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0;
#else
    PyObject *use_cline;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
      use_cline = PyDict_GetItem(*cython_runtime_dict, __pyx_n_s_cline_in_traceback);
    } else
#endif
    {
      PyObject *ptype, *pvalue, *ptraceback;
      PyObject *use_cline_obj;
      PyErr_Fetch(&ptype, &pvalue, &ptraceback);
      use_cline_obj = __Pyx_PyObject_GetAttrStr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        use_cline = NULL;
      }
      PyErr_Restore(ptype, pvalue, ptraceback);
    }
    if (!use_cline) {
        c_line = 0;
        PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (PyObject_Not(use_cline) != 0) {
        c_line = 0;
    }
    return c_line;
#endif
}

/* CodeObjectCache */
    static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, (size_t)new_max*sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
    #include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        __Pyx_PyThreadState_Current, /*PyThreadState *tstate,*/
        py_code,                     /*PyCodeObject *code,*/
        __pyx_d,              /*PyObject *globals,*/
        0                            /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

/* CIntFromPyVerify */
    #define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntToPy */
    static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
    static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
    static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* CIntFromPy */
    static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* FastTypeChecks */
    #if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = a->tp_base;
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    int res = exc_type1 ? __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type1) : 0;
    if (!res) {
        res = __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
    return res;
}
#endif
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* CheckBinaryVersion */
    static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* FunctionExport */
    static int __Pyx_ExportFunction(const char *name, void (*f)(void), const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    union {
        void (*fp)(void);
        void *p;
    } tmp;
    d = PyObject_GetAttrString(__pyx_m, (char *)"__pyx_capi__");
    if (!d) {
        PyErr_Clear();
        d = PyDict_New();
        if (!d)
            goto bad;
        Py_INCREF(d);
        if (PyModule_AddObject(__pyx_m, (char *)"__pyx_capi__", d) < 0)
            goto bad;
    }
    tmp.fp = f;
#if PY_VERSION_HEX >= 0x02070000
    cobj = PyCapsule_New(tmp.p, sig, 0);
#else
    cobj = PyCObject_FromVoidPtrAndDesc(tmp.p, (void *)sig, 0);
#endif
    if (!cobj)
        goto bad;
    if (PyDict_SetItemString(d, name, cobj) < 0)
        goto bad;
    Py_DECREF(cobj);
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(cobj);
    Py_XDECREF(d);
    return -1;
}

/* ModuleImport */
    #ifndef __PYX_HAVE_RT_ImportModule
#define __PYX_HAVE_RT_ImportModule
static PyObject *__Pyx_ImportModule(const char *name) {
    PyObject *py_name = 0;
    PyObject *py_module = 0;
    py_name = __Pyx_PyIdentifier_FromString(name);
    if (!py_name)
        goto bad;
    py_module = PyImport_Import(py_name);
    Py_DECREF(py_name);
    return py_module;
bad:
    Py_XDECREF(py_name);
    return 0;
}
#endif

/* TypeImport */
    #ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name,
    size_t size, int strict)
{
    PyObject *py_module = 0;
    PyObject *result = 0;
    PyObject *py_name = 0;
    char warning[200];
    Py_ssize_t basicsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
#endif
    py_module = __Pyx_ImportModule(module_name);
    if (!py_module)
        goto bad;
    py_name = __Pyx_PyIdentifier_FromString(class_name);
    if (!py_name)
        goto bad;
    result = PyObject_GetAttr(py_module, py_name);
    Py_DECREF(py_name);
    py_name = 0;
    Py_DECREF(py_module);
    py_module = 0;
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if (!strict && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility. Expected %zd, got %zd",
            module_name, class_name, basicsize, size);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    else if ((size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s has the wrong size, try recompiling. Expected %zd, got %zd",
            module_name, class_name, basicsize, size);
        goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(py_module);
    Py_XDECREF(result);
    return NULL;
}
#endif

/* InitStrings */
    static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            PyErr_Clear();
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type %.200s).  "
                "The ability to return an instance of a strict subclass of int "
                "is deprecated, and may be removed in a future version of Python.",
                Py_TYPE(result)->tp_name)) {
            Py_DECREF(result);
            return NULL;
        }
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type %.200s)",
                 type_name, type_name, Py_TYPE(result)->tp_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(x);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
